function [first,X,Y,Z,v,probOut,user,popup_handle] = e05jb_contours(first,prob,bl,bu,X,Y,Z,v,xglobs,legendx,legendy,xres,yres,user,create_fig,val,popup_handle)
global prob_tmp
prob_tmp = prob;

n = length(bl);

if (n ~= 2)
    return
end

if (first == 0)
    xsp = (bu(1) - bl(1))/xres;
    ysp = (bu(2) - bl(2))/yres;
    [X, Y] = meshgrid(bl(1):xsp:bu(1), bl(2):ysp:legendy(1));
    Z = zeros(size(X));
    loc_user = struct('fcalls', 0, ...
        'oldx', zeros(n, 1), ...
        'prob', prob, ...
        'plot', 0);

    for i = 1:size(X, 1)

        for j = 1:size(X, 2)
            Z(i,j) = e05jb_objective(2, [X(i, j); Y(i, j)], 0, loc_user);
        end

    end

end

scrsz = get(0, 'ScreenSize');

if (first == 0)
    if create_fig
        figure('Name', ['Surface plot of ' prob], 'NumberTitle', 'off', ...
            'Position', [scrsz(1) + 100, scrsz(2) + 20, scrsz(3)*0.8, ...
                         scrsz(4)*0.8]);
        % Create a button and popup menu and wait for them to be used before continuing
        uicontrol('Position', [10 20 140 40], ...
            'String', 'Next', ...
            'Callback', @e05jb_button_handler);
        popup_handle = uicontrol('Style', 'popup', 'String', ...
            'Ackley|Beale|camel|Easom|GoldsteinPrice|Langerman|Levy|Michalewicz|peaks|perm(2,0.5)|perm0(2,10)|Rastrigin|Schwefel|Zakharov', ...
            'Position', [10 60 140 40], 'TooltipString', 'Chooses the prob', ...
            'Value', val, 'Callback', @e05jb_chooseprob);
    end
    surfc(X, Y, Z);
    uiwait(gcf);
    window_title = ['Global Minimization by Multilevel Coordinate Search: ', ...
        prob, ' (I)'];
    fig_title = ['Global Minimization: ', prob, ' (I)'];
    first = 1;
else
    window_title = ['Global Minimization by Multilevel Coordinate Search: ', ...
        prob, ' (II)'];
    fig_title = ['Global Minimization: ', prob, ' (II)'];
end
probOut = prob_tmp;
if strcmp(probOut,prob)
    if isempty(user.fig2)
        set(popup_handle, 'Enable', 'Off');
    end
    fig2 = figure('Name', window_title, 'NumberTitle', 'off', ...
        'Position', [scrsz(1) + 100, scrsz(2) + 20, scrsz(3)*0.8, scrsz(4)*0.8]);
    user.fig2 = fig2;
    if (strcmp(prob, 'camel'))
        v = [-0.9, -0.58, -0.266, 0.05, 0.366, 0.68, 1, 2.2, 2.35, 3, 5, 10, 20, 50];
        contour(X, Y, Z, v);
    elseif (strcmp(prob, 'Beale') || strcmp(prob, 'GoldsteinPrice'))
        v = exp(1:10);
        contour(X, Y, Z, v);
    else
        contour(X, Y, Z);
    end
    user.axes = gca;
    colorbar;
    title(fig_title, ...
        'FontSize', 14);
    hold on
    
    % Mark global minima
    plot(xglobs(1, :).', xglobs(2, :).', ...
        'g*', ...
        'Linewidth', 2, ...
        'MarkerSize', 14);

    % Put a legend on the graph
    fill(legendx, legendy, 'w');
    plot(legendx(1) + 2*(bu(1) - bl(1))/100, legendy(1) - 4*(bu(2) - bl(2))/100, ...
        'g*', ...
        'Linewidth', 2, ...
        'MarkerSize', 12);
    text(legendx(1) + 4*(bu(1) - bl(1))/100, legendy(1) - 4*(bu(2) - bl(2))/100, ...
        'Position of glob. min.', ...
        'FontSize', 14);
    plot(legendx(1) + 2*(bu(1) - bl(1))/100, legendy(1) - 8*(bu(2) - bl(2))/100, ...
        'rs', ...
        'Linewidth', 2, ...
        'MarkerSize', 10);
    text(legendx(1) + 4*(bu(1) - bl(1))/100, legendy(1) - 8*(bu(2) - bl(2))/100, ...
        'Current search box', ...
        'FontSize', 14);
    plot((bl(1) + bu(1))/2 - 2*(bu(1) - bl(1))/100, legendy(1) - 8*(bu(2) - bl(2))/100, ...
        'bs', ...
        'Linewidth', 2, ...
        'MarkerSize', 10);
    text((bl(1) + bu(1))/2, legendy(1) - 8*(bu(2) - bl(2))/100, ...
        'Prev. search boxes', ...
        'FontSize', 14);

    % Set plot_evals to 1 to plot evaluation points
    plot_evals = 1;

    if (plot_evals == 1)
        plot((bl(1) + bu(1))/2 - 2*(bu(1) - bl(1))/100, legendy(1) - 4*(bu(2) - bl(2))/100, ...
            'bx', ...
            'Linewidth', 2, ...
            'MarkerSize', 6);
        text((bl(1) + bu(1))/2, legendy(1) - 4*(bu(2) - bl(2))/100, ...
            'Objective evaluation points', ...
            'FontSize', 14);
    end

    xlabel('x_1'); ylabel('x_2');
end
