function e05jb_demo(pause_time,prob,target)
if (nargin < 3)
    target = 0;

    if (nargin < 2)
        prob = 'camel';

        if (nargin < 1)
            pause_time = 0.1;
        end

    end

end

% Problem data
xres = 100;
yres = 100;

create_fig = 1;
popup_handle = [];

% If prob is changed, then start from the beginning
prob_change=1;
while prob_change
    prob_change=0;

    if (strcmp(prob, 'Ackley'))
        bl = [-15; -15];
        bu = [30; 30];
        fglob = 0;
        xglob = [0; 0];
        val = 1;
    elseif (strcmp(prob, 'Beale'))
        bl = [-4.5; -4.5];
        bu = -bl;
        fglob = 0;
        xglob = [3; 0.5];
        val = 2;
    elseif (strcmp(prob, 'camel'))
        bl = [-3; -2];
        bu = -bl;
        fglob = -1.0316284535;
        nglob = 2;
        xglob = zeros(length(bl), nglob);
        xglob(:,1) = [0.08984201; -0.71265640];
        xglob(:,2) = -xglob(:,1);
        val = 3;
    elseif (strcmp(prob, 'Easom'))
        bl = [-100; -100];
        bu = -bl;
        fglob = -1;
        xglob = [x01aa; x01aa];
        val = 4;
    elseif (strcmp(prob, 'GoldsteinPrice'))
        bl = [-2; -2];
        bu = -bl;
        fglob = 3;
        xglob = [0; -1];
        xres = 500;
        yres = 500;
        val = 5;
    elseif (strcmp(prob, 'Langerman'))
        bl = [0; 0];
        bu = [10; 10];
        fglob = -1.4;
        xglob = [1; 1];
        val = 6;
    elseif (strcmp(prob, 'Levy'))
        bl = [-10; -10];
        bu = -bl;
        fglob = 0;
        xglob = [1; 1];
        val = 7;
    elseif (strcmp(prob, 'Michalewicz'))
        bl = [0; 0];
        bu = [x01aa; x01aa];
        fglob = -1.8013; % Approx.
        xglob = [2.203; x01aa/2]; % Approx.
        val = 8;
    elseif (strcmp(prob, 'peaks'))
        bl = [-3; -3];
        bu = -bl;
        fglob = -6.55; % Approx.
        xglob = [0.23; -1.63]; % Approx.
        val = 9;
    elseif (strcmp(prob, 'perm(2,0.5)'))
        bl = [-2; -2];
        bu = -bl;
        fglob = 0;
        xglob = (1:2).';
        val = 10;
    elseif (strcmp(prob, 'perm0(2,10)'))
        bl = [-1; -1];
        bu = -bl;
        fglob = 0;
        xglob = 1./(1:2).';
        val = 11;
    elseif (strcmp(prob, 'Rastrigin'))
        bl = [-5; -5];
        bu = -bl;
        fglob = 0;
        xglob = [2; 2];
        val = 12;
    elseif (strcmp(prob, 'Schwefel'))
        bl = [-500; -500];
        bu = -bl;
        fglob = 1;
        xglob = [420.9687; 420.9687];
        val = 13;
    elseif (strcmp(prob, 'Zakharov'))
        bl = [-5; -5];
        bu = [10; 10];
        fglob = 0;
        xglob = [0; 0];
        val = 14;
    else
        error(['Problem ' prob ' not coded.'])
    end

    % Initialize e05jb
    n = length(bl);
    [comm, ifail] = e05ja();

    if (ifail == 0)

        % The struct 'user' will be used to communicate previous points back
        % into the objective function, for plotting purposes. Plotting occurs
        % if user.plot ~= 0
        % Set user.savemovie = 1 to create a bunch of PNG files with a sequence
        % of movie frames.
        user = struct('fcalls', 0, ...
            'oldx', zeros(n, 1), ...
            'plot', 1, ...
            'boxl_old', zeros(n,1), ...
            'boxu_old', zeros(n,1), ...
            'mypause', pause_time, ...
            'savemovie', 0, ...
            'framenumber', 0, ...
            'legendx', zeros(2*n, 1), ...
            'prob', prob, ...
            'bl', bl, ...
            'bu', bu, ...
            'fhandle', [], ...
            'thandle', [], ...
            'fig2', []);

        % Vanilla call.
        % We may need to increase some defaults for the more difficult problems

        if (strcmp(prob, 'Ackley') || ...
                strcmp(prob, 'Langerman') || ...
                strcmp(prob, 'perm(2,0.5)') || ...
                strcmp(prob, 'Rastrigin') || ...
                strcmp(prob, 'Schwefel'))
            comm = e05jd('Function Evaluations Limit = 10000000', comm);
        end

        if (strcmp(prob, 'Schwefel'))
            comm = e05jf('Static Limit', nag_int(10*n), comm);
        end

        if (target ~= 0)
            comm = e05jg('Target Objective Value', fglob, comm);
        end

        if (user.plot ~= 0)
            legendx = [bl(1) + 9*(bu(1) - bl(1))/100, bu(1) - 9*(bu(1) - bl(1))/100, ...
                bu(1) - 9*(bu(1) - bl(1))/100, bl(1) + 9*(bu(1) - bl(1))/100];
            user.legendx = legendx;
            legendy = [bu(2) + 22*(bu(2) - bl(2))/100, bu(2) + 22*(bu(2) - bl(2))/100, ...
                bu(2) + 8*(bu(2) - bl(2))/100, bu(2) + 8*(bu(2) - bl(2))/100];
            user.legendy = legendy;
            first = 0;
            [first, X, Y, Z, v, probOut, user, popup_handle] = ...
                e05jb_contours(first, prob, bl, bu, 0, 0, 0, 0, ...
                xglob, legendx, legendy, xres, yres, user, create_fig, ...
                val, popup_handle);
            if ~strcmp(probOut,prob)
                prob_change = 1;
                prob = probOut;
                set(gcf,'Name',['Surface plot of ' prob]);
                delete(gca);
                create_fig = 0;
            end
        end

        if prob_change == 0
            disp(sprintf('\n'));
            disp('Solve with no options or init.-list data');
            ibound = nag_int(0);             % All bounds will be given;
            iinit = nag_int(0);              % Default initialization method;
            list = zeros(n,3);             % Only need to _declare_ the initialization-list data:
            numpts = zeros(n, 1, nag_int_name); % these will be _set_ internally.
            initpt = zeros(n, 1, nag_int_name);

            if (user.plot ~= 0)
                % Create a button and wait for it to be pressed before continuing
                hbutton = uicontrol('Position', [10 20 140 40], ...
                    'String', 'Start', ...
                    'Callback', @e05jb_button_handler);
                uiwait(user.fig2); % Wait until "Start" button is pressed.
            else
                input('Press return to start...');
            end

            [bl, bu, listOut, numptsOut, initptOut, xbest, obj, comm, userOut, ifail] = ...
                e05jb('e05jb_objective', ibound, bl, bu, list, numpts, initpt, 'e05jb_monitor', comm, 'user', user);

            disp(['e05jb (vanilla) exited with ifail = ' num2str(ifail)]);

            if (ifail == 0)
                disp('xbest:');
                disp(xbest);
                disp(['obj = ' num2str(obj)]);
            end

            if (user.plot ~= 0)
                complete_plot(xbest, obj, userOut.fcalls, user.axes, legendx, legendy, bl, bu);
            end

            if (user.savemovie == 1)
                FF = getframe(gcf);
                FX = frame2im(FF);
                filename = sprintf('frame%d.png', user.framenumber);
                user.framenumber = user.framenumber + 1;
                imwrite(FX,filename,'png');
            end

            if (user.plot ~= 0)
                set(hbutton, ...
                    'String', 'Next');
                uiwait(user.fig2); % Wait until "Next" button is pressed.
            else
                input('Press return to continue...');
            end

            if (user.plot ~= 0)
                [first, X, Y, Z, v, probOut, user, popup_handle] = ...
                    e05jb_contours(first, prob, bl, bu, X, Y, Z, v, ...
                    xglob, legendx, legendy, xres, yres, user, ...
                    create_fig, val, popup_handle);
            end

            % Set some options. No need to reinitialize: n hasn't changed, and we
            % didn't set any options above.
            disp(sprintf('\n'));
            disp('Solve with options and init.-list data');

            if (user.plot ~= 0)
                % Create a button and wait for it to be pressed before continuing
                hbutton2 = uicontrol('Position', [10 20 140 40], ...
                    'String', 'Start', ...
                    'Callback', @e05jb_button_handler);
                uiwait(user.fig2); % Wait until "Start" button is pressed.
            end

            % Echo the setting of opt. params.
            comm = e05jd('List', comm);

            comm = e05jd('Function Evaluations Limit = 100000', comm);
            comm = e05jf('Static Limit', nag_int(3*n), comm);

            % Increase infbnd by factor of 10.
            [infbnd] = e05jl('Infinite Bound Size', comm);
            comm = e05jg('Infinite Bound Size', 10*infbnd, comm);

            comm = e05je('Local Searches', 'on', comm);

            % Set the initialization-list data.
            if (strcmp(prob, 'camel') || ...
                    strcmp(prob, 'GoldsteinPrice') || ...
                    strcmp(prob, 'peaks'))
                iinit = nag_int(3);                      % We're providing the data this time:
                list = zeros(n, 3);
                list(:, 1) = bl; list(:, 3) = bu;

                if (strcmp(prob, 'camel'))
                    list(:, 2) = [-1; -0.25];
                elseif (strcmp(prob, 'GoldsteinPrice'))
                    list(:, 2) = [-1; -1];
                elseif (strcmp(prob, 'peaks'))
                    list(:, 2) = [-1; 0];
                end

                numpts = zeros(n, 1, nag_int_name);
                initpt = zeros(n, 1, nag_int_name); 
                for i=1:n
                  numpts(i) = 3; % Three splitting points for each dimension;
                  initpt(i) = 2; % Second point in each row to be the 'initial' point.
                end
            else
                iinit = nag_int(2);              % Linesearch initialization method;
                list = zeros(n, 192);          % Only need to _declare_ the initialization-list data:
                numpts = zeros(n, 1, nag_int_name); % these will be _set_ internally.
                initpt = zeros(n, 1, nag_int_name);
            end

            [bl, bu, listOut, numptsOut, initptOut, xbest, obj, comm, userOut, ifail] = ...
                e05jb('e05jb_objective', ibound, bl, bu, list, numpts, initpt, 'e05jb_monitor', comm, 'iinit',  iinit, 'user', user);

            disp(['e05jb (options) exited with ifail = ' num2str(ifail)]);

            if (ifail == 0)
                disp('xbest:');
                disp(xbest);
                disp(['obj = ' num2str(obj)]);
            end

            if (user.plot ~= 0)
                complete_plot(xbest, obj, userOut.fcalls, user.axes, legendx, legendy, bl, bu);
            end

            if (user.savemovie == 1)
                FF = getframe(gcf);
                FX = frame2im(FF);
                filename = sprintf('frame%d.png', user.framenumber);
                imwrite(FX,filename,'png');
            end

            if (user.plot ~= 0)
                set(hbutton2, ...
                    'String', 'Close');
            end
        end
    end
end

function complete_plot(xbest,objbest,fcalls,axes,legendx,legendy,bl,bu)
plot(axes, ...
     xbest(1), xbest(2), ...
    'ro', ...
    'Linewidth', 2, ...
    'Markersize', 10);
plot(axes, ...
     legendx(1) + 2*(bu(1) - bl(1))/100, legendy(1) - 12*(bu(2) - bl(2))/100, ...
    'ro', ...
    'Linewidth', 2, ...
    'MarkerSize', 10);
t = sprintf('Final f val. = %7.4g at (%7.4f,%7.4f)', objbest, xbest(1), xbest(2));
text(legendx(1) + 4*(bu(1) - bl(1))/100, legendy(1) - 12*(bu(2) - bl(2))/100, t, ...
    'FontSize', 14, 'Parent', axes);
t = sprintf('Number of objective-function evaluations = %3d', fcalls);
fry = [legendy(3), legendy(4), ...
    legendy(3) - 6*(bu(2) - bl(2))/100, legendy(4) - 6*(bu(2) - bl(2))/100];
fill(legendx, fry, 'w', 'Parent', axes);
text(legendx(1) + 2*(bu(1) - bl(1))/100, legendy(3) - 3*(bu(2) - bl(2))/100, ...
    t, 'FontSize', 14, 'Parent', axes);
hold off
