function [f,user,inform] = e05jb_objective(n,x,nstate,user)
if (n==2)
    inform = nag_int(0);
else
    inform = nag_int(-1);
end

if (inform >= 0)

    % We're prepared to evaluate the objective at this point
    
    user.fcalls = user.fcalls + 1;
    
    if (nstate == 1)
        disp(sprintf('\n'));
        disp('(OBJFUN was just called for the first time)');
        user.oldx = x;
    else
        oldx = user.oldx;
        
        if (user.plot ~= 0)
            plot(user.axes, ... 
                 oldx(1), oldx(2), ...
                 'bx', ...
                 'MarkerSize', 6);
        end
        
    end
    
    user.oldx = x;
 
    if (user.plot ~= 0)
        plot(user.axes, ... 
             x(1), x(2), ...
             'rx', ...
             'MarkerSize', 6);
        t = sprintf('Number of objective-function evaluations = %3d', user.fcalls);
        bl = user.bl;
        bu = user.bu;
        fry = [user.legendy(3), user.legendy(4), ...
               user.legendy(3) - 6*(bu(2) - bl(2))/100, user.legendy(4) - 6*(bu(2) - bl(2))/100];
        if ~isempty(user.fhandle) 
            delete(user.fhandle);
        end
        user.fhandle = fill(user.legendx, fry, 'w', 'Parent', user.axes);
        if ~isempty(user.thandle) 
            delete(user.thandle);
        end
        user.thandle = text(user.legendx(1) + 2*(bu(1) - bl(1))/100, user.legendy(3) - 3*(bu(2) - bl(2))/100, ...
             t, 'FontSize', 14, 'Parent', user.axes);

        % Set user.savemovie = 1 in the main program to create a bunch of PNG
        % files with a sequence of movie frames.
        if (user.savemovie == 1)
            FF = getframe(gcf);
            FX = frame2im(FF);
            filename = sprintf('frame%d.png', user.framenumber);
            user.framenumber = user.framenumber + 1;
            imwrite(FX,filename,'png');
        end
  
        drawnow;
        pause(user.mypause);
    end
    
    if (strcmp(user.prob, 'Ackley'))
        f = 20 + exp(1) - 20*exp(-0.2*sqrt(sum(x(1:double(n)).^2)/double(n))) - ...
            exp(sum(cos(2*x01aa*x(1:double(n))))/double(n));
    elseif (strcmp(user.prob, 'Beale'))
        f = (1.5 - x(1)*(1 - x(2)))^2 + (2.25 - x(1)*(1 - x(2)^2))^2 + ...
            (2.625 - x(1)*(1 - x(2)^3))^2;
    elseif (strcmp(user.prob, 'camel'))
        f = (4 - 2.1*x(1)^2 + x(1)^4/3)*x(1)^2 + x(1)*x(2) + ...
            (-4 + 4*x(2)^2)*x(2)^2;
    elseif (strcmp(user.prob, 'Easom'))
        f = -cos(x(1))*cos(x(2))* ...
            exp(-(x(1) - pi)^2 - (x(2) - pi)^2);
    elseif (strcmp(user.prob, 'GoldsteinPrice'))
        f = (1 + ...
             (x(1) + x(2) + 1)^2*...
             (19 - 14*x(1) + 3*x(1)^2 - 14*x(2) + 6*x(1)*x(2) + 3*x(2)^2))*...
            (30 + ...
             (2*x(1) - 3*x(2))^2*...
             (18 - 32*x(1) + 12*x(1)^2 + 48*x(2) - 36*x(1)*x(2) + 27*x(2)^2));
    elseif (strcmp(user.prob, 'Langerman'))
        m = 5;
        f = 0;
        a = [3, 5, 2, 1, 7];
        b = [5, 2, 1, 4, 9];
        c = [1, 2, 5, 2, 3];
        
        for i = 1:m
            s = (x(1) - a(i))^2 + (x(2) - b(i))^2;
            f = f - c(i)*exp(-s/x01aa)*cos(s*x01aa);
        end

    elseif (strcmp(user.prob, 'Levy'))
        z = 1 + (x(1:double(n)) - 1)/4;
        f = sin(x01aa*z(1))^2 + ...
            sum((z(1:(double(n) - 1)) - 1).^2.*(1 + 10*(sin(x01aa*z(1:(double(n) - 1)) + 1)).^2)) + ...
            (z(n) - 1)^2*(1 + (sin(2*x01aa*z(n)))^2);
    elseif (strcmp(user.prob, 'Michalewicz'))
        m = 10;
        f = 0;

        for i = 1:double(n)
            f = f - sin(x(i))*(sin(i*x(i)^2/x01aa))^(2*m);
        end

    elseif (strcmp(user.prob, 'peaks'))
        f = peaks(x(1), x(2));
    elseif (strcmp(user.prob, 'perm(2,0.5)'))
        s_out = 0;
        
        for j = 1:double(n)
            s_in = 0;
            
            for i = 1:double(n)
                s_in = s_in + (double(i)^double(j) + 0.5)*((x(i)/double(i))^double(j) - 1);
            end
            
            s_out = s_out + s_in^2;
        end
        
        f = s_out;
    elseif (strcmp(user.prob, 'perm0(2,10)'))
        s_out = 0;
        
        for j = 1:double(n)
            s_in = 0;
            
            for i = 1:double(n)
                s_in = s_in + (double(i) + 10.0)*(x(i)^j - double(1)/i^j);
            end
            
            s_out = s_out + s_in^2;
        end
        
        f = s_out;
    elseif (strcmp(user.prob, 'Rastrigin'))
        f = 10*double(n) + sum((x(1:double(n)) - 2).^2 - 10*cos(2*x01aa*(x(1:double(n)) - 2)));
    elseif (strcmp(user.prob, 'Schwefel'))
        f = 418.9829*double(n) + sum(-x(1:double(n)).*sin(sqrt(abs(x(1:double(n))))));
    elseif (strcmp(user.prob, 'Zakharov'))
        s2 = 0;

        for i = 1:double(n)
            s2 = s2 + 0.5*double(i)*x(i);
        end
        
        f = sum(x(1:double(n)).^2) + s2^2 + s2^4;
    else
        error(['Problem ' user.prob ' not coded.']);
    end

end
