% Portfolio optimization demo. Modern Portfolio Theory (by Harry Markovitz)
%
% Written by Marcin Krzysztofik (Marcin.Krzysztofik@nag.co.uk). June 2010.
%
% It uses the following m-files:
%
%   frontier.m - calculates the efficient frontier for given assets;
%
%   get_extreme_returns.m - finds maximum/minimum attainable returns for
%   given assets. Uses NAG e04nc optimizer, set to solve 
%   a LP (linear programming) problem;
%
%   opt_data.m - holds the input data for optimization;
%
%   optimize.m - finds minimum risk portfolio for a given return, using 
%   NAG e04nc optimizer, set to solve a convex QP (quadratic programming)
%   problem.

function varargout = Optimization_demo(varargin)
% PLOT M-file for Plot.fig
%      PLOT, by itself, creates a new PLOT or raises the existing
%      singleton*.
%
%      H = PLOT returns the handle to a new PLOT or the handle to
%      the existing singleton*.
%
%      PLOT('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in PLOT.M with the given input arguments.
%
%      PLOT('Property','Value',...) creates a new PLOT or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before Plot_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to Plot_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help Plot

% Last Modified by GUIDE v2.5 10-Jun-2010 09:30:57

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @Optimization_demo_OpeningFcn, ...
                   'gui_OutputFcn',  @Optimization_demo_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before Plot is made visible.
function Optimization_demo_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to Plot (see VARARGIN)

% Choose default command line output for Plot
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes Plot wait for user response (see UIRESUME)
% uiwait(handles.figure1);

%initial constraints
l_c = -1; %short sell up to 100%
u_c = 1; %up to 100% can be invested in each asset

% the minimum depends on number of assets
set_slider1(handles);
%get the efficient frontier
[x, y, min_ret, max_ret, portf_ret] = frontier(l_c, u_c, 0);
%find the minimum risk portfolio
portf_ret = get_optimal_ret(x, y);
%get the structure of a portfolio
[portf_std, weights] = optimize(portf_ret, l_c, u_c);
%set the slider for the returns
set_slider3(portf_ret, min_ret, max_ret, handles);
%do the plots
plot_set(x, y, portf_ret, portf_std, weights, handles)

%set the size of the figure window
%screen_size = get(0, 'ScreenSize');
%screen_width = screen_size(3);
%screen_height = screen_size(4);

%fig_x = 0.1*screen_width;
%fig_y = 0.1*screen_height;
%fig_w = 0.8*screen_width;
%fig_h = 0.8*screen_height;

%set(handles.figure1, 'Position', [fig_x fig_y fig_w fig_h])


% --- Executes on slider1 (upper constraint) movement.
function slider1_Callback(hObject, eventdata, handles)
% hObject    handle to slider1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider
u_c = get(handles.slider1, 'Value');
l_c = get(handles.slider2, 'Value');
portf_ret = get(handles.slider3, 'Value');

set(handles.edit_up, 'Value', u_c);
set(handles.edit_up, 'String', ...
    strcat(num2str(round(u_c*100*100)/100), '%'));

%do the plotting
[x, y, min_ret, max_ret, portf_ret] = frontier(l_c, u_c, portf_ret);
[portf_std, weights] = optimize(portf_ret, l_c, u_c);
set_slider3(portf_ret, min_ret, max_ret, handles);
plot_set(x, y, portf_ret, portf_std, weights, handles)


% --- Executes on slider2 (lower constraint) movement.
function slider2_Callback(hObject, eventdata, handles)
% hObject    handle to slider2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider
l_c = get(handles.slider2, 'Value');
u_c = get(handles.slider1, 'Value');
portf_ret = get(handles.slider3, 'Value');

set(handles.edit_lo, 'Value', l_c);
set(handles.edit_lo, 'String', ...
    strcat(num2str(round(l_c*100*100)/100), '%'));

%do the plotting
[x, y, min_ret, max_ret, portf_ret] = frontier(l_c, u_c, portf_ret);
[portf_std, weights] = optimize(portf_ret, l_c, u_c);
set_slider3(portf_ret, min_ret, max_ret, handles);
plot_set(x, y, portf_ret, portf_std, weights, handles)


% --- Executes on checking/unchecking of sweep_up checkbox.
function sweep_up_Callback(hObject, eventdata, handles)
% hObject    handle to sweep_up (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of sweep_up

pause_interval = 0.01;

minval = get(handles.slider1, 'Min');
maxval = get(handles.slider1, 'Max');
step = (maxval - minval) / 95;

l_c = get(handles.slider2, 'Value');

%start the sweeping in a random direction
% Initialize the Mersenne Twister
[state, ifail] = g05kg(nag_int(3), nag_int(0));
%generate one number
[state, x, ifail] = g05tl(nag_int(1), nag_int(1), nag_int(2), state);
direction = x(1);

%this should be working for 5 seconds if activated via button
tic;

%if the checkbox is checked
while get(handles.sweep_up, 'Value') == 1.0
    set(handles.pushbutton2, 'String', 'Stop sweeping');
    u_c = get(handles.slider1, 'Value');
    portf_ret = get(handles.slider3, 'Value');
    if direction == 1 %move to the right
        %if there is still space to move to the right then move
        if u_c < maxval
            u_c = u_c + step;
            %maxval cannot be exceeded
            if u_c > maxval
                u_c = maxval;
            end
            set(handles.slider1, 'Value', u_c);
            set(handles.edit_up, 'Value', u_c);
            set(handles.edit_up, 'String', ...
                strcat(num2str(round(u_c*100*100)/100), '%'));
            
            %do the plotting
            [x, y, min_ret, max_ret, portf_ret] = ...
                frontier(l_c, u_c, portf_ret);
            [portf_std, weights] = optimize(portf_ret, l_c, u_c);
            set_slider3(portf_ret, min_ret, max_ret, handles);
            plot_set(x, y, portf_ret, portf_std, weights, handles)
            
            pause(pause_interval)
        %if the slider is at maxval we change the direction
        else
            direction = 2;
        end
    else %move to the left
        %if there is still space to move to the left then move
        if u_c > minval
            u_c = u_c - step;
            %minval cannot be exceeded
            if u_c < minval
                u_c = minval;
            end
            set(handles.slider1, 'Value', u_c);
            set(handles.edit_up, 'Value', u_c);
            set(handles.edit_up, 'String', ...
                strcat(num2str(round(u_c*100*100)/100), '%'));
                      
            %do the plotting
            [x, y, min_ret, max_ret, portf_ret] = ...
                frontier(l_c, u_c, portf_ret);
            [portf_std, weights] = optimize(portf_ret, l_c, u_c);
            set_slider3(portf_ret, min_ret, max_ret, handles);
            plot_set(x, y, portf_ret, portf_std, weights, handles)
            
            pause(pause_interval)
        %if the slider is at minval we change the direction
        else
            direction = 1;
        end
    end
    %if the sweeping was activated via button then it should be working
    %just for 5 seconds
    t_end = toc;
    if t_end >= 5 && get(handles.pushbutton2, 'Value') == 1.0
        set(handles.sweep_up, 'value', 0.0)
    end
    if get(handles.pushbutton2, 'Value') == 0.0
        set(handles.pushbutton2, 'String', 'Sweep at random!');
    end    
end


% --- Executes on checking/unchecking of sweep_lo checkbox.
function sweep_lo_Callback(hObject, eventdata, handles)
% hObject    handle to sweep_lo (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of sweep_lo

pause_interval = 0.01;

minval = get(handles.slider2, 'Min');
maxval = get(handles.slider2, 'Max');
step = (maxval - minval) / 100;

u_c = get(handles.slider1, 'Value');

%start the sweeping in a random direction
% Initialize the Mersenne Twister
[state, ifail] = g05kg(nag_int(3), nag_int(0));
%generate one number
[state, x, ifail] = g05tl(nag_int(1), nag_int(1), nag_int(2), state);
direction = x(1);

%this should be working for 5 seconds if activated via button
tic;

%if the checkbox is checked
while get(handles.sweep_lo, 'Value') == 1.0
    set(handles.pushbutton2, 'String', 'Stop sweeping');
    l_c = get(handles.slider2, 'Value');
    portf_ret = get(handles.slider3, 'Value');
    if direction == 1 %move to the right
        %if there is still space to move to the right then move
        if l_c < maxval
            l_c = l_c + step;
            %maxval cannot be exceeded
            if l_c > maxval
                l_c = maxval;
            end
            set(handles.slider2, 'Value', l_c);
            set(handles.edit_lo, 'Value', l_c);
            set(handles.edit_lo, 'String', ...
                strcat(num2str(round(l_c*100*100)/100), '%'));
            
            %do the plotting
            [x, y, min_ret, max_ret, portf_ret] = ...
                frontier(l_c, u_c, portf_ret);
            [portf_std, weights] = optimize(portf_ret, l_c, u_c);
            set_slider3(portf_ret, min_ret, max_ret, handles);
            plot_set(x, y, portf_ret, portf_std, weights, handles)
            
            pause(pause_interval)
        %if the slider is at maxval we change the direction
        else
            direction = 2;
        end
    else %move to the left
        %if there is still space to move to the left then move
        if l_c > minval
            l_c = l_c - step;
            %minval cannot be exceeded
            if l_c < minval
                l_c = minval;
            end
            set(handles.slider2, 'Value', l_c);
            set(handles.edit_lo, 'Value', l_c);
            set(handles.edit_lo, 'String', ...
                strcat(num2str(round(l_c*100*100)/100), '%'));
            
            %do the plotting
            [x, y, min_ret, max_ret, portf_ret] = ...
                frontier(l_c, u_c, portf_ret);     
            [portf_std, weights] = optimize(portf_ret, l_c, u_c);
            set_slider3(portf_ret, min_ret, max_ret, handles);
            plot_set(x, y, portf_ret, portf_std, weights, handles) 
            
            pause(pause_interval)
        %if the slider is at minval we change the direction
        else
            direction = 1;
        end
    end
    %if the sweeping was activated via button then it should be working
    %just for 5 seconds
    t_end = toc;
    if t_end >= 5 && get(handles.pushbutton2, 'Value') == 1.0
        set(handles.sweep_lo, 'value', 0.0)
    end
    if get(handles.pushbutton2, 'Value') == 0.0
        set(handles.pushbutton2, 'String', 'Sweep at random!');
    end
end


% --- Executes on entering text into edit_up.
function edit_up_Callback(hObject, eventdata, handles)
% hObject    handle to edit_up (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_up as text
%        str2double(get(hObject,'String')) returns contents of edit_up as a double

%check whether the entered data is fine
u_c = str2double(strrep(get(handles.edit_up, 'String'), '%', ''));

if isnan(u_c) == 1  
    u_c = 0;
else
    u_c = u_c/100;
end

l_c = get(handles.edit_lo, 'Value');
slider_min = get(handles.slider1, 'Min');
slider_max = get(handles.slider1, 'Max');
portf_ret = get(handles.slider3, 'Value');

%if a value too big or too small is entered, it gets corrected
if u_c > slider_max
    u_c = slider_max;
elseif u_c < slider_min
    u_c = slider_min;
end

set(handles.slider1, 'Value', u_c);
set(handles.edit_up, 'Value', u_c)
set(handles.edit_up, 'String', ...
    strcat(num2str(round(u_c*100*100)/100), '%'));

%do the plotting
[x, y, min_ret, max_ret, portf_ret] = frontier(l_c, u_c, portf_ret);
[portf_std, weights] = optimize(portf_ret, l_c, u_c);
set_slider3(portf_ret, min_ret, max_ret, handles);
plot_set(x, y, portf_ret, portf_std, weights, handles)


% --- Executes on entering text into edit_lo.
function edit_lo_Callback(hObject, eventdata, handles)
% hObject    handle to edit_lo (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_lo as text
%        str2double(get(hObject,'String')) returns contents of edit_lo as a double

%check whether the entered data is fine
l_c = str2double(strrep(get(handles.edit_lo, 'String'), '%', ''));

if isnan(l_c) == 1  
    l_c = 0;
else
    l_c = l_c/100;
end

u_c = get(handles.edit_up, 'Value');
slider_min = get(handles.slider2, 'Min');
slider_max = get(handles.slider2, 'Max');
portf_ret = get(handles.slider3, 'Value');
  
%if a value too big or too small is entered, it gets corrected
if l_c > slider_max
    l_c = slider_max;
elseif l_c < slider_min
    l_c = slider_min;
end

set(handles.slider2, 'Value', l_c);
set(handles.edit_lo, 'Value', l_c)
set(handles.edit_lo, 'String', ...
    strcat(num2str(round(l_c*100*100)/100), '%'));

%do the plotting
[x, y, min_ret, max_ret, portf_ret] = frontier(l_c, u_c, portf_ret);
[portf_std, weights] = optimize(portf_ret, l_c, u_c);
set_slider3(portf_ret, min_ret, max_ret, handles);
plot_set(x, y, portf_ret, portf_std, weights, handles)


% --- Executes on slider3 (return) movement.
function slider3_Callback(hObject, eventdata, handles)
% hObject    handle to slider3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider
u_c = get(handles.slider1, 'Value');
l_c = get(handles.slider2, 'Value');
portf_ret = get(handles.slider3, 'Value');

Ydata = get(get(handles.axes1, 'Children'), 'Ydata');
Xdata = get(get(handles.axes1, 'Children'), 'Xdata');
y = Ydata{3}'/100;
x = Xdata{3}'/100;

set(handles.edit_ret, 'String', ...
    strcat(num2str(round(portf_ret*100*100)/100), '%'));

%do the plotting
[portf_std, weights] = optimize(portf_ret, l_c, u_c);
plot_set(x, y, portf_ret, portf_std, weights, handles)


% --- Executes on checking/unchecking of sweep_ret checkbox.
function sweep_ret_Callback(hObject, eventdata, handles)
% hObject    handle to sweep_ret (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of sweep_ret

pause_interval = 0.01;

minval = get(handles.slider3, 'Min');
maxval = get(handles.slider3, 'Max');
step = (maxval - minval) / 100;

u_c = get(handles.slider1, 'Value');
l_c = get(handles.slider2, 'Value');

Ydata = get(get(handles.axes1, 'Children'), 'Ydata');
Xdata = get(get(handles.axes1, 'Children'), 'Xdata');
y = Ydata{3}'/100;
x = Xdata{3}'/100;

%start the sweeping in a random direction
% Initialize the Mersenne Twister
[state, ifail] = g05kg(nag_int(3), nag_int(0));
%generate one number
[state, rnd, ifail] = g05tl(nag_int(1), nag_int(1), nag_int(2), state);
direction = rnd(1);

%this should be working for 5 seconds if activated via button
tic;

%if the checkbox is checked
while get(handles.sweep_ret, 'value') == 1.0
    set(handles.pushbutton2, 'String', 'Stop sweeping');
    portf_ret = get(handles.slider3, 'Value');
    if direction == 1 %move to the right
        %if there is still space to move to the right then move
        if portf_ret < maxval
            portf_ret = portf_ret + step;
            %maxval cannot be exceeded
            if portf_ret > maxval
                portf_ret = maxval;
            end
            set(handles.slider3, 'Value', portf_ret);
            set(handles.edit_ret, 'Value', portf_ret);
            set(handles.edit_ret, 'String', ...
                strcat(num2str(round(portf_ret*100*100)/100), '%'));
            
            %do the plotting
            [portf_std, weights] = optimize(portf_ret, l_c, u_c);
            plot_set(x, y, portf_ret, portf_std, weights, handles)
            
            pause(pause_interval)
        %if the slider is at maxval we change the direction
        else
            direction = 2;
        end
    else %move to the left
        %if there is still space to move to the left then move
        if portf_ret > minval
            portf_ret = portf_ret - step;
            %minval cannot be exceeded
            if portf_ret < minval
                portf_ret = minval;
            end
            set(handles.slider3, 'Value', portf_ret);
            set(handles.edit_ret, 'Value', portf_ret);
            set(handles.edit_ret, 'String', ...
                strcat(num2str(round(portf_ret*100*100)/100), '%'));
            
            %do the plotting
            [portf_std, weights] = optimize(portf_ret, l_c, u_c);
            plot_set(x, y, portf_ret, portf_std, weights, handles)  
            
            pause(pause_interval)
        %if the slider is at minval we change the direction
        else
            direction = 1;
        end
    end
    %if the sweeping was activated via button then it should be working
    %just for 5 seconds
    t_end = toc;
    if t_end >= 5 && get(handles.pushbutton2, 'Value') == 1.0
        set(handles.sweep_ret, 'value', 0.0)
    end
    if get(handles.pushbutton2, 'Value') == 0.0
        set(handles.pushbutton2, 'String', 'Sweep at random!');
    end
end


% --- Executes on entering text into edit_ret.
function edit_ret_Callback(hObject, eventdata, handles)
% hObject    handle to edit_ret (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_ret as text
%        str2double(get(hObject,'String')) returns contents of edit_ret as a double

%check whether the entered data is fine
portf_ret = str2double(strrep(get(handles.edit_ret, 'String'), '%', ''));

if isnan(portf_ret) == 1  
    portf_ret = 0;
else
    portf_ret = portf_ret/100;
end

u_c = get(handles.edit_up, 'Value');
l_c = get(handles.edit_lo, 'Value');
slider_min = get(handles.slider3, 'Min');
slider_max = get(handles.slider3, 'Max');

Ydata = get(get(handles.axes1, 'Children'), 'Ydata');
Xdata = get(get(handles.axes1, 'Children'), 'Xdata');
y = Ydata{3}'/100;
x = Xdata{3}'/100;

%if a value too big or too small is entered, it gets corrected
if portf_ret > slider_max
    portf_ret = slider_max;
elseif portf_ret < slider_min
    portf_ret = slider_min;
end

set(handles.slider3, 'Value', portf_ret);
set(handles.edit_ret, 'Value', portf_ret)
set(handles.edit_ret, 'String', ...
    strcat(num2str(round(portf_ret*100*100)/100), '%'));

% do the plotting
[portf_std, weights] = optimize(portf_ret, l_c, u_c);
plot_set(x, y, portf_ret, portf_std, weights, handles)


% --- Executes when figure1 is resized.
function figure1_ResizeFcn(hObject, eventdata, handles)
% hObject    handle to figure1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%fig_size = get(handles.figure1, 'Position');

%fig_w = fig_size(3);
%fig_h = fig_size(4);

%set(handles.axes1, 'OuterPosition', [0.0*fig_w 0.0*fig_h ...
 %   0.4*fig_w 0.4*fig_h]);
%set(handles.axes2, 'Position', [0.55*fig_w 0.0909*fig_h ...
    %0.4*fig_w 0.4*fig_h]);
%set(handles.axes2, 'OuterPosition', [0.35*fig_w 0.00*fig_h ...
 %   0.44*fig_w 0.4*fig_h]);

%width = res(3);
%set(handles.axes1, 'Position', w/2)
%set(handles.axes2, 'Position', w/2)


% --- Does random sweeping of sliders
function pushbutton2_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%if the button is captioned 'stop sweeping' the clicking it will force the
%loop to stop
if strcmp(get(handles.pushbutton2, 'String'), 'Stop sweeping') == 1
    set(handles.pushbutton2, 'Value', 0.0);
end

if get(handles.pushbutton2, 'Value') == 0.0 % don't sweep
    set(handles.sweep_up, 'Value', 0.0);
    set(handles.sweep_lo, 'Value', 0.0);
    set(handles.sweep_ret, 'Value', 0.0);
    sweep_up_Callback(0, 0, handles)
    sweep_lo_Callback(0, 0, handles)
    sweep_ret_Callback(0, 0, handles)
    set(handles.pushbutton2, 'String', 'Sweep at random!');
    set(handles.pushbutton2, 'Value', 0.0);
else
    set(handles.pushbutton2, 'String', 'Stop sweeping');
    % Initialize the Mersenne Twister
    [state, ifail] = g05kg(nag_int(3), nag_int(0));
    while get(handles.pushbutton2, 'Value') == 1.0 
        %generate one pseudorandom number
        [state, x, ifail] = g05tl(nag_int(1), nag_int(1), nag_int(3), state);
        which_slider = x(1);
        if which_slider == 1
            set(handles.sweep_up, 'Value', 1.0);
            set(handles.sweep_lo, 'Value', 0.0);
            set(handles.sweep_ret, 'Value', 0.0);
            sweep_up_Callback(0, 0, handles)
        elseif which_slider == 2
            set(handles.sweep_lo, 'Value', 1.0);
            set(handles.sweep_up, 'Value', 0.0);
            set(handles.sweep_ret, 'Value', 0.0);
            sweep_lo_Callback(0, 0, handles)
        else
            set(handles.sweep_ret, 'Value', 1.0);
            set(handles.sweep_up, 'Value', 0.0);
            set(handles.sweep_lo, 'Value', 0.0);
            sweep_ret_Callback(0, 0, handles)        
        end
    end
end


% --- Finds the minimum variance portfolio
function pushbutton3_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

u_c = get(handles.edit_up, 'Value');
l_c = get(handles.edit_lo, 'Value');

Ydata = get(get(handles.axes1, 'Children'), 'Ydata');
Xdata = get(get(handles.axes1, 'Children'), 'Xdata');
y = Ydata{3}'/100;
x = Xdata{3}'/100;

portf_ret = get_optimal_ret(x, y);

set(handles.slider3, 'Value', portf_ret);
set(handles.edit_ret, 'Value', portf_ret)
set(handles.edit_ret, 'String', ...
    strcat(num2str(round(portf_ret*100*100)/100), '%'));

% do the plotting
[portf_std, weights] = optimize(portf_ret, l_c, u_c);
plot_set(x, y, portf_ret, portf_std, weights, handles)


% finds the optimal return (with minimum variance)
function [portf_ret, portf_std] = get_optimal_ret(x, y)

[portf_std, pos_portf_std] = min(x);
portf_ret = y(pos_portf_std);


%set the values for the slider
function set_slider3(portf_ret, min_ret, max_ret, handles)

set(handles.text_ret_min, 'String', ...
    strcat(num2str(round(min_ret*100*100)/100), '%'));
set(handles.text_ret_max, 'String', ...
    strcat(num2str(round(max_ret*100*100)/100), '%'));

% if the return which is currently plotted does not fit within the range,
% we need to make it fit
if portf_ret >= max_ret
    portf_ret = max_ret;
elseif portf_ret <= min_ret
    portf_ret = min_ret;
end

% the string is in percentage (rounded value), the value not
set(handles.edit_ret, 'String', ...
    strcat(num2str(round(portf_ret*100*100)/100), '%'));
set(handles.edit_ret, 'Value', portf_ret);

% if minval==0.1 then no need for slider3. At this point the frontier
% consists of just one point
minval = get(handles.slider1, 'Value');

if minval ~= 0.1
    set(handles.slider3, 'Enable', 'On');
    set(handles.sweep_ret, 'Enable', 'On');
    set(handles.edit_ret, 'Enable', 'On');
    set(handles.slider3, 'Value', portf_ret);
    set(handles.slider3, 'Max', max_ret);
    set(handles.slider3, 'Min', min_ret);
else
    set(handles.slider3, 'Enable', 'Off');
    set(handles.sweep_ret, 'Enable', 'Off');   
    set(handles.edit_ret, 'Enable', 'Off');
end


% make and format the plots
function plot_set(x, y, portf_ret, portf_std, weights, handles)

%get data to plot each stock
[n, a, std, means, l_c, u_c, xx, labels, empty_labels] = opt_data();

plot(handles.axes1, 100*x, 100*y, ...
    100*std, 100*means, '.', 100*portf_std, 100*portf_ret, '*');
bar(handles.axes2, 100*weights );

%set the value of the standard deviation for given return
set(handles.text_risk, 'String', ...
    strcat(num2str(round(portf_std*100*100)/100), '%'))

%select the first axes as the current axes
set(handles.figure1,'CurrentAxes',handles.axes1)

%format this plot
if max(y) >= 0.3 || min(y) <= -0.3
    xlim([0 100]);
    ylim([-60 60]);
else
    xlim([0 80]);
    ylim([-30 30]);
end
title('The efficient frontier', 'FontWeight', 'b', 'FontSize', 13);
xlabel('Risk (%)');
ylabel('Return (%)');

%l = get(handles.axes1, 'Children');
%set(l, 'LineWidth', 1.5)

%select the second axes as the current axes
set(handles.figure1,'CurrentAxes',handles.axes2)

%format this plot
xlim([0 n+1]);
if max(weights) >= 1 || min(weights) <= -1
    ylim([-200 200]);
    text_pos = -190;
else
    ylim([-100 100]);
    text_pos = -95;
end

handles.plot2_title = title('Structure of the portfolio', ...
    'FontWeight', 'b', 'FontSize', 13');
set(get(handles.axes2,'Children'), 'FaceColor', [1 0.5490 0])
xlabel('Asset');
ylabel('Structure (%)');
set(handles.axes2, 'XTickLabel', (empty_labels));

for i = 1 : n
    text(i, text_pos, labels(i), 'Rotation', 90, ...
        'HorizontalAlignment', 'left', 'FontWeight', 'Bold')
end


%set the minimum for slider1 depending on the number of assets
function set_slider1(handles)

% number of assets
[n, a, std, means, l_c, u_c, x, labels, empty_labels] = opt_data();

set(handles.slider1, 'Min', 1/n)
set(handles.text_up_min, 'String', ...
    strcat(num2str(round(100*100/n)/100), '%'));


% --- Outputs from this function are returned to the command line.
function varargout = Optimization_demo_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;
