function e04cb_demo(pause_time)

if (nargin < 1)
  pause_time = 0.1;
end

% Problem data
% user struct will be used to communicate previous points back
% into the objective function, for plotting purposes. Plotting
% occurs if user.plot ~= 0
% Set user.savemovie = 1 to create a bunch of PNG files with a sequence
% of movie frames.
user = struct('print', 1, ...
              'plot', 1, ...
              'savemovie', 0, ...
              'framenumber', 0, ...
              'mypause', pause_time, ...
              'sim_old', Inf);
tolf = sqrt(x02aj());
tolx = tolf;
maxcal = nag_int(100);
xinit = [-1; 1];

if (user.plot ~= 0)
  
  % Box bounds for the purposes of the contour plot
  bl = [-1.5; -2];
  bu = -bl;

  e04cb_contours(xinit, bl, bu);
end

% Create a button and wait for it to be pressed before continuing
hbutton = uicontrol('Position', [10 20 140 40], ...
                    'String', 'Start', 'Callback', @button_handler);
uiwait(gcf); % Wait until "Start" button is pressed.

[xmin, fmin, user, ifail] = e04cb(xinit, tolf, tolx, ...
                                  'e04cb_objective', 'e04cb_monitor', ...
                                  maxcal, 'user', user);

disp(sprintf('\n'));

if (ifail ~= 0)
  disp(['e04cb returned with ifail = ' num2str(ifail)]);
else
  disp(['The final function value is ' num2str(fmin)]);
  disp('at the point');
  disp(xmin);
end

if (user.plot ~= 0)
  plot(xmin(1),xmin(2),'ro','Linewidth',2,'Markersize',10);
  plot(bl(1)+0.2, bu(2)+0.4, 'ro', 'Linewidth', 2, 'MarkerSize', 10);
  t = sprintf('Final f val. = %7.4g', fmin);
  text(bl(1)+0.3, bu(2)+0.4, t, 'FontSize', 14);
  t = sprintf('at (%7.4f,%7.4f)', xmin(1), xmin(2));
  text(bl(1)+0.3, bu(2)+0.2, t, 'FontSize', 14);
  hold off
end

set(hbutton, 'String', 'Close');

function button_handler(hObject, eventData)
label = get(hObject, 'String');

if strcmp(label, 'Close') || strcmp(label, 'Quit')
   close;
elseif strcmp(label, 'Pause')
   set(hObject, 'String', 'Resume');
   uiwait;
elseif strcmp(label, 'Start') || strcmp(label, 'Resume')
   set(hObject, 'String', 'Pause');
   uiresume;
elseif strcmp(label, 'Next')
   set(hObject, 'String', 'Close');
   uiresume;
end
