function [mode, objf, objgrd, user] = e04uc_peaks_objfun(mode, n, x, objgrd, nstate, user)

% Set use_gradients to 0 if you don't want to supply
% gradients. Then e04wd needs more function evaluations.
use_gradients = user.useGradients;

objf = 3 * (1-x(1))^2 * exp(-(x(1)^2) - (x(2)+1)^2) - ...
      10 * (x(1)/5 - x(1)^3 - x(2)^5) * exp(-x(1)^2 - x(2)^2)- ...
      1/3 * exp(-(x(1)+1)^2 - x(2)^2);
  
if mode == 1 || mode == 2

  % Derivatives: objgrd(i) is the derivative with
  % respect to x(i), for i = 1, 2.

  if use_gradients == 1
    objgrd(1) = ...
    ( ( (-60*x(1)*x(2)^5) + (-60*x(1)^4) + 102*x(1)*x(1)-6) * exp((-x(2)* ...
                                                  x(2))+(-x(1)*x(1)))+(2*x(1)+2 ...
                                                  )*exp((-x(2)*x(2))+(- ...
                                                  x(1)*x(1))+ (-2*x(1))-1)+ ...
      ((-18*x(1)^3)+36*x(1)*x(1)-18)*exp((-x(2)*x(2))+(-2*x(2))+(-x(1)*x(1))-1))/3;

    objgrd(2) = ...
    (((-60*x(2)^6)+150*x(2)^4+((-60*x(1)^3)+12*x(1))*x(2))*exp((-x(2)* ...
                                                  x(2))+(-x(1)*x(1)))+ ...
     2*x(2)*exp((-x(2)*x(2))+(-x(1)*x(1))+(-2*x(1))-1)+(((-18*x(1)*x(1))+36*x(1)-18)*x(2)+(-18*x(1)*x(1))+ ...
                                                  36*x(1)-18)*exp((-x(2)*x(2))+(-2*x(2))+(-x(1)*x(1))-1))/3;
  end

end

% Show progress on the graph plot, if required
if (user.plot == 1)
  oldx(1) = user.xPrev;
  oldx(2) = user.yPrev;

  % Draw a green line from previous to current point
  line([oldx(1),x(1)], [oldx(2),x(2)], 'Color', [0 1 0], 'Linewidth', 2);

  % Change the previous plotted point to blue
  plot(oldx(1), oldx(2), 'b*', 'Linewidth', 2, 'MarkerSize', 8);

  % Plot the new point in red
  plot(x(1), x(2), 'r*', 'Linewidth', 2, 'MarkerSize', 8);
 
  % Save the latest point in the user array for use next time around.
  user.xPrev = x(1);
  user.yPrev = x(2);

  % Display the current number of objective function evaluations
  frx = [1.2 3.45 3.45 1.2];
  fry = [-3.2 -3.2 -3.9 -3.9];
  fill3(frx, fry, ones(4,1), 'w');
  t = sprintf('Number of evaluations of objective function = %d', user.nCalls);
  t1 = text(1.25, -3.35, 1, t, 'FontSize', 10);
  t = sprintf('Current point is (%5.2f, %5.2f)', x(1), x(2));
  t2 = text(1.25, -3.55, 1, t, 'FontSize', 10);
  t = sprintf('Objective function at this point = %13.08f', objf);
  t3 = text(1.25, -3.75, 1, t, 'FontSize', 10);
  
  % Set savemovie = 1 to create a bunch of PNG files with a sequence
  % of movie frames.
  savemovie = 0;
  if savemovie == 1
    FF = getframe(gcf);
    [FX,Map] = frame2im(FF);
    filename = sprintf('frame%d.png', user.nCalls);
    imwrite(FX,filename,'png');
  end
  
  % Slow down the animation
  drawnow;
  pause(user.myPause);

  % One more function evaluation has been made
  user.nCalls = user.nCalls + 1;
  
  delete(t1);
  delete(t2);
  delete(t3);
end

