% Written by Marcin Krzysztofik (Marcin.Krzysztofik@nag.co.uk).  June 2010.

% This functions returns the values of min/max possible attainable
% returns in the portfolio optimization problem.

% Inputs:
%
%   l_c - lower constraint for short selling (0 - no short selling, -1 -
%   short selling up to -100% allowed);
%   
%   u_c - upper constraint for investment (0.5 - up to 50% of the capital 
%   can be invested into each asset).
%
% Outputs:
%
%   min_ret - the minimum attainable return in the frontier;
%
%   max_ret - the maximum attainable return in the frontier.

function [min_ret, max_ret] = get_extreme_returns(l_c, u_c)

% take the data from another m-file
[n, a, std, means, l_cc, u_cc, x, labels, empty_labels] = opt_data();

% lower bounds for weights and lin constr
bl = ones(1, n)*l_c;
bl(n+1) = 1;
% upper bounds for weights and lin constr
bu = ones(1, n)*u_c;
bu(n+1) = 1;

% parameters for the optimizer
c = ones(1, n);
istate = zeros(n+1, 1, nag_int_name);
kx = zeros(n, 1, nag_int_name);
b = [0];

% initialize the optimizer, LP problem
[cwsav, lwsav, iwsav, rwsav, ifail] = e04wb('e04nc');
[lwsav, iwsav, rwsav, inform] = ...
    e04ne('Problem Type=LP', lwsav, iwsav, rwsav);

% find the minimum return
[istateOut, kxOut, xOut_min, aOut, bOut, iter, ...
    min_ret, clamda, lwsavOut, iwsavOut, rwsavOut, ifail] = ...
    e04nc(c, bl, bu, means, istate, kx, x, a, b, lwsav, iwsav, rwsav);

% find the maximum return
[istateOut, kxOut, xOut_max, aOut, bOut, iter, ...
    max_ret, clamda, lwsavOut, iwsavOut, rwsavOut, ifail] = ...
    e04nc(c, bl, bu, -means, istate, kx, x, a, b, lwsav, iwsav, rwsav);

% e04nc is a minimizer so instead of maximising f it needs to minimize -f
max_ret = -max_ret;

end
