function h02da_demo_new
% Demonstrates the mixed integer nonlinear solver (h02da)
%

% Main window size
fw = 1310;
fh = 700;

% Figure
f = figure('Visible','off',...
           'Name','Mixed Integer Nonlinear Optimization: h02da',...
           'NumberTitle','off',...
           'MenuBar','none',...
           'Position',[360,500,fw,fh]);
% Help
hm = uimenu(f,'Label','Help','CallBack',@displayhelp);

% Defaults
rhodef = 0.1;
pdef = 5;
maxitdef = 200;
accdef = 1e-4;

% Components
l = 80; % distance from LHS of figure
s = 50; % spacing between controls
tw = 50; % text label width
sl = 200; % slider length
sh = 25; % slider height

hrho = uicontrol('Style','slider',...
                 'Min',0,...
                 'Max',1,...
                 'Value',rhodef,...
                 'SliderStep',[0.05 0.1],...
                 'Callback',@rhovalue,...
                 'Position',[l fh-s sl sh]);
       uicontrol('Style','text',...
                 'Position',[l-tw fh-s tw sh],...
                 'String','rho');
       erho = uicontrol('Style','edit',...
                        'String',num2str(rhodef),...
                        'Enable','inactive',...
                        'Position',[l+sl+5 fh-s tw sh]);
      
hp  = uicontrol('Style','slider',...
                 'Min',1,...
                 'Max',8,...
                 'Value',pdef,...
                 'SliderStep',[0.1 0.1],...
                 'Callback',@pvalue,...
                 'Position',[l fh-2*s sl sh]);
      uicontrol('Style','text',...
                'Position',[l-tw fh-2*s tw sh],...
                'String','p');
      ep = uicontrol('Style','edit',...
                     'String',num2str(pdef),...
                     'Enable','inactive',...
                     'Position',[l+sl+5 fh-2*s tw sh]);
     
hmaxit  = uicontrol('Style','slider',...
                 'Min',1,...
                 'Max',500,...
                 'Value',maxitdef,...
                 'SliderStep',[0.1 0.1],...
                 'Callback',@maxitvalue,...
                 'Position',[fw-(l+sl+tw) fh-s sl sh]);
      uicontrol('Style','text',...
                'Position',[fw-(l+sl+2*tw+5) fh-s tw sh],...
                'String','maxit');
      emaxit = uicontrol('Style','edit',...
                     'String',num2str(maxitdef),...
                     'Enable','inactive',...
                     'Position',[fw-(l+tw-5) fh-s tw sh]);
                 
hacc  = uicontrol('Style','slider',...
                 'Min',1e-8,...
                 'Max',1e-2,...
                 'Value',accdef,...
                 'SliderStep',[0.1 0.1],...
                 'Callback',@accvalue,...
                 'Position',[fw-(l+sl+tw) fh-2*s sl sh]);
      uicontrol('Style','text',...
                'Position',[fw-(l+sl+2*tw+5) fh-2*s tw sh],...
                'String','acc');
      eacc = uicontrol('Style','edit',...
                     'String',num2str(accdef),...
                     'Enable','inactive',...
                     'Position',[fw-(l+tw-5) fh-2*s tw sh]);
 
hsolve = uicontrol('Style','pushbutton',...
                   'String','Solve',...
                   'Callback',@solver,...
                   'Position',[l fh-3.5*s 70 35]);

% Data
[rd,colnames,rownames] = get_data();

% Data table
hcolnames = cellfun(@(x)['<html><h3>' x '</h3></html>'],colnames,'uni',false);

tble = uitable(f,...
               'Data',rd,...
               'ColumnName',hcolnames,...
               'ColumnFormat',{'char' 'char' 'char' 'char' 'char' 'char' 'char' 'char'},...
               'RowName',rownames);
tble.Position(3) = tble.Extent(3); % width
tble.Position(4) = tble.Extent(4); % height

% Set the data
d.rho = rhodef;
d.p = pdef;
d.maxit = maxitdef;
d.acc = accdef;
d.erho = erho;
d.ep = ep;
d.emaxit = emaxit;
d.eacc = eacc;
d.tble = tble;
d.rd = rd;
d.colnames = colnames;
d.reset = false;
guidata(f,d);

% Make the UI visible.
f.Visible = 'on';
end

function resettable(d)
if d.reset
    set(d.tble,'ColumnName',...
        cellfun(@(x)(['<html><h3>' x '</h3></html>']),d.colnames,'uni',false));
    set(d.tble,'Data',d.rd);
end
end

function rhovalue(hobject,evendata)
d = guidata(hobject);

resettable(d);

v = get(hobject,'Value');

set(d.erho,'String',num2str(v));
d.rho = v;
guidata(hobject,d);
end

function pvalue(hobject,evendata)
d = guidata(hobject);

resettable(d);

v = round(get(hobject,'Value'));

set(d.ep,'String',num2str(v));
d.p = v;
guidata(hobject,d);
end

function maxitvalue(hobject,evendata)
d = guidata(hobject);

resettable(d);

v = round(get(hobject,'Value'));

set(d.emaxit,'String',num2str(v));
d.maxit = v;
guidata(hobject,d);
end

function accvalue(hobject,evendata)
d = guidata(hobject);

resettable(d);

v = get(hobject,'Value');

set(d.eacc,'String',num2str(v));
d.acc = v;
guidata(hobject,d);
end

function [rd,colnames,rownames] = get_data()

% Returns data
rownames = {'1','2','3','4','5','6','7','8','9','10','11','12','13',...
            '14','15','16','17','18','19','20','21','22',...
            ''};
colnames = {'US 3 month t-bills' 'US government long bonds' ...
'S&P 500' 'Wilshire 5000' 'Nasdaq composite' ...
'Lehman Brothers bonds index' 'Morgan Stanley EAFE Index' 'Gold Index'};

rd = {1.075  0.942  0.852  0.815  0.698  1.023  0.851  1.677;
    1.084  1.020  0.735  0.716  0.662  1.002  0.768  1.722;
    1.061  1.056  1.371  1.385  1.318  1.123  1.354  0.760;
    1.052  1.175  1.236  1.266  1.280  1.156  1.025  0.960;
    1.055  1.002  0.926  0.974  1.093  1.030  1.181  1.200;
    1.077  0.982  1.064  1.093  1.146  1.012  1.326  1.295;
    1.109  0.978  1.184  1.256  1.307  1.023  1.048  2.212;
    1.127  0.947  1.323  1.337  1.367  1.031  1.226  1.296;
    1.156  1.003  0.949  0.963  0.990  1.073  0.977  0.688; 
    1.117  1.465  1.215  1.187  1.213  1.311  0.981  1.084;
    1.092  0.985  1.224  1.235  1.217  1.080  1.237  0.872;
    1.103  1.159  1.061  1.030  0.903  1.150  1.074  0.825;
    1.080  1.366  1.316  1.326  1.333  1.213  1.562  1.006;
    1.063  1.309  1.186  1.161  1.086  1.156  1.694  1.216;
    1.061  0.925  1.052  1.023  0.959  1.023  1.246  1.244;
    1.071  1.086  1.165  1.179  1.165  1.076  1.283  0.861;
    1.087  1.212  1.316  1.292  1.204  1.142  1.105  0.977;
    1.080  1.054  0.968  0.938  0.830  1.083  0.766  0.922;
    1.057  1.193  1.304  1.342  1.594  1.161  1.121  0.958;
    1.036  1.079  1.076  1.090  1.174  1.076  0.878  0.926;
    1.031  1.217  1.100  1.113  1.162  1.110  1.326  1.146;
    1.045  0.889  1.012  0.999  0.968  0.965  1.078  0.990};
end

function solver(hobject,eventdata)

gd = guidata(hobject);
rd = gd.rd;

% Constants
zero = 0;
one = 1;
bigish = 10000;

% Sizes
t = nag_int(size(rd,1)); % t dates of m instruments;
m = nag_int(size(rd,2)); % optimise p instruments with minimum return rho

% No. of constraints
nclin = nag_int(3) + m;
ncnln = nag_int(0);

% Get param from GUI
rho = gd.rho;
p = gd.p;
maxit = nag_int(gd.maxit);
acc = gd.acc;

% Covariance matrix
vc = cov(cell2mat(rd));

% Factorise covariance matrix
[fvc,info] = f07fd('L',vc);
if info>0
   display('The covariance matrix is not positive definite (f07fd)');
   return
end

% Create full matrix because there's no in-built symmetric matrix*vector
% and set up communication structure
user.chol = fvc + fvc' - diag(diag(fvc));
user.vc = vc;
user.m = m;

% Variables
n = 2*m;
varcon = zeros(n+nclin+ncnln,1,nag_int_name);

% Holdings xi are continuous in [0,Inf)
varcon(1:m) = nag_int(0);

% yi are binary indicators
varcon(m+1:n) = nag_int(1);

% Bounds
bl = zeros(n,1) + 1e-6;
bu = [ones(m,1).*bigish; ones(m,1)];

% Linear constraints, A
lda = nclin;
a = zeros(lda,n);
d = zeros(nclin,1);

% Sum{xi}=1
varcon(n+1) = nag_int(3);
a(1,1:m) = one;
d(1) = one;

% Sum{yi}=p
varcon(n+2) = nag_int(3);
a(2,m+1:n) = -one;
d(2) = -p;

% Sum{meanri*xi}>=rho
varcon(n+3) = nag_int(4);
a(3,1:m) = mean(cell2mat(rd));
d(3) = rho;

% xi<=yi
varcon(n+4:n+nclin) = nag_int(4);
for i=1:m
  a(3+i,i) = -one;
  a(3+i,m+i) = one;
end
d(4:nclin) = zero;

% Initial estimate
x = [ones(m,1)./double(m); zeros(m,1)];

% Options
iopts = zeros(200, 1, nag_int_name);
opts = zeros(100, 1);
[iopts, opts, ~] = h02zk('Initialize = h02da', iopts, opts);

% Catch any errors in msgbox
nag_issue_warnings(true);

% Optimise
[~, x, ~, ~, ~, objmip, ~, ifail] = h02da( ...
         ncnln, a, d, bl, bu, varcon, x, 'h02ddm', @objfun, iopts, opts, ...
         'maxit',maxit,'acc',acc,'user', user);

% Results
if  ifail==0    
    colnames = cellfun(@applyresult,num2cell(x(9:16)'),gd.colnames,'uni',false);
    set(gd.tble,'ColumnName',colnames);
    
    xx = x(1:8)';
    xx(x(9:16)<0.5) = 0;
    res = cellfun(@(x)['<html><body text="#0000FF">' num2str(x) '</center></body></html>'],...
        num2cell(xx),'uni',false);
    set(gd.tble,'Data',[rd; res]);
    
    gd.reset = true;
    guidata(hobject,gd);
else
  uiwait(msgbox({['h02da returns error code ' num2str(ifail)] ...
           ''...
           'Change one or more parameter values'},'NAG ifail','Modal'));
end

% Done
end

function cres = applyresult(x,c)
    hs = '<html><h3 style="background-color:';
    he = '</h3></html>';
    green = 'green">';
    red = 'red">';
    
    if x>0
        cres = [hs green c he];
    else
        cres = [hs red c he];
    end
end

function [mode, objmip, objgrd, user] = ...
         objfun(mode, n, varcon, x, objgrd, nstate, user)

objmip = 0;
objgrd(1:n) = 0;

m = user.m;

if mode==0
% Objective value
  temp = user.chol*x(1:m);
  objmip = temp'*temp;
else
  objgrd(1:m) = 2.*(x(1:m)'*user.vc);
end
end

function displayhelp(hobj,eventdata)
c.Interpreter = 'tex';
c.WindowStyle = 'modal';

msgbox({'This demonstration optimizes a portfolio selection of {\it p} assets from eight available assets with a return of at least \rho.' ...
    '' ...
    'Specifically, minimize:' ...
    '              {\it x}^T\Sigma{\it x}' ...
    'subject to:' ...
    '              {\it r}^T{\it x} \geq \rho' ...
    '              {\it x}_1+{\it x}_2+\ldots{\it x}_8 = 1'...
    '              {\it p}{\it x}-values \geq 0' ...
    '' ...
    'where {\it r} is the mean of returns and \Sigma is their covariance matrix.' ...
    '' ...
    '' ...
    'The maximum number of iterations of the optimizer and the requested accuracy can be controlled by the parameters{\bf maxit} and{\bf acc} respectively.' ...
    '' ...
    '' ...
    'After a successful optimization column labels in the table are colour coded as follows' ...
    '   \bullet red:   excluded from the portfolio' ...
    '   \bullet green: included in the portfolio' ...
    '' ...
    'Holding proportions {\it x} are displayed in blue in the last row.'},...
             'Cardinality constrained portfolio optimization',c);
end
