% Written by Marcin Krzysztofik (Marcin.Krzysztofik@nag.co.uk).  June 2010.

% Portfolio optimization, convex QP problem, Markovitz Theory

% Inputs:
%
%   l_c - lower constraint for short selling (0 - no short selling, -1 -
%   short selling up to -100% allowed);
%   
%   u_c - upper constraint for investment (0.5 - up to 50% of the capital 
%   can be invested into each asset);
%
%   req_ret - the return for which we want to get the standard deviation.
%
% Outputs:
%
%   obj - the minimal standard deviation obtained for req_ret;
%   
%   xOut - the vector of weights of assets in this particular portfolio.

function [obj, xOut] = optimize(req_ret, l_c, u_c)

% get the input data
[n, a, std, means, l_cc, u_cc, x, labels, empty_labels] = opt_data();

% returns in first row, 1st gen lin constr, sum (ret*weights)=req_ret
% 2nd row- 2nd gen lin constr, sum (weights)=1
c = [means; ones(1, n)];

% lower bounds for weights and lin constr
bl = ones(1, n)*l_c;
bl(n+1) = req_ret;
bl(n+2) = 1;
% upper bounds for weights and lin constr
bu = ones(1, n)*u_c;
bu(n+1) = req_ret;
bu(n+2) = 1;

% parameters for the optimizer
cvec = [0];
istate = zeros(n+2, 1, nag_int_name);
kx = zeros(n, 1, nag_int_name);
b = [0];

% initalization of the optimizer
[cwsav, lwsav, iwsav, rwsav, ifail] = e04wb('e04nc');
[lwsav, iwsav, rwsav, inform] = ...
    e04ne('Problem Type=QP1', lwsav, iwsav, rwsav);

% optimizer, convex QP problem
[istateOut, kxOut, xOut, aOut, bOut, iter, ...
    obj, clamda, lwsavOut, iwsavOut, rwsavOut, ifail] = ...
    e04nc(c, bl, bu, cvec, istate, kx, x, a, b, lwsav, iwsav, rwsav);

% the function returns the value of the risk, which we need to transform
obj = sqrt(obj*2);

end

