function c02af_example

% diary c02af_example.r
fprintf('c02af example results\n\n');

n = nag_int(5);
a = [5, 30, -0.2,     50, -2, 10;
     6, 20, -6.0, 100000, 40,  1];

ex1(n,a);

ex2(n,a);

% diary off

function ex1(n, a)
  % Compute the roots of the polynomial
  [z, ifail] = c02af(a, n);

  disp('Computed roots of polynomial:');
  cz = z(1,:) + i*z(2,:);
  disp(cz');

function ex2(n, a)
  % Compute the roots of the polynomial
  [z, ifail] = c02af(a, n);

  % Estimate the accuracy of the computed roots using a perturbation analysis

  % Form the coefficients of the perturbed polynomial
  abar = a;
  epsbar = 3*x02aj;
  for j=1:2:n+1
    abar(:,j) = (1+epsbar)*abar(:,j);
  end
  for j=2:2:n+1
    abar(:,j) = (1-epsbar)*abar(:,j);
  end

  % Compute the roots of the perturbed polynomial
  [zbar, ifail] = c02af(abar, nag_int(n));

  % Perform error analysis
  % Initialise markers to 0 (unmarked)
  m = zeros(n, 1);
  r = zeros(n, 1);
  rmax = x02al;
  
  cz = z(1,:) + i*z(2,:);
  czbar = zbar(1,:) + i*zbar(2,:);
  for j=1:n
    deltai = rmax;
    r1 = abs(cz(j));

    % Loop over all perturbed roots (stored in zbar)
    for k=1:n
      % Compare the current unperturbed root to all unmarked perturbed roots.
      if m(k) == 0
        r2 = abs(czbar(k));
        deltac = abs(r1-r2);
        
        if deltac < deltai
          deltai = deltac;
          jmin = k;
        end
      end
    end

    % Mark the selected perturbed root
    m(jmin) = 1;

    % Compute the relative error
    if r1 ~= 0
      r3 = abs(czbar(jmin));
      di = min(r1,r3);
      r(j) = max(deltai/max(di,deltai/rmax),x02aj);
    end

  end

  fprintf('\nComputed Roots of Polynomial   Error Estimate\n');
  fprintf('                              (machine dependent)\n');
  for j=1:n
    if (z(2,j)<0)
      fprintf('%9.4f - %7.4fi           %8.2e\n',z(1,j),-z(2,j),r(j));
    else
      fprintf('%9.4f + %7.4fi           %8.2e\n',z(1,j),z(2,j),r(j));
    end
  end
