function c09fz_example

% diary c09fz_example.r
fprintf('c09fz example results\n\n');

% 3D Data
m = nag_int(4);
n = nag_int(4);
f = nag_int(4);
a = zeros(m,n,f);
a(1:2:m,:,1:2:f) = 0.01;
a(2:2:m,:,2:2:f) = 0.01;
a(2:2:m,:,1:2:f) = 1;
a(1:2:m,:,2:2:f) = 1;

% Add Noise
genid = nag_int(3);
subid = nag_int(0);
seed(1) = nag_int(642521);
[state, ifail] = g05kf(genid, subid, seed);
[state, x, ifail] = g05sk(m*n*f, 0, 1.0e-4, state);
an = a + reshape(x,[m,n,f]);

fprintf('\nInput data a:\n');
disp(a);
fprintf('\nNoisy data an:\n');
disp(an);

% Wavelet setup
wavnam = 'Haar';
mode = 'Period';
wtrans = 'Multilevel';
[nwl, nf, nwct, nwcn, nwcf, icomm, ifail] = ...
c09ac(...
      wavnam, wtrans, mode, m, n, f);

% Multi-level wavelet transform on noisy data
[c, dwtlvm, dwtlvn, dwtlvfr, icomm, ifail] = ...
c09fc(...
       n, f, an, nwct, nwl, icomm);

% Reconstruct without thresholding of detail coefficients
[b, ifail] = c09fd(nwl, c, m, n, f, icomm);
 
% Mean square error of noisy reconstruction
mse = (norm(reshape(a-b,[m*n*f,1]))^2)/(double(m*n*f));
fprintf('Without denoising Mean Square Error is %9.6f\n',mse);

% De-noise by applying hard threshold to detail coefficients
thresh = 0.01*sqrt(2*log(double(m*n*f)));
nt = 0;
nnt = 0;
for ilev = 1:nwl
  level = nag_int(nwl - ilev + 1);

  for detail = nag_int(1:7)
    % Extract the selected set of coefficients. 
    [d, icomm, ifail] = c09fy(...
                              level, detail, c, icomm);

    % Threshold
    d1 = dwtlvm(ilev);
    d2 = dwtlvn(ilev);
    d3 = dwtlvfr(ilev);
    for i = 1:d1
      for j = 1:d2
        for k = 1:d3
          if abs(d(i,j,k))<thresh
            d(i,j,k) = 0;
            nt = nt + 1;
          end
          nnt = nnt + 1;
        end
      end
    end
    % Insert de-noised coefficients back into c
    [c, icomm, ifail] = c09fz(...
                               level, detail, c, d, icomm);
  end
end

fprintf('\nNumber of coefficients denoised is %3d out of %3d\n',nt,nnt);

% Reconstruct data after threholding
[b, ifail] = c09fd(nwl, c, m, n, f, icomm);

% Mean square error of de-noised reconstruction
mse = (norm(reshape(a-b,[m*n*f,1]))^2)/(double(m*n*f));
fprintf('With denoising Mean Square Error is %9.6f\n\n',mse);

disp('Reconstruction of denoised input: ');
disp(b);

% diary off
