function d02cj_example

% diary d02cj_example.r
fprintf('d02cj example results\n\n');

% For communication with save.
global ykeep ncall xkeep;

% Initialize variables and arrays.
x      = 0;
xend   = 10;
y      = [0.5; 0.5; pi/5];
tol    = 0.0001;
relabs = 'Default';

ncall = 0;
ykeep = zeros(1,length(y));
xkeep = zeros(1,1);

fprintf('Case 1: intermediate output, root-finding\n\n');
for j = 4:5
    tol = double(10)^(-j);
    disp(['Calculation with tol = ',num2str(tol)]);
    disp(' X     Y(1)       Y(2)       Y(3)');

    % output outputs intermediate results.
    [xOut, yOut, ifail] = d02cj(x, xend, y, @fcn, tol, ...
        relabs, @output, @g);
    disp(' ');
    disp(['Root of Y(1) = 0.0 at ',num2str(xOut)]);
    disp('Solution is' );
    fprintf('  %8.4f   %8.4f   %8.4f\n\n', yOut);
end

fprintf('Case 2: no intermediate output, root-finding\n\n');
for j = 4:5
    tol = double(10)^(-j);
    disp(['Calculation with tol = ',num2str(tol)]);

    [xOut, yOut, ifail] = d02cj(x, xend, y, @fcn, tol, ...
        relabs, 'd02cjx', @g);
    disp(['Root of Y(1) = 0.0 at ',num2str(xOut)]);
    disp('Solution is' );
    fprintf('  %8.4f   %8.4f   %8.4f\n\n', yOut);

    % Store the x value for plotting.
    xOut1 = xOut;
end

fprintf('Case 3: intermediate output, no root-finding\n\n');
for j = 4:5
    tol = double(10)^(-j);
    disp(['Calculation with tol = ',num2str(tol)]);
    disp(' X     Y(1)       Y(2)       Y(3)');

    % save stores intermediate values in xkeep, ykeep, which are
    % plotted later (it also outputs them).
    [xOut, yOut, ifail] = d02cj(x, xend, y, @fcn, tol, ...
        relabs, @save, 'd02cjw');
    fprintf('\n');
end

fprintf(['Case 4: no intermediate output, no root-finding ', ...
    '(integrate to xend)\n\n']);
for j = 4:5
    tol = double(10)^(-j);
    disp(['Calculation with tol = ',num2str(tol)]);
    disp(' X     Y(1)       Y(2)       Y(3)');
    fprintf('%2d   %8.4f   %8.4f   %8.4f\n', x, y);

    [xOut, yOut, ifail] = d02cj(x, xend, y, @fcn, tol, ...
        relabs, 'd02cjx','d02cjw');
    fprintf('\n');
end
% Plot results.
nres = 0.5*length(xkeep);
xplot = xkeep(nres+1:2*nres);
yplot = ykeep(nres+1:2*nres, :);
fig1 = figure;
display_plot(xplot, yplot, xOut1)
% print(fig1,'-dpng','-r75','d02cj_fig1.png');
% print(fig1,'-deps','-r75','d02cj_fig1.eps');

% diary off

function xsolOut = save(xsol, y)
% For communication with main routine.
global ykeep ncall xkeep;

% This version of the intermediate output routine stores the values
% (so they can be plotted in the main routine).
ncall = ncall+1;
ykeep(ncall,:) = y;
xkeep(ncall,:) = xsol;
fprintf('%2d   %8.4f   %8.4f   %8.4f\n', xsol, y);
xsolOut = xsol + 2;

function xsolOut = output(xsol, y)
  % Output intermediate values of solution.
  fprintf('%2d   %8.4f   %8.4f   %8.4f\n', xsol, y);
  xsolOut = xsol + 2;

function f = fcn(x,y)
  % Evaluate the derivatives.
  f = zeros(3,1);
  f(1) = tan(y(3));
  f(2) = -0.032*tan(y(3))/y(2) - 0.02*y(2)/cos(y(3));
  f(3) = -0.032/y(2)^2;

function result = g(x,y)
  % Evaluate g(x,y) when root-finding option is selected.
  result = y(1);

function display_plot(xplot, yplot, xOut1)
  % Plot the three curves.
  plot(xplot, yplot(:,1), '-+', ...
       xplot, yplot(:,2), '--x', ...
       xplot, yplot(:,3), ':*');
  % Mark the height=0 point.
  do_stem(xplot, yplot, xOut1);
  % Add title.
  title('ODE Solution using Adams Method with Root-finding');
  % Label the axes.
  xlabel('x');
  ylabel('Solution');
  % Add legend.
  legend('height','velocity','angle','height = 0','Location','Best');

function do_stem(xplot, yplot, xOut1)
  % Find the x bin that xOut1 lies in.
  for i = 1:length(xplot)
    if xplot(i) > xOut1
      break
    end
  end
  % Use linear interpolation to find the corresponding y values on the
  % two curves.
  dx = xplot(i)-xplot(i-1);
  ddx = xOut1-xplot(i-1);
  d1 = ddx/dx;

  f1 = yplot(i-1,2) + d1*(yplot(i,2)-yplot(i-1,2));
  f2 = yplot(i-1,3) + d1*(yplot(i,3)-yplot(i-1,3));
  % Plot the line from the x axis to the two y values.
  hold on
  stem([xOut1,xOut1],[f1,0],'k:s');
  hold on
  stem([xOut1,xOut1],[f2,0],'k:s');
