function d02hb_example

% diary d02hb_example.r
fprintf('d02hb example results\n\n');

% Set up initial values for first case.
n = nag_int(2);
n1 = nag_int(2);
p = [0.2;0];
pe = [1e-05;0.001];
e = [0.0001;0.0001];
m1 = nag_int(6);
marray = zeros(1,m1);

fprintf('d02hb example program results \n');

fprintf('\nCase 1 \n\n');

[pOut1, solnOut1, w1, ifail] = ...
  d02hb(...
         p, pe, e, m1, @fcn1, @bc1, @range1, 'n1', n1, 'n', n);

% Output results.
fprintf('Final parameters \n')
fprintf('    %1.3e   ',pOut1);
fprintf('\n\n Final solution \n')
fprintf('X-value     Components of solution \n')

[x0, x1] = range1(p);
h = (x1-x0)/double(m1-1);
for i = 1:m1;
  m = x0 + double(i-1)*h;
  fprintf('%7.3f', m);
  fprintf('%12.4f',solnOut1(1:n,i));
  fprintf('\n');
  % Save results for plotting.
  marray(i) = m;
end

fprintf('\nCase 2 \n\n');
% Set up initial values for second case.
n = nag_int(3);
n1 = n;
p = [32;6000;0.54];
pe = [1e-05;1e-4;1e-4];
e = [1e-2;1e-2;1e-2];
m1 = nag_int(6);
qarray = zeros(1,m1);

[pOut2, solnOut2, w2, ifail] = ...
  d02hb(...
         p, pe, e, nag_int(m1), @fcn2, @bc2, @range2, 'n1', n1, 'n', n);
% Output results.
fprintf('Final parameters \n')
fprintf('    %1.3e   ',pOut2);
fprintf('\n\n Final solution \n')
fprintf(' X-value        Components of solution \n')

[x0, x1] = range2(pOut2);
h = (x1-x0)/double(m1-1);
for i = 1:m1; 
  q = x0 + double(i-1)*h;
  fprintf('%6.0f', q);
  fprintf('%12.4f',solnOut2(1:n,i));
  fprintf('\n');
  % Save results for plotting.
  qarray(i) = q;
end

% Plot results.
fig1 = figure;
display_plot(marray, solnOut1, 'Derivative');
% print(fig1,'-dpng','-r75','d02hb_fig1.png');
% print(fig1,'-deps','-r75','d02hb_fig1.eps');

fig2 = figure;
display_plot(marray, solnOut2, 'Angle');
% print(fig2,'-dpng','-r75','d02hb_fig2.png');
% print(fig2,'-deps','-r75','d02hb_fig2.eps');

% diary off

function [g1, g2] = bc1(p)
  % Evaluate boundary conditions.
  z=0.1;
  g1(1) = 0.1+p(1)*sqrt(z)*0.1+0.01*z;
  g1(2) = p(1)*0.05/sqrt(z) + 0.01;
  g2(1) = 1/6;
  g2(2) = p(2);

function [g1, g2] = bc2(p)
  % Evaluate boundary conditions.
  g1(1) = 0.0;
  g1(2) = 500.0;
  g1(3) = 0.5;
  g2(1) = 0.0;
  g2(2) = 450.0;
  g2(3) = p(3);

function f = fcn1(x, y, p)
  % Evaluate derivatives.
  f = zeros(2,1);
  f(1) = y(2);
  f(2) = (y(1)^3-y(2))/2/x;

function f = fcn2(x, y, p)
  % Evaluate derivatives.
  f = zeros(3,1);
  f(1) = tan(y(3));
  f(2) = -p(1)*tan(y(3))/y(2) - 0.00002*y(2)/cos(y(3));
  f(3) = -p(1)/y(2)^2;

function [x0, x1] = range1(p)
  % Evaluate boundary points.
  x0 = 0.1;
  x1 = 16.0;

function [x0, x1] = range2(p)
  % Evaluate boundary points.
  x0 = 0.0;
  x1 = p(2);

function display_plot(x, y, ylabelString)
  % Choose the type of plot based on the y axis label.
  if strncmp(ylabelString, 'Derivative', 8)
    % Plot the results as a linear graph (2 y axes).
    [haxes, hline1, hline2] = plotyy(x, y(1,:), x, y(2,:));
    % Set the axis limits and the tick specifications to beautify the plot.
    set(haxes(1), 'YLim', [0.1 0.18]);
    set(haxes(1), 'XMinorTick', 'on', 'YMinorTick', 'on');
    set(haxes(1), 'YTick', [0.1 0.12 0.14 0.16 0.18]);
    set(haxes(2), 'YLim', [0 0.02]);
    set(haxes(2), 'YMinorTick', 'on');
    set(haxes(2), 'YTick', [0:0.005:0.02]);
    for iaxis = 1:2
      % These properties must be the same for both sets of axes.
      set(haxes(iaxis), 'XLim', [0 16]);
      set(haxes(iaxis), 'XTick', [0:4:16]);
    end
    set(gca, 'box', 'off'); 
    % Set the title.
    title({['Parameterised Two-point BVP using'], ...
           ['Initial Value Techniques & Newton Iteration']}, ...
          'position',[8,0.17]);
    % Label the axes.
    xlabel('x');
    ylabel(haxes(1),'Solution');
    yh2 = ylabel(haxes(2),ylabelString);
    set(yh2,'position',[15,0.01]);
    % Add legend.
    legend('solution y(x)','derivative y''(x)','Location','South');
    % Set some features of the two lines.
    set(hline1, 'Linewidth', 0.5, 'Marker', '+', 'LineStyle', '-');
    set(hline2, 'Linewidth', 0.5, 'Marker', 'x', 'LineStyle', '--');
  else
    % Plot the results as a linear graph (2 y axes).
    [haxes, hline1, hline2] = plotyy(x, y(1,:), x, y(3,:));
    % Add a third curve,
    hold on
    hline3 = plot(x, y(2,:),'Color','Magenta');
    % Set the axis limits and the tick specifications to beautify the plot.
    set(haxes(1), 'YLim', [-1000 1000]);
    set(haxes(1), 'XMinorTick', 'on', 'YMinorTick', 'on');
    set(haxes(1), 'YTick', [-1000:500:1000]);
    set(haxes(2), 'YLim', [-1 1]);
    set(haxes(2), 'YMinorTick', 'on');
    set(haxes(2), 'YTick', [-1:0.5:1]);
    for iaxis = 1:2
      % These properties must be the same for both sets of axes.
      set(haxes(iaxis), 'XLim', [0 16]);
      set(haxes(iaxis), 'XTick', [0:4:16]);
    end
    set(gca, 'box', 'off');
    % Set the title.
    title('Parameterized projectile problem');
    % Label the axes.
    xlabel('x');
    ylabel(haxes(1),'Height and Velocity');
    ylabel(haxes(2),ylabelString);
    % Add legend.
    legend('height','velocity','angle','Location','South');
    % Set some features of the three lines.
    set(hline1, 'Linewidth', 0.5, 'Marker', '+', 'LineStyle', '-');
    set(hline2, 'Linewidth', 0.5, 'Marker', '*', 'LineStyle', ':');
    set(hline3, 'Linewidth', 0.5, 'Marker', 'x', 'LineStyle', '--');
end
