function d02ja_example

% diary d02ja_example.r
fprintf('d02ja example results\n\n');

% Set up initial values.
n     = nag_int(2);
k1max = 8;
kpmax = 15;
lw    = 2*(kpmax+n)*(k1max+1)+7*k1max;
x0    = -1;
x1    = 1;
c     = zeros(1,kpmax);
p     = zeros(1,kpmax);

fprintf('  KP   K1   Chebyshev coefficients\n');
for kp = nag_int(10:5:kpmax)
  for k1 = nag_int(4:2:k1max)

        [c, ifail] = d02ja(n, @cf, @bc, x0, x1, k1, kp);
        % Output results.
        fprintf('%4d ',kp, k1);

        for kind = 1:k1
            fprintf('%8.4f  ',c(kind));
            if mod(kind, 6) == 0 && kind ~= k1
                fprintf('\n          ');
            end
        end
        fprintf('\n');
    end
end

% Now prepare to evaluate and plot the last solution.
fprintf('\n');
k1     = nag_int(8);
k1m1   = nag_int(k1-1);
m      = 9;
ia1    = nag_int(1);
xarray = zeros(k1,1);
yarray = zeros(k1,1);

fprintf('Last computed solution evaluated at %1d equally spaced points\n\n', m);
fprintf('    X         Y\n');
for i = 1:m
    x = (x0*double(m-i)+x1*double(i-1))/double(m-1);

    % Calling e02ak to evaluate the polynomial from its Chebyshev
    % representation.
    [y, ifail] = e02ak(k1m1, x0, x1, c, ia1, x);

    % Output results, and save them for plotting.
    fprintf('%8.4f  %8.4f  \n',x,y);
    xarray(i) = x;
    yarray(i) = y;
end
fig1 = figure;
display_plot(xarray, yarray)
% print(fig1,'-dpng','-r75','d02ja_fig1.png');
% print(fig1,'-deps','-r75','d02ja_fig1.eps');

% diary off

function [j, rhs] = bc(i)
% Define the boundary conditions.
rhs = 0;
if (i == 1)
    j = nag_int(1);
else
    j = nag_int(-1);
end

function result = cf(j, x)
% Define the differential equation to be solved.
if (j == 2)
    result = 0;
else
    result = 1;
end

function display_plot(x, y)
% Plot results.
plot(x, y, '-+');
% Add title.
title({'Two-point Boundary-value Problem for ODE',...
    'by Chebyshev-series using Collocation and Least-squares'})
% Label the axes.
xlabel('x');
ylabel('y');
