function d02ne_example

% diary d02ne_example.r
fprintf('d02ne example results\n\n');

fprintf('Example 1\n');
ex1;
fprintf('\nExample 2\n');
ex2;

% diary off

function ex1
  % Stiff Robertson problem

  % Initialize the problem, specifying that the Jacobian is to be
  % evaluated analytically using the provided routine jac.
  neq    = nag_int(3);
  maxord = nag_int(5);
  jceval = 'Analytic';
  hmax   = 0;
  h0     = 0;
  itol   = nag_int(1);
  lcom   = nag_int(200);
  [icom, com, ifail] = d02mw(...
                              neq, maxord, jceval, hmax, h0, itol, lcom);

  % Specify that the Jacobian is banded
  mu = nag_int(2);
  ml = nag_int(1);
  [icom, ifail] = d02np(neq, ml, mu, icom);

  % Set initial values
  rtol = [1e-3; 1e-3; 1e-3];
  atol = [1e-6; 1e-6; 1e-6];
  y    = [1; 0; 0];
  ydot = zeros(neq,1);
  t    = 0;
  tout = 0.02;

  % Use the user parameter to pass the band dimensions through to jac.
  % An alternative would be to hard code values for ml and mu in jac.
  user = {ml, mu};

  fprintf('\n    t              y\n');
  fprintf('%8.4f   %12.6f %12.6f %12.6f\n', t, y);

  itask = nag_int(0);
  % Obtain the solution at 5 equally spaced values of T.
  for j = 1:5
    if ifail == 0
      [t, y, ydot, rtol, atol, itask, icom, com, user, ifail] = ...
      d02ne(...
            t, tout, y, ydot, rtol, atol, itask, @res1, @jac1, ...
            icom, com, 'user', user);
      fprintf('%8.4f   %12.6f %12.6f %12.6f\n', t, y);
      tout = tout + 0.02;
      icom = d02mc(icom);
    end
  end

  fprintf('\nThe integrator completed task, ITASK = %d\n', itask);

function ex2

  % Setup
  neq = nag_int(1);
  maxord = nag_int(5);
  jceval = 'Analytic';
  hmax   = 0;
  h0     = 0;
  itol   = nag_int(1);
  lcom   = nag_int(200);
  [icom, com, ifail] = d02mw(...
                              neq, maxord, jceval, hmax, h0, itol, lcom);

  % Set initial values
  rtol = [0];
  atol = [1e-8];
  t      = 0;
  tout = 0.2;
  y = [2];
  ydot = [0];
  fprintf('\n    t            y\n');
  fprintf('%8.4f   %12.6f\n', t, y);

  itask = nag_int(0);
  % Obtain the solution at 5 equally spaced values on continuation path.
  for j = 1:5
    if ifail == 0
      [t, y, ydot, rtol, atol, itask, icom, com, user, ifail] = ...
      d02ne(...
            t, tout, y, ydot, rtol, atol, itask, @res2, @jac2, ...
            icom, com);
      fprintf('%8.4f   %12.6f\n', t, y);
      tout = tout + 0.2;
      icom = d02mc(icom);
    end
  end
  fprintf('\nThe integrator completed task, ITASK = %d\n', itask);

function [pd, user] = jac1(neq, t, y, ydot, pd, cj, user)
  ml = user{1};
  mu = user{2};

  stride = 2*ml+mu+1;
  % Main diagonal pdfull(i,i), i=1,neq
  md = mu + ml + 1;
  pd(md) = -0.04 - cj;
  pd(md+stride) = -1.0e4*y(3) - 6.0e7*y(2) - cj;
  pd(md+2*stride) = -cj;
  % 1 sub-diagonal pdfull(i+1:i), i=1,neq-1
  ms = md + 1;
  pd(ms) = 0.04;
  pd(ms+stride) = 6.0e7*y(2);
  % First super-diagonal pdfull(i-1,i), i=2, neq
  ms = md - 1;
  pd(ms+stride) = 1.0e4*y(3);
  pd(ms+2*stride) = -1.0e4*y(2);
  % Second super-diagonal pdfull(i-2,i), i=3, neq
  ms = md - 2;
  pd(ms+2*stride) = 1.0e4*y(2);

function [r, ires, user] = res1(neq, t, y, ydot, ires, user)
  r = zeros(neq, 1);
  r(1) = -0.04*y(1) + 1.0e4*y(2)*y(3)                   - ydot(1);
  r(2) =  0.04*y(1) - 1.0e4*y(2)*y(3) - 3.0e7*y(2)*y(2) - ydot(2);
  r(3) =                                3.0e7*y(2)*y(2) - ydot(3);

function [pd, user] = jac2(neq, t, y, ydot, pd, cj, user)
  pd(1) = -2*y(1) + 0.1*t*y(1)*exp(y(1));

function [r, ires, user] = res2(neq, t, y, ydot, ires, user)
  r(1) = 4 - y(1)^2 + t*0.1*exp(y(1));
