function d02pu_example

% diary d02pu_example.r
fprintf('d02pu example results\n\n');

% Set initial conditions and input
n      = nag_int(4);
method = nag_int(3);
tstart = 0;
tend   = 3*pi;
ecc    = 0.7;
yinit  = [1-ecc; 0; 0; sqrt((1+ecc)/(1-ecc))];
hstart = 0;
thresh = [1e-10; 1e-10; 1e-10; 1e-10];
npts   = 96;
ygot   = zeros(n, 1);
ymax   = zeros(n, 1);


% Set control for output
tol    = 1e-6;
tinc   = (tend-tstart)/npts;



% Call setup function
[iwsav, rwsav, ifail] = d02pq( ...
                              tstart, tend, yinit, tol, thresh, method);

fprintf('\nCalculation with TOL = %8.1e\n\n', tol);
fprintf('    t         y1         y2         y3         y4\n');
fprintf(' %6.3f   %8.4f   %8.4f   %8.4f   %8.4f\n', tstart, yinit);

twant = tend;

ifail = nag_int(3);
while ifail == 2 || ifail == 3 || ifail == 4
  [tgot, ygot, ypgot, ymax, user, iwsav, rwsav, ifail] = ...
    d02pe(@f, twant, ygot, ymax, iwsav, rwsav);
end


% Print Solution
fprintf(' %6.3f   %8.4f   %8.4f   %8.4f   %8.4f\n', tgot, ygot);

% Compute and print error estimates.
[rmserr, errmax, terrmx, iwsav, rwsav, ifail] = d02pu(n, iwsav, rwsav);

fprintf('\nComponentwise error assessment\n');
fprintf('%9.2e ', rmserr);
fprintf('\nMaximum global error, at t = %6.3f, was %9.2e\n', terrmx, errmax);

[fevals, stepcost, waste, stepsok, hnext, iwsav, ifail] = ...
    d02pt(iwsav, rwsav);
fprintf('\nCost of the integration in evaluations of f is %d\n', fevals);

% diary off

function [yp, user] = f(t, n, y, user)
  r = sqrt(y(1)^2+y(2)^2);
  yp = [y(3); y(4); -y(1)/r^3; -y(2)/r^3];
