function d02qf_example

% diary d02qf_example.r
fprintf('d02qf example results\n\n');

% Initialize parameters for Adams method setup
neqf   = nag_int(2);
neqg   = nag_int(2);
vectol = true;
atol   = [1e-06;  1e-06];
rtol   = [0.0001; 0.0001];
onestp = false;
crit   = true;
tcrit  = 10;
hmax   = 0;
maxstp = nag_int(0);
alterg = false;
sophst = true;

lrwork = 23*(1+neqf) + 14*neqg;
rwork  = zeros(lrwork,1);
liwork = 21 + 4*neqg;
iwork  = zeros(liwork, 1, nag_int_name);

% Adams method setup
[statef, alterg, rwork, iwork, ifail] = ...
  d02qw(...
        'Start', neqf, vectol, atol, rtol, onestp, crit, tcrit, hmax, ...
        maxstp, neqg, alterg, sophst, rwork, iwork);

% Adams method integration
t      = 0;
y      = [0; 1];
tout   = 10;
root   = true;

while (t<tout && root)
  [t, y, root, rwork, iwork, ifail] = ...
    d02qf(...
           @fcn, t, y, tout, @g, neqg, rwork, iwork);

  if (root)
    [yp, tcurr, hlast, hnext, odlast, odnext, nsucc, nfail, tolfac, ...
     badcmp, ifail] = d02qx(...
                             neqf, rwork, iwork);

    % Display solution and root statistics.
    [index, itype, events, resids, ifail] = ...
      d02qy(...
            neqg, rwork, iwork);

    fprintf('\nRoot at %13.5e\n',t);
    fprintf('for event equation %2d with type %3d and residual %13.5e\n',...
           index, itype, resids(index))
    fprintf(' y(1) = %13.5e  y''(1) = %13.5e\n',y(1), yp(1));
    ind = find(events);
    for i = ind
      if i~=index
        fprintf('and also for event equation %2d with type %3d',i, events(i));
        fprintf(' and residual %13.5e\n',resids(i));
      end
    end
  end
end

% diary off

function f = fcn(neqf, x, y)
  f=zeros(neqf,1);
  f(1)=y(2);
  f(2)=-y(1);

function result = g(neqf, x, y, yp, k)
  if (k == 1)
    result = yp(1);
  else
    result = y(1);
  end
