function d02ra_example

% diary d02ra_example.r
fprintf('d02ra example results\n\n');

% Initialize variables and arrays.
n      = 3;
np     = nag_int(17);
mnp    = nag_int(40);
numbeg = nag_int(2);
nummix = nag_int(0);
tol    = 0.0001;
init   = nag_int(0);
x      = zeros(mnp,1);
x(np)  = 10;
y      = zeros(n,mnp);
ijac   = nag_int(1);
deleps = 0.1;

% Call NAG routine.
[np, x, y, abt, deleps, ifail] = ...
  d02ra(...
        np, numbeg, nummix, tol, init, x, y, @fcn, @g, ijac, @jacobf, ...
        @jacobg, deleps, @jaceps, @jacgep);

% Output results.
fprintf(['Calculation using analytic Jacobians \n\n'], ...
        ['Solution on final mesh of %4d points\n\n'], np);
fprintf('     x         y_1       y_2      y_3\n');
for j = 1:np
  fprintf('%10.4f',x(j),y(1:n,j));
  fprintf('\n');
end
fprintf('\n   Maximum estimated error by components\n');
fprintf('%12.2e', abt);
fprintf('\n');
% Plot results.
fig1 = figure;
display_plot(x(1:np), y(:,1:np))
% print(fig1,'-dpng','-r75','d02ra_fig1.png');
% print(fig1,'-deps','-r75','d02ra_fig1.eps');

% diary off

function f = fcn(x, eps, y, n)
  % Evaluate derivative vector.
  f = zeros(n,1);
  f(1) = y(2);
  f(2) = y(3);
  f(3) = -y(1)*y(3) - 2*(1-y(2)^2)*eps;

function bc = g(eps, ya, yb, n)
  % Evaluate the boundary conditions.
  bc = zeros(n,1);
  bc(1) = ya(1);
  bc(2) = ya(2);
  bc(3) = yb(2) - 1;

function f = jaceps(x, eps, y, n)
  % Evaluate the derivative w.r.t. eps.
  f = zeros(n,1);
  f(1) = 0;
  f(2) = 0;
  f(3) = -2*(1-y(2)^2);

function bcep = jacgep(eps, ya, yb, n)
  % Evaluate the derivatives of the boundary conditions w.r.t. eps.
  bcep = zeros(n,1);

function f = jacobf(x, eps, y, n)
  % Evaluate the Jacobian.
  f = zeros(n,n);
  f(1,2) = 1;
  f(2,3) = 1;
  f(3,1) = -y(3);
  f(3,2) = 4*y(2)*eps;
  f(3,3) = -y(1);

function [aj, bj] = jacobg(eps, ya, yb, n)
  % Evaluate the Jacobian for the boundary conditions.
  aj = zeros(n,n);
  bj = zeros(n,n);
  aj(1,1) = 1;
  aj(2,2) = 1;
  bj(3,2) = 1;

function display_plot(x, y)
  % Plot two of the curves, then add the other one.
  [haxes, hline1, hline2] = plotyy(x, y(1,:), x, y(2,:));
  % We want the third curve to be plotted on the right-hand y-axis.
  axes(haxes(2));
  hold on
  hline3 = plot(x, y(3,:));
  % Set the axis limits and the tick specifications to beautify the plot.
  set(haxes(1), 'YLim', [0 10]);
  set(haxes(1), 'YMinorTick', 'on');
  set(haxes(1), 'YTick', [0:2:10]);
  set(haxes(2), 'YLim', [0 2]);
  set(haxes(2), 'YMinorTick', 'on');
  set(haxes(2), 'YTick', [0:0.4:2]);
  for iaxis = 1:2
    % These properties must be the same for both sets of axes.
    set(haxes(iaxis), 'XLim', [0 10]);
    set(haxes(iaxis), 'XTick', [0:2:10]);
  end
  set(gca, 'box', 'off'); 
  % Add title.
  title('Solution of Third-order BVP');
  % Label the axes.
  xlabel('x');
  ylabel(haxes(1), 'y');
  ylabel(haxes(2), 'y'' and y''''');
  % Add a legend.
  legend('y''','y''''','y','Location','North')
  % Set some features of the three lines.
  set(hline1,'Linewidth',0.25,'Marker','+','LineStyle','-','Color','red');
  set(hline2,'Linewidth',0.25,'Marker','x','LineStyle','--','Color','green');
  set(hline3,'Linewidth',0.25,'Marker','*','LineStyle',':','Color','blue');
