function d03ee_example

% d03ee example: Solve elliptic equation by discretising using d02ee
%                and solving the disretised system using d03ed.

% diary d03ee_example.r
fprintf('d03ee example results\n\n');

%  u= sin(kx)sin(ly)
global k l;
k = 4; l = 5;

% Discretise on unit square usind d03ee
xmin = 0; xmax = 1;
ymin = 0; ymax = 1;
ngx = nag_int(33); ngy = ngx;
scheme = 'Upwind';
[a, rhs, ifail] = ...
d03ee( ...
       xmin, xmax, ymin, ymax, @pdef, @bndy, ngx, ngy, scheme);

% Solve using d03ed
maxit = nag_int(100); acc = 0.0001; iout = nag_int(0);
ub = zeros(ngx*ngy, 1);
[a, rhs, ub, resid, u, numit, ifail] = ...
d03ed( ...
       ngx, ngy, a, rhs, ub, maxit, acc, iout);

fprintf('Number of iterations to solution = %2d.\n', numit);
fprintf('Maximum residual                   %8.1e\n',resid(numit));

% plot solution
u2 = u(1:ngx*ngy);
umat = reshape(u2,ngx,ngy);
hx = 1/double(ngx+1); 
hy = hx;
x(1:ngx) = hx:hx:1-hx;
y(1:ngy) = hy:hy:1-hy;
[xs,ys] = meshgrid(y,x);

fig1 = figure;
meshc(xs,ys,umat);

xlabel('x'); ylabel('y'); zlabel ('u(x,y)');
exact = sprintf('u = sin %dx sin %dy',k,l);
title({'d03ee example:  U_{xx}+U_{yy}+50(U_x+U_y) = f;',exact});
view(-24,28);
% print(fig1,'-dpng','-r75','d03ee_fig1.png');
% print(fig1,'-deps','-r75','d03ee_fig1.eps');
% diary off
 
function [alpha, beta, gamma, delta, epsilon, phi, psi] = pdef(x,y)
  global k l;
  u = sin(k*x)*cos(l*y);
  ux = k*cos(k*x)*sin(l*y); uy = l*sin(k*x)*cos(l*y);
  uxx = -k*k*u; uyy = -l*l*u;
  uxy = k*l*cos(k*x)*cos(l*y);
  alpha = 1; beta = 0; gamma = 1; delta = 50; epsilon = 50;
  phi = 0;
  psi = alpha*uxx + beta*uxy + gamma*uyy + delta*ux + epsilon*uy + phi*u;


function [a, b, c] = bndy(x, y, ibnd)
  global k l;
  u = sin(k*x)*sin(l*y);
  if (ibnd == 2 || ibnd == 1)
    % Solution prescribed
    a = 1;
    b = 0;
    c = u;
  elseif (ibnd == 0)
    % Derivative prescribed: uy(y=0) = l*sin(k*x)
    a = 0;
    b = 1;
    c = -l*sin(k*x);
  elseif (ibnd == 3)
    % Derivative prescribed: ux(x=0) = k*sin(l*y)
    a = 0;
    b = 1;
    c = -k*sin(l*y);
  end
