function d03fa_example

% diary d03fa_example.r
fprintf('d03fa example results\n\n');

xs = 0; xf = 1;    l = 16;
ys = 0; yf = 2*pi; m = 32;
zs = 0; zf = pi/2; n = 20;

bdxs = zeros(m+1, n+1); bdxf = zeros(m+1, n+1);
bdys = zeros(l+1, n+1); bdyf = zeros(l+1, n+1);
bdzs = zeros(l+1, m+1); bdzf = zeros(l+1, m+1);

lbdcnd = nag_int(1); 
mbdcnd = nag_int(0); 
nbdcnd = nag_int(2); 
lambda = -2;

% Define the grid points for later use.
dx = (xf-xs)/l;  x  = [xs:dx:xf];
dy = (yf-ys)/m;  y  = [ys:dy:yf];
dz = (zf-zs)/n;  z  = [zs:dz:zf];

% Define the array of derivative boundary values (z only here).
XY = [x.^4]'*sin(y);
bdzf = -XY;

% Define the f array including boundary conditions.
YZ = [sin(y)]'*cos(z);
for i = 1:l
  f(i,:,:) = (4*x(i)^2*(3-x(i)^2))*YZ;
  g(i,:,:) = x(i)^4*YZ;
end
g(l+1,:,:) = YZ;
f(1,:,:)   = 0;
f(l+1,:,:) = YZ;
f(:,:,1)   = XY;

[u, pertrb, ifail] = d03fa( ...
                            xs, xf, nag_int(l), lbdcnd, bdxs, bdxf, ...
                            ys, yf, nag_int(m), mbdcnd, bdys, bdyf, ...
                            zs, zf, nag_int(n), nbdcnd, bdzs, bdzf, lambda, f);
% calculate error
maxerr   = max(max(max(abs(u - g))));
fprintf('Maximum error in computed solution = %10.3e\n',maxerr);

fig1 = figure;
[xs,ys,zs] = meshgrid(y,x,z);
slice(xs,ys,zs,u,[1.8 5],[0.95],[0.78]);
axis([y(1) y(end) x(1) x(end) 0 2]);
xlabel('y'); ylabel('x'); zlabel('z');
title('Helholtz Equation solution in a box');
view(111, 26);
h = colorbar;
ylabel(h, 'u(x,y,z)')
% print(fig1,'-dpng','-r75','d03fa_fig1.png');
% print(fig1,'-deps','-r75','d03fa_fig1.eps');

% diary off
