function d03ph_example

% diary d03ph_example.r
fprintf('d03ph example results\n\n');

npde   = nag_int(1);
m      = nag_int(0);
ts     = 0.0001;
tout   = 0.2;
x      = [0:0.05:1];
u      = [exp(ts*(1-x))-1,ts];
ncode  = nag_int(1);
xi     = [1];
rtol   = [0.0001];
atol   = [0.0001];
itol   = nag_int(1);
normt  = 'A';
laopt  = 'F';
algopt = zeros(30,1);
rsave  = zeros(4000, 1);
isave  = zeros(100, 1, nag_int_name);
itask  = nag_int(1);
itrace = nag_int(0);
ind    = nag_int(0);
cwsav  = {''; ''; ''; ''; ''; ''; ''; ''; ''; ''};
lwsav  = false(100, 1);
iwsav  = zeros(505, 1, nag_int_name);
rwsav  = zeros(1100, 1);

% Loop over t values
nt = 20;
t = zeros(nt,1);
alpha = (32)^(1/19);
for j = 1:nt
  t(j) = 0.1*alpha^(j-1);
end

for t_i = 1:nt
  tout = t(t_i);
  [ts, u, rsave, isave, ind, user, cwsav, lwsav, iwsav, rwsav, ifail] = ...
  d03ph( ...
         npde, m, ts, tout, @pdedef, @bndary, u, x, ncode, ...
         @odedef, xi, rtol, atol, itol, normt, laopt, algopt, ...
         rsave, isave, itask, itrace, ind, cwsav, lwsav, iwsav, rwsav);

  if mod(t_i+4,6)==0
    fprintf('\nThe solution at t = %7.4f is:\n',ts);
    for j = 0:2
      fprintf('%10s%12s','x','u(x,t)');
    end
    fprintf('\n');
    for i = 1:7
      for j = 0:2
        fprintf('%12.2f%10.4f',x(i+j*7),u(i+j*7));
      end
      fprintf('\n');
    end
  end
  v(:,t_i) = u(1:(end-1));
end

fig1 = figure;
mesh(t,x,v);
xlabel('t');
ylabel('x');
zlabel('u(x,t)');
title('Coupled Parabolic PDE/ODE using Finite-differences and BDF');
% print(fig1,'-dpng','-r75','d03ph_fig1.png');
% print(fig1,'-deps','-r75','d03ph_fig1.eps');

% diary off

function [p, q, r, ires, user] = pdedef(npde, t, x, u, ux, ncode, v, vdot, ...
                                        ires, user)
      p = zeros(npde, npde);
      q = zeros(npde, 1);
      r = zeros(npde, 1);

      p(1,1) = v(1)*v(1);
      r(1) = ux(1);
      q(1) = -x*ux(1)*v(1)*vdot(1);

function [beta, gamma, ires, user] = bndary(npde, t, u, ux, ncode, ...
                                            v, vdot, ibnd, ires, user)
  beta = zeros(npde, 1);
  gamma = zeros(npde, 1);
  beta(1) = 1.0d0;
  if (ibnd == 0)
    gamma(1) = -v(1)*exp(t);
  else
    gamma(1) = -v(1)*vdot(1);
  end

function [f, ires, user] = odedef(npde, t, ncode, v, vdot, nxi, xi, ucp, ...
                                  ucpx, rcp, ucpt, ucptx, ires, user)
  f = zeros(ncode,1);
  if (ires == 1)
    f(1) = vdot(1) - v(1)*ucp(1,1) - ucpx(1,1) - 1 - t;
  elseif (ires == -1)
    f(1) = vdot(1);
  end
