function d03pp_example

% diary d03pp_example.r
fprintf('d03pp example results\n\n');

% Initialize input arguments
npde   = nag_int(1);
m      = nag_int(0);
ts     = 0;
tout   = 0.2;
npts   = 61;
u      = zeros(npts, 1);
dx     = 1/(npts-1);
x      = [0:dx:1];
ncode  = nag_int(0);
xi     = [];
rtol   = [5e-05];
atol   = [5e-05];
itol   = nag_int(1);
normt  = 'A';
laopt  = 'F';
algopt = zeros(30, 1);
remesh = true;
xfix   = [];
nrmesh = nag_int(3);
dxmesh = 0.5;
trmesh = 0;
ipminf = nag_int(0);
rsave  = zeros(5018, 1);
isave  = zeros(25, 1, nag_int_name);
itask  = nag_int(1);
itrace = nag_int(0);
ind    = nag_int(0);
cwsav  = {''; ''; ''; ''; ''; ''; ''; ''; ''; ''};
lwsav  = false(100, 1);
iwsav  = zeros(505, 1, nag_int_name);
rwsav =  zeros(1100, 1);

% Integrate from t=0 to t=1 in steps of 0.1
usol = zeros(11,npts);
xsol = zeros(11,npts);
tsol = zeros(11,npts);
tout = [0.01,0.02,0.04,0.06,0.08,0.1,0.2,0.3,0.4,0.7,1.0];
for j = 1:11
  [ts, u, x, rsave, isave, ind, user, cwsav, lwsav, iwsav, rwsav, ifail] = ...
  d03pp( ...
         npde, m, ts, tout(j), @pdedef, @bndary, @uvinit, ...
         u, x, ncode, @odedef, xi, rtol, atol, itol, normt, laopt, ...
         algopt, remesh, xfix, nrmesh, dxmesh, trmesh, ipminf, @monitf, ...
         rsave, isave, itask, itrace, ind, cwsav, lwsav, iwsav, rwsav);

  xsol(j,:) = x;
  tsol(j,:) = ts;
  usol(j,:) = u;

end

fprintf(' Number of time steps           = %6d\n',isave(1));
fprintf(' Number of function evaluations = %6d\n',isave(2));
fprintf(' Number of Jacobian evaluations = %6d\n',isave(3));
fprintf(' Number of iterations           = %6d\n',isave(5));

fig1=figure;
mesh(xsol,tsol,usol);
title('Burger''s Equation solution using moving mesh');
xlabel('x');
ylabel('t');
zlabel('u(x,t)');
view(22,24);
% print(fig1,'-dpng','-r75','d03pp_fig1.png');
% print(fig1,'-deps','-r75','d03pp_fig1.eps');

% diary off

function [p, q, r, ires, user] = pdedef(npde, t, x, u, ux, ncode, v, vdot, ...
                                        ires, user)
  p = zeros(npde, npde);
  q = zeros(npde, 1);
  r = zeros(npde, 1);

  p(1,1) = 1.0d0;
  r(1) = 0.005*ux(1);
  q(1) = u(1)*ux(1);


function [beta, gamma, ires, user] = bndary(npde, t, u, ux, ncode, v, ...
                                            vdot, ibnd, ires, user)
  beta = zeros(npde, 1);
  gamma = zeros(npde, 1);

  if (ibnd == 0)
     ue = exact(t,1,[0]);
  else
     ue = exact(t,1,[1]);
  end
  gamma(1) = u(1) - ue(1);

function [u, v, user] = uvinit(npde, npts, nxi, x, xi, ncode, user)
  u = zeros(npde, npts);
  v = zeros(ncode, 1);

  u(1,1:npts) = exact(0,npts,x);

function [f, ires, user] = odedef(npde, t, ncode, v, vdot, nxi, xi, ...
                                  ucp, ucpx, rcp, ucpt, ucptx, ires, user)
  f = zeros(ncode, 1);

function [fmon, user] = monitf(t, npts, npde, x, u, r, user)
  fmon = zeros(npts, 1);

  drdx = (r(2)-r(1))/(x(2)-x(1));
  fmon(1) = abs(drdx); 
  for i = 2:npts-1
    % second derivative ..
    drdx  = 2*(r(1,i+1)-r(1,i))/(x(i+1)-x(i-1));
    fmon(i) = abs(drdx);
  end
  fmon(npts) = fmon(npts-1);

function ue = exact(t,n,x);
  e = 0.005;
  ue = zeros(n,1);
  for i = 1:n
    a = (x(i)-0.25-0.75*t)/(4*e);
    b = (0.9*x(i)-0.325-0.495*t)/(2*e);
    if (a>0 && a>b)
      a = exp(-a);
      c = (0.8*x(i)-0.4-0.24*t)/(4*e);
      c = exp(c);
      ue(i) = (0.5+0.1*c+a)/(1+c+a);
    elseif (b>0 && b>=a)
      b = exp(-b);
      c = (-0.8*x(i)+0.4+0.24*t)/(4*e);
      c = exp(c);
      ue(i) = (0.1+c/2+b)/(1+c+b);
    else
      a = exp(a);
      b = exp(b);
      ue(i) = (1+a/2+0.1*b)/(1+a+b);
    end
  end
