function d03ps_example

% diary d03ps_example.r
fprintf('d03ps example results\n\n');


global usav1 usav2;
fprintf(' Example 1\n\n');
ex1;
% Plot results.
fig1 = figure;
ex1_plot;
% print(fig1,'-dpng','-r75','d03ps_fig1.png');
% print(fig1,'-deps','-r75','d03ps_fig1.eps');
fprintf('\n\n\n Example 2\n\n');
ex2;
% Plot results.
fig2 = figure;
ex2_plot;
% print(fig2,'-dpng','-r75','d03ps_fig2.png');
% print(fig2,'-deps','-r75','d03ps_fig2.eps');

% diary off

function ex1
  % First d03ps example.  Advection and diffusion of a cloud of material.
  global xsav1 tsav1 usav1;

  % Set values for problem parameters.
  ncode  = nag_int(0);
  nxi    = nag_int(0);
  npde   = nag_int(1);

  % Number of points on calculation mesh, and on interpolated mesh.
  npts   = 61;

  neqn   = npde*npts + ncode;
  lisave = 25 + neqn;
  lrsave = 5000;

  % Define some arrays.
  algopt = zeros(30, 1);
  u      = zeros(npde, npts);
  x      = [0:1/(npts-1):1];
  rsave  = zeros(lrsave, 1);
  isave  = zeros(lisave, 1, nag_int_name);

  itrace = nag_int(0);
  itol   = nag_int(1);
  norm_p = '1';
  atol   = [0.0001];
  rtol   = [0.0001];

  xfix   = [];

  remesh = true;
  nrmesh = nag_int(3);
  dxmesh = 0.0;
  trmesh = 0.0;
  con    = 2.0/(npts-1.0);
  xratio = 1.5;
  ipminf = nag_int(0);
  xi     = [0.0];
  laopt  = 'B';

  % b.d.f. integration.
  algopt = zeros(30,1);
  algopt(1)  = 1.0;
  algopt(13) = 0.005;

  % We run through the calculation twice - once to collect the results
  % for plotting, and once to do interpolation to a coarser grid and
  % and output the results.  Set ind = 0 to (re)start the integration
  % in time, and set time to its initial value. Set itask = 2, so that
  % the integrator takes one time-step at a time.
  ind   = nag_int(0);
  ts    = 0.0;
  itask = nag_int(2);
  tout  = 0.3;
  tnext = 0.0;

  fprintf(' npts = %4d atol = %10.3e  rtol = %10.3e \n\n', npts, atol, rtol);
  isav  = 0;
  while ts < tout
    [ts, u, x, rsave, isave, ind, ifail] = ...
    d03ps( ...
           npde, ts, tout, @ex1_pdedef, @ex1_numflx, @ex1_bndary, ...
           @ex1_uvinit, u, x, ncode, 'd03pek', xi, rtol, atol, itol, ...
           norm_p, laopt, algopt, remesh, xfix, nrmesh, dxmesh, ...
           trmesh, ipminf, @ex1_monitf, rsave, isave, itask, itrace, ...
           ind, 'nxi', nxi, 'xratio', xratio, 'con', con);
    
    if ts >= tnext
      % Save this timestep, and this set of results.  Note we have to
      % save the x coordinates at this timestep as well, because the
      % algorithm is automatically remeshing the spatial grid.
      isav = isav+1;
      tsav1(isav) = ts;
      for i=1:npts
        usav1(isav,i) = u(1,i);
        xsav1(isav,i) = x(i);
      end
      tnext = tnext + 0.01;
    end
  end

  % Now prepare to run through the calculation again.  First, set up
  % the points on the interpolation grid.
  intpts = 7;
  for i = 0:intpts - 1
      xinterp(i+1) = 0.2 + i*0.1;
  end

  % Set ind = 0 to (re)start the integration in time, set the time to its
  % initial value, and set itask = 1 to compute solution at t=tout by
  % overshooting & interpolating. Reset the calculation mesh, which had been
  % modified by the earlier calls to d03ps.
  ind   = nag_int(0);
  itask = nag_int(1);
  ts = 0.0;
  dx = 1/(npts-1);
  x = [0:dx:1];

  for it = 1:3
    tout = 0.1*it;
    [ts, u, x, rsave, isave, ind, ifail] = ...
    d03ps( ...
           npde, ts, tout, @ex1_pdedef, @ex1_numflx, @ex1_bndary, ...
           @ex1_uvinit, u, x, ncode, 'd03pek', xi, rtol, atol, itol, ...
           norm_p, laopt, algopt, remesh, xfix, nrmesh, dxmesh, trmesh, ...
           ipminf, @ex1_monitf, rsave, isave, itask, itrace, ...
           ind, 'nxi', nxi, 'xratio', xratio, 'con', con);

    fprintf(' t = %6.3f\n', ts);
    fprintf(' x       ');
    fprintf('%9.4f', xinterp);
    fprintf('\n');

    % Call d03pz to do interpolation of results onto coarser grid.
    m = nag_int(0);
    itype = nag_int(1);
    [uinterp, ifail] = d03pz( ...
                              m, u, x, xinterp, itype);
    fprintf(' approx u');
    fprintf('%9.4f', uinterp);
    fprintf('\n\n');
  end

  % Output some statistics.
  fprintf([' Number of integration steps in time = %6d\n', ...
           ' Number of function evaluations = %6d\n', ...
           ' Number of Jacobian evaluations = %6d\n', ...
           ' Number of iterations = %6d\n'], ...
          isave(1), isave(2), isave(3), isave(5));

function [p, c, d, s, ires] = ex1_pdedef(npde, t, x, u, ux, ncode, v, ...
                                         vdot, ires)
  p = zeros(npde, npde);
  c = zeros(npde, 1);
  d = zeros(npde, 1);
  s = zeros(npde, 1);

  p(1,1) = 1;
  c(1) = 0.2d-2;
  d(1) = ux(1);


function [flux, ires] = ex1_numflx(npde, t, x, ncode, v, uleft, uright, ires)
  flux = zeros(npde, 1);
  flux(1) = uleft(1);


function [g, ires] = ex1_bndary(npde, npts, t, x, u, ncode, v, vdot, ibnd, ires)
  g = zeros(npde, 1);

  if (ibnd == 0)
    g(1) = u(1,1);
  else
    g(1) = u(1,npts);
  end


function [u, v] = ex1_uvinit(npde, npts, nxi, x, xi, ncode)
  u = zeros(npde, npts);
  v = zeros(ncode, 1);

  for i = 1:double(npts)
    if (x(i) > 0.2 && x(i) <= 0.4)
      u(1,i) = sin(pi*(5*x(i)-1));
    end
  end


function [r, ires] = ex1_odedef(npde, t, ncode, v, vdot, nxi, xi, ucp, ucpx, ...
                                ucpt, ires)
  r = zeros(ncode, 1);


function [fmon] = ex1_monitf(t, npts, npde, x, u)
  fmon = zeros(npts, 1);

  for i = 2:double(npts) - 1
    h1 = x(i) - x(i-1);
    h2 = x(i+1) - x(i);
    h3 = (x(i+1)-x(i-1))/2;
    % second derivatives ..
    fmon(i) = abs(((u(1,i+1)-u(1,i))/h2-(u(1,i)-u(1,i-1))/h1)/h3);
  end
  fmon(1) = fmon(2);
  fmon(npts) = fmon(double(npts)-1);


function ex2
  % Second d03ps example. Linear advection equation w/ non-linear source term.
  global xsav2 tsav2 usav2;

  % Set values for problem parameters.
  ncode = nag_int(0);
  nxi   = nag_int(0);
  npde  = nag_int(1);

  % Number of points on calculation mesh, and on interpolated mesh.
  npts = 61;

  neqn   = npde*npts + ncode;
  lisave = 25 + neqn;
  lrsave = 5000;

  % Define some arrays.
  algopt = zeros(30, 1);
  u = zeros(npde, npts);
  x = zeros(npts, 1);
  rsave = zeros(lrsave, 1);
  isave = zeros(lisave, 1, nag_int_name);

  itrace = nag_int(0);
  itol = nag_int(1);
  norm_p = '1';
  atol = [0.0005];
  rtol = [0.05];

  % Initialize calculation mesh.
  dx = 1/(npts-1);
  x = [0:dx:1];
  xfix = [];

  % Set remesh parameters.
  remesh = true;
  nrmesh = nag_int(5);
  dxmesh = 0;
  trmesh = 0;
  con = dx;
  xratio = 1.5;
  ipminf = nag_int(0);
  xi = [0.0];
  laopt = 'B';

  % b.d.f. integration.
  algopt = zeros(30,1);
  % Theta integration.
  algopt(1) = 2.0;
  algopt(6) = 2.0;
  algopt(7) = 2.0;
  % Maximum time step.
  algopt(13) = 0.0025;

  % We run through the calculation twice - once to collect the results for
  % plotting, and once to do interpolation to a coarser grid and and output
  % the results.  Set ind = 0 to (re)start the integration in time, and set
  % the time to its initial value. Set itask = 2, tp take one time-step at
  % a time.
  ind = nag_int(0);
  ts = 0.0;
  itask = nag_int(2);
  tout = 0.4;
  tnext = 0.0;

  isav = 0;
  while ts < tout
    [ts, u, x, rsave, isave, ind, ifail] = ...
    d03ps( ...
           npde, ts, tout, @ex2_pdedef, @ex2_numflx, @ex2_bndary, ...
           @ex2_uvinit, u, x, ncode, 'd03pek', xi, rtol, atol, itol, ...
           norm_p, laopt, algopt, remesh, xfix, nrmesh, dxmesh, trmesh, ...
           ipminf, @ex2_monitf, rsave, isave, itask, itrace, ...
           ind, 'nxi', nxi, 'xratio', xratio, 'con', con);
    
    if ts >= tnext
      % Save this timestep, and this set of results.  Note we have to
      % save the x coordinates at this timestep as well, because the
      % algorithm is automatically remeshing the spatial grid.
      isav = isav+1;
      tsav2(isav) = ts;
      usav2(isav,1:npts) = u(1,:);
      xsav2(isav,1:npts) = x;
      tnext = tnext + 0.01;
    end
  end

  % Now prepare to run through the calculation again.  First, set up
  % the points on the interpolation grid.
  intpts = 7;
  xinterp = [0.2:0.1:0.8];

  % We set ind = 0 to (re)start the integration in time, set the time to its
  % initial value, and set itask = 1 to compte solution at t=tout by
  % overshooting & interpolating.
  % We also need to reset the calculation mesh, since this has been modified
  % by the earlier calls to d03ps.
  ind   = nag_int(0);
  itask = nag_int(1);
  ts = 0;
  dx = 1/(npts-1);
  x = [0:dx:1];

  for it = 1:2
    tout = 0.2*it;
    [ts, u, x, rsave, isave, ind, ifail] = ...
    d03ps( ...
           npde, ts, tout, @ex2_pdedef, @ex2_numflx, @ex2_bndary, ...
           @ex2_uvinit, u, x, ncode, 'd03pek', xi, rtol, atol, itol, ...
           norm_p, laopt, algopt, remesh, xfix, nrmesh, dxmesh, trmesh, ...
           ipminf, @ex2_monitf, rsave, isave, itask, itrace, ...
           ind, 'nxi', nxi, 'xratio', xratio, 'con', con);

    if it == 1
      fprintf(' npts = %4d atol = %10.3e  rtol = %10.3e \n\n', npts, ...
              atol, rtol)
    end
    fprintf(' t = %6.3f\n', ts);

    % Call d03pz to do interpolation of results onto coarser grid.
    m = nag_int(0);
    itype = nag_int(1);
    [uinterp, ifail] = d03pz(m, u, x, xinterp, itype);

    % Check against exact solution.
    [ue] = ex2_exact(tout, xinterp, intpts);
    fprintf('        x      approx u    exact u\n');
    for i=1:intpts
      fprintf('%9.4f   %9.4f   %9.4f\n', xinterp(i), uinterp(i), ue(i));
    end
    fprintf('\n\n');
  end

  % Output some statistics.
  fprintf([' Number of integration steps in time = %6d\n', ...
           ' Number of function evaluations = %6d\n', ...
           ' Number of Jacobian evaluations = %6d\n', ...
           ' Number of iterations = %6d\n'], ...
          isave(1), isave(2), isave(3), isave(5));


function [p, c, d, s, ires] = ex2_pdedef(npde, t, x, u, ux, ...
      ncode, v, vdot, ires)
  % Evaluate pij, ci, di, si to partially define the system of PDEs.

  p = zeros(npde, npde);
  c = zeros(npde, 1);
  d = zeros(npde, 1);
  s = zeros(npde, 1);

  p(1,1) = 1;
  c(1) = 0;
  d(1) = 0;
  s(1) = -100*u(1)*(u(1) - 1)*(u(1) - 0.5);

function [fmon] = ex2_monitf(t, npts, npde, x, u)
  % Supplies and evaluates a remesh monitor function.
  persistent icount;
  persistent xa;

  fmon = zeros(npts, 1);

  % Locate shock.
  uxmax = 0;
  xmax = 0;
  for i = 2:npts-1
    h1 = x(i) - x(i-1);
    ux = abs((u(1,i) - u(1,i-1))/h1);
    if (ux > uxmax)
      uxmax = ux;
      xmax = x(i);
    end
  end

  % Assign width (on first call only).
  if (isempty(icount))
    icount = 1;
    xleft = xmax - x(1);
    xright = x(npts) - xmax;
    if (xleft > xright)
      xa = xright;
    else
      xa = xleft;
    end
  end

  % Assign monitor function.
  xl = xmax - xa;
  xr = xmax + xa;
  for i = 1:npts
    if ((x(i) > xl) && (x(i) < xr))
      fmon(i) = 1.0 + cos(pi*(x(i) - xmax)/xa);
    else
      fmon(i) = 0.0;
    end
  end

function [g, ires] = ex2_bndary(npde, npts, t, x, u, ncode, v, ...
                                vdot, ibnd, ires)
  % Evaluate giL and giR to describe the physical & numerical boundary
  % conditions.

  % Solution known to be constant at both boundaries.
  if (ibnd == 0)
    [ue] = ex2_exact(t, x(1), 1);
    g(1) = ue(1,1) - u(1,1);
  else
    [ue] = ex2_exact(t, x(npts), 1);
    g(1) = ue(1,1) - u(1,npts);
  end

function [flux, ires] = ex2_numflx(npde, t, x, ncode, v, uleft, ...
                                   uright, ires)
  % Supplies numerical flux for each PDE given the L & R values of u.
  flux = zeros(npde, 1);
  flux(1) = uleft(1);


function [u, v] = ex2_uvinit(npde, npts, nxi, x, xi, ncode)
  % Specify initial values (at t=t0) of u(x,t) and v(t) for all x.

  u = zeros(npde, npts);
  v = zeros(ncode, 1);
  t = 0.0;
  [u] = ex2_exact(t, x, npts);

function [u] = ex2_exact(t, x, npts)
  % Calculate exact solution (for comparison and use in boundary conditions).

  s = 0.1;
  del = 0.01;
  rm = -1.0/del;
  rn = 1.0 + s/del;

  for i = 1:npts
    psi = x(i) - t;
    if (psi < s)
      u(1,i) = 1.0;
    elseif (psi > (del+s))
      u(1,i) = 0.0;
    else
      u(1,i) = rm*psi + rn;
    end
  end

function ex1_plot
  % Plot the results.
  global xsav1 tsav1 usav1;

  % Plot array as a mesh.
  mesh(xsav1, tsav1, usav1);

  % Label the axes, and set the title.
  xlabel('x');
  ylabel('t');
  zlabel('U(x,t)');
  title('Advection and Diffusion of a Cloud of Material');

  % Set the axes limits tight to the x and y range.
  axis([xsav1(1) xsav1(end) tsav1(1) tsav1(end) 0 1]);
  view(-12, 56);

function ex2_plot
  % Plot the results.
  global xsav2 tsav2 usav2;

  % Plot array as a mesh.
  mesh(xsav2, tsav2, usav2);

  % Label the axes, and set the title.
  xlabel('x');
  ylabel('t');
  zlabel('U(x,t)');
  title('Linear Advection with Non-linear Source Term');

  % Set the axes limits tight to the x and y range.
  axis([xsav2(1) xsav2(end) tsav2(1) tsav2(end) 0 1]);
  view(15, 30);
