function e01sa_example

% diary e01sa_example.r
fprintf('e01sa example results\n\n');

% Scattered Grid Data
x = [11.16; 12.85; 19.85; 19.72; 15.91;  0.00; 20.87;  3.45; 14.26; ...
     17.43; 22.80;  7.58; 25.00;  0.00;  9.66;  5.22; 17.25; 25.00; ...
     12.13; 22.23; 11.52; 15.20;  7.54; 17.32;  2.14;  0.51; 22.69; ...
      5.47; 21.67;  3.31];
y = [ 1.24;  3.06; 10.72;  1.39;  7.74; 20.00; 20.00; 12.78; 17.87; ...
      3.46; 12.39;  1.98; 11.87;  0.00; 20.00; 14.66; 19.57;  3.87; ...
     10.79;  6.21;  8.53;  0.00; 10.69; 13.78; 15.03;  8.37; 19.63; ...
     17.13; 14.36; 0.33];
f = [22.15; 22.11;  7.97; 16.83; 15.30; 34.60;  5.74; 41.24; 10.74; ...
     18.60;  5.47; 29.87;  4.40; 58.20;  4.73; 40.36;  6.43;  8.74; ...
     13.71; 10.25; 15.74; 21.60; 19.31; 12.11; 53.10; 49.43;  3.25; ...
     28.63;  5.52; 44.08];
% Triangulate and obtain details of interpolant
[triang,grads,ifail] = e01sa(x,y,f);

% Convert to for for trimesh plotter.
n       = size(x,1);
[tri,k] = triang2tri(triang,n);
fprintf('Number of triangles in triangulation = %d\n\n',k);


% Display mesh
fig1 = figure;
trimesh(tri(1:k,1:3),x,y,f);
xlabel('x');
ylabel('y');
zlabel('f(x,y)')
title('Triangulation of scattered data using e01sa');
view(51,18);
% print(fig1,'-dpng','-r75','e01sa_fig1.png');
% print(fig1,'-deps','-r75','e01sa_fig1.eps');

% Evaluate interpolant at on regular mesh (px,py)
px = [3:3:21];
py = [2:3:17];
for i = 1:6
  for j = 1:7
    [pf(i,j), ifail] = e01sb( ...
                              x, y, f, triang, grads, px(j), py(i));
  end
end

% Display interpolated values
matrix = 'General';
diag = 'Non-unit';
format = 'F7.2';
mtitl  = 'Spline evaluated on a regular mesh (x across, y down):';
chlab  = 'Character';
rlabs  = cellstr(num2str(py'));
clabs  = cellstr(num2str(px'));
ncols  = nag_int(80);
indent = nag_int(0);
[ifail] =  x04cb( ...
                  matrix, diag, pf, format, mtitl, chlab, ...
                  rlabs, chlab, clabs, ncols, indent);

% diary off

function [tri,k] = triang2tri(triang,n)
%  Convert triang array to form that MATLAB trimesh function can use

max_t = nag_int(2*n-5);
tri   = nag_int(zeros(max_t,3));
iend  = nag_int(0);
k     = iend;
t     = nag_int([0;0;0]);

for i = 1:n
   % set up loop of nodes connected to node i
   ibeg = iend + 1;
   iend = triang(6*n+i);
   % first connection setup
   t(1) = i;
   t(2) = triang(ibeg);
   % loop over remaining connections
   ibeg = ibeg + 1;
   for j = ibeg:iend
      t(3) = triang(j);
      if t(3)>0
        if t(2)>i || t(3)>i
        % new triangle here 
          k = k + 1;
          tri(k,1:3) = t(1:3);
        end
        % shuffle down for next connected node
        t(2) = t(3);
      end
   end
end
