function e02da_example

% diary e02da_example.r
fprintf('e02da example results\n\n');

% Data points
npts = 30;
x = [ 0.60 -0.95  0.87  0.84  0.17 -0.87  1.00  0.10  0.24 -0.77 ...
      0.32  1.00 -0.63 -0.66  0.93  0.15  0.99 -0.54  0.44 -0.72 ...
      0.63 -0.40  0.20  0.43  0.28 -0.24  0.86 -0.41 -0.05 -1.00];
y = [-0.52 -0.61  0.93  0.09  0.88 -0.70  1.00  1.00  0.30 -0.77 ...
     -0.23 -1.00 -0.26 -0.83  0.22  0.89 -0.80 -0.88  0.68 -0.14 ... 
      0.67 -0.90 -0.84  0.84  0.15 -0.91 -0.35 -0.16 -0.35 -1.00];
f = [ 0.93 -1.79  0.36  0.52  0.49 -1.76  0.33  0.48  0.65 -1.82 ...
      0.92  1.00  8.88 -2.01  0.47  0.49  0.84 -2.42  0.47  7.15 ... 
      0.44 -3.34  2.78  0.44  0.70 -6.52  0.66  2.32  1.66 -1.00];
npts = size(x,2);
w    = ones(npts,1);
w(1:6) = 10;

% Fit parameters
px       = 10;
lamda    = zeros(px,1);
lamda(5) = -0.5;
py       = 8;
mu       = zeros(py, 1);
epsilon  = 1e-06;

% sort points into panel order
[point, ifail] = e02za( ...
                        lamda, mu, x, y);

fprintf('Interior x-knots:\n');
if px==8
  fprintf('None');
else
  fprintf('%10.4f',lamda(5:px-4));
end
fprintf('\n\nInterior y-knots:\n');
if py==8
  fprintf('None');
else
  fprintf('%10.4f',mu(5:py-4));
end
fprintf('\n');

% Fit bicubic spline
[lamda, mu, dl, c, sigma, rank, ifail] = ...
    e02da(...
          x, y, f, w, lamda, mu, point, epsilon);

fprintf('\nSum of squares of residual RHS = %16.2e\n\n', sigma);
fprintf('Rank = %5d\n\n',rank);

% evaluate spline at data points
[ff, ifail] = e02de( ...
                     x, y, lamda, mu, c);

res = ff'-f;
fprintf('Spline evaluated at original data points\n');
fprintf('       x          y          f          fit    residual\n');
fprintf('%11.4f%11.4f%11.4f%11.4f%11.2e\n',[x; y; f; ff'; res]);
wres = res.*w';
sres = dot(wres,wres);
fprintf('\nSum of squared weighted residuals = %16.2e\n\n',sres)
fprintf('Spline coefficients:\n');
c = reshape(c,[px-4,py-4]);
disp(c);

% plot on regular mesh
xm = [-0.95:0.05:0.95]; ym = xm;
nn = size(xm,2);
for i = 1:nn
  xx = xm(i)*ones(1,nn);
  [fit(:,i), ifail] = e02de( ...
                             xx, ym, lamda, mu, c);
end

fig1 = figure;
meshc(xm,ym,fit);
title('Least-squares bi-cubic spline fit');
xlabel('y');
ylabel('x');
zlabel('spline fit');

% print(fig1,'-dpng','-r75','e02da_fig1.png');
% print(fig1,'-deps','-r75','e02da_fig1.eps');

% diary off
