function e05us_example

% diary e05us_example.r
fprintf('e05us example results\n\n');

npts   = nag_int(1);
repeat = true;
nclin  = nag_int(1);
ncnln  = nag_int(1);
nb     = nag_int(1);
n      = nag_int(2);

bl = [0.4; -4.0; 1.0; 0.0];
bu = [1e25; 1e25; 1e25; 1e25];

a = [1,    1];
x = [0.4;  0];
y = [0.49; 0.49; 0.48; 0.47; 0.48; 0.47; 0.46; 0.46; 0.45; 0.43; 0.45; ...
     0.43; 0.43; 0.44; 0.43; 0.43; 0.46; 0.45; 0.42; 0.42; 0.43; 0.41 ; ...
     0.41; 0.40; 0.42; 0.40; 0.40; 0.41; 0.40; 0.41; 0.41; 0.40; 0.40 ; ...
     0.40; 0.38; 0.41; 0.40; 0.40; 0.41; 0.38; 0.40; 0.40; 0.39; 0.39];

  % Initialise e05us
iopts = zeros(740, 1, nag_int_name);
opts  = zeros(485, 1);
[iopts, opts, ifail] = e05zk(...
                             'Initialize = e05us', iopts, opts);
[iopts, opts, ifail] = e05zk(...
                             'List', iopts, opts);
[iopts, opts, ifail] = e05zk(...
                             'print level = 10', iopts, opts);

wstat = warning();
warning('OFF');
% Solve the problem
[x, objf, f, fjac, iter, c, cjac, clamda, istate, ...
                        iopts, opts, user, info, ifail] = ...
  e05us(...
        n, ncnln, a, bl, bu, y, @confun, @objfun, ...
        npts, @start, repeat, nb, iopts, opts);

warning(wstat);

if ifail == 8
  l = double(info(nb));
  fprintf('\nOnly %d solutions found\n', info(nb));
else
  l = double(nb);
end
l = min(l,3);

for i=1:l
  fprintf('\nSolution number %d\n\n', i);
  fprintf('e04us returned with ifail = %d\n\n', info(i));

  fprintf(' Variable Istate           Value Lagrange Multiplier\n');
  for j=1:n
    fprintf(' %3d%9d%19.6g%13.4g\n', j, istate(j,i), x(j,i), clamda(j,i));
  end

  if nclin > 0
    ax = a*x(:,i);
    fprintf('\n L Con    Istate           Value Lagrange Multiplier\n');
    for k=n+1:n+nclin
      j=k-n;
      fprintf(' %3d%9d%19.6g%13.4g\n', j, istate(k,i), ax(j), clamda(k,i));
    end
  end

  if ncnln > 0
    fprintf('\n NL Con  Istate            Value Lagrange Multiplier\n');
    for k=n+nclin+1:n+nclin +ncnln
      j=k-n-nclin;
      fprintf(' %3d%9d%19.6g%13.4g\n', j, istate(k,i), c(j,i), clamda(j,i));
    end
  end

  fprintf('\nFinal objective value = %15.7g\n', objf(i));
  fprintf('clamda: ');
  disp(transpose(clamda(1:double(n+nclin +ncnln),i)));
  fprintf('\n ------------------------------------------------------\n');
end

% diary off

function [mode, c, cjsl, user] = confun(mode, ncnln, n, ldcjsl, ...
                                        needc, x, cjsl, nstate, user)
  c = zeros(ncnln, 1);

  if (nstate == 1)
    % First call to confun.  Set all Jacobian elements to zero.
    % Note that this will only work when 'Derivative Level = 3'
    % (the default).
     cjsl(1:double(ncnln),1:double(n)) = 0;
  end

  if ( needc(1) > 0 )

    if (mode==0 || mode==2)
      c(1) = -0.09 - x(1)*x(2) + 0.49*x(2);
    end

    if (mode==1 || mode==2)
      cjsl(1,1) = -x(2);
      cjsl(1,2) = -x(1) + 0.49;
    end

  end


function [mode, f, fjac, user] = objfun(mode, m, n, ldfj, needfi, x, fjac, ...
                                        nstate, user)
  % Evaluate the subfunctions and their 1st derivatives.

  a = [ 8;  8; 10; 10; 10; 10; 12; 12; 12; 12; 14;
       14; 14; 16; 16; 16; 18; 18; 20; 20; 20; 22;
       22; 22; 24; 24; 24; 26; 26; 26; 28; 28; 30;
       30; 30; 32; 32; 34; 36; 36; 38; 38; 40; 42];

  f = zeros(m, 1);

  if needfi > 0
    f(needfi) = x(1) + (0.49-x(1))*exp(-x(2)*(a(needfi)-8.0));
    return;
  end

  temp = exp(-x(2)*(a-8.0));

  if (mode==0 || mode==2)
    f = x(1) + (0.49-x(1))*temp;
  end

  if (mode==1 || mode==2)
    fjac(:,1) = 1.0 - temp;
    fjac(:,2) = -(0.49-x(1)).*(a-8.0).*temp;
  end


function [quas, mode, user] = start(npts, quas, n, repeat, bl, bu, mode, user)
  quas(1, 1) = 0.4;
  quas(2, 1) = 0.0;
