function f08ku_example

% diary f08ku_example.r
fprintf('f08ku example results\n\n');

% Two cases of preceding reduction to bidiagonal form by QR or LQ
% Case 1: m > n, precede by QR
ex1;
% Case 2: m < n, precede by LQ
ex2;

% diary off

function ex1
  m = nag_int(6);
  n = nag_int(4);
  a = [ 0.96 - 0.81i, -0.03 + 0.96i, -0.91 + 2.06i, -0.05 + 0.41i;
       -0.98 + 1.98i, -1.20 + 0.19i, -0.66 + 0.42i, -0.81 + 0.56i;
        0.62 - 0.46i,  1.01 + 0.02i,  0.63 - 0.17i, -1.11 + 0.60i;
       -0.37 + 0.38i,  0.19 - 0.54i, -0.98 - 0.36i,  0.22 - 0.20i;
        0.83 + 0.51i,  0.20 + 0.01i, -0.17 - 0.46i,  1.47 + 1.59i;
        1.08 - 0.28i,  0.20 - 0.12i, -0.07 + 1.23i,  0.26 + 0.26i];

  % Factorize A = QR
  [QR, tau, info] = f08as(a);

  % Generate Q from QR
  [Q, info] = f08at(QR, tau);

  % Extract R from QR
  R = triu(QR(1:n,1:n));

  % Bidiagonalize R = Q1 B P^H
  [B, d, e, tauq, taup, info] = ...
  f08ks(R);

  % Update Q: Q2 = Q*Q1 (so A = QR = Q2 B P^H)
  vect = 'Q';
  side = 'Right';
  trans = 'No transpose';
  [Q2, info] = f08ku( ...
                      vect, side, trans, n, B, tauq, Q);

  fprintf('Example 1: bidiagonal matrix B\n   Main diagonal  ');
  fprintf(' %7.3f',d);
  fprintf('\n   super-diagonal ');
  fprintf(' %7.3f',e);
  fprintf('\n\n');
  disp('Example 1: Orthogonal matrix Q');
  disp(Q2);

function ex2
  m = nag_int(3);
  n = nag_int(4);
  a = [0.28 - 0.36i  0.50 - 0.86i -0.77 - 0.48i  1.58 + 0.66i;
      -0.50 - 1.10i -1.21 + 0.76i -0.32 - 0.24i -0.27 - 1.15i;
       0.36 - 0.51i -0.07 + 1.33i -0.75 + 0.47i -0.08 + 1.01i];

  % Factorize A = LQ
  [LQ, tau, info] = f08av(a);

  % Generate Q from LQ
  [Q, info] = f08aw(LQ, tau);

  % Extract L from LQ
  L = tril(LQ(1:m,1:m));

  % Bidiagonalize L = Q1 B P^H
  [B, d, e, tauq, taup, info] = ...
  f08ks(L);

  % Update Q: P2 = P^H*Q (so A = LQ = Q1 B P2)
  vect = 'P';
  side = 'Left';
  trans = 'Conjugate Transpose';
  [P2, info] = f08ku( ...
                      vect, side, trans, n, B, taup, Q);

  fprintf('Example 2: bidiagonal matrix B\n   Main diagonal  ');
  fprintf(' %7.3f',d);
  fprintf('\n   super-diagonal ');
  fprintf(' %7.3f',e);
  fprintf('\n\n');
  disp('Example 2: Orthogonal matrix P^H');
  disp(P2);
