function f08zs_example

% diary f08zs_example.r
fprintf('f08zs example results\n\n');

a = [ 0.96 - 0.81i, -0.03 + 0.96i, -0.91 + 2.06i;
     -0.98 + 1.98i, -1.20 + 0.19i, -0.66 + 0.42i;
      0.62 - 0.46i,  1.01 + 0.02i,  0.63 - 0.17i;
      1.08 - 0.28i,  0.20 - 0.12i, -0.07 + 1.23i];
b = [ 0.5 - 1i,  0 + 0i,  0 + 0i,  0 + 0i;
      0   + 0i,  1 - 2i,  0 + 0i,  0 + 0i;
      0   + 0i,  0 + 0i,  2 - 3i,  0 + 0i;
      0   + 0i,  0 + 0i,  0 + 0i,  5 - 4i];
d = [ 6.00 - 0.40i;
     -5.27 + 0.90i;
      2.72 - 2.13i;
     -1.30 - 2.80i];
[n,m] = size(a);
p = size(b,2);

% Compute the generalized QR factorization of (A,B) as
% A = Q*(R),   B = Q*(T11 T12)*Z
%       (0)          ( 0  T22)

[a, taua, b, taub, info] = f08zs( ...
                                  a, b);

% Compute c = (c1) = (Q**H)*d
%             (c2)
[c, info] = f08au( ...
                   'Left', 'Conjugate Transpose', a, taua, d);

% Putting Z*y = w = (w1), set w1 = 0, storing the result in y1
%                   (w2)
y = zeros(p, 1);
if n > m

  % Solve T22*w2 = c2 for w2, storing result in y2
  indy = m+p-n+1:p;
  [y(indy), info] = f07ts( ...
                           'Upper', 'No transpose', 'Non-unit', ...
                           complex(b(m+1:n, indy)), c(m+1:n));

  % Compute estimate of the square root of the residual sum of squares
  % norm(y) = norm(w2)
  rnorm = norm(y(indy), 1);

  % Form c1 - T12*w2 in c
  c = c - b(:,indy)*y(indy);
end

% Solve R*x = c1 - T12*w2 for x
[c(1:m), info] = f07ts( ...
                        'Upper', 'No transpose', 'Non-unit', a(1:m,:), c(1:m));

% Compute y = (Z^H)*w
[b(max(1, n-p+1):n,:), y, info] = ...
f08cx( ...
       'Left', 'Conjugate Transpose', b(max(1, n-p+1):n,:), taub, y);

disp('Generalized least squares solution');
disp(c(1:m));

ncols  = nag_int(80);
indent = nag_int(0);
[ifail] = x04db( ...
                 'Gen', ' ', y, 'B', '1P,E10.2', 'Residual vector', ...
                 'N', 'N', ncols, indent);

fprintf('\nSquare root of the residual sum of squares\n%11.2e\n', ...
        rnorm);

% diary off
