function f11dg_example

% diary f11dg_example.r
fprintf('f11dg example results\n\n');

n   = nag_int(9);
nz  = nag_int(33);
a    = zeros(20*nz, 1);
irow = zeros(20*nz, 1, nag_int_name);
icol = zeros(20*nz, 1, nag_int_name);
a(1:nz)    = [64; -20; -20; -12;  64; -20; -20; -12;  64; -20; -12;
              64; -20; -20; -12; -12;  64; -20; -20; -12; -12;  64;
             -20; -12;  64; -20; -12; -12;  64; -20; -12; -12;  64];
irow(1:nz) = [ 1;   1;   1;   2;   2;   2;   2;   3;   3;   3;   4;
               4;   4;   4;   5;   5;   5;   5;   5;   6;   6;   6;
               6;   7;   7;   7;   8;   8;   8;   8;   9;   9;   9];
icol(1:nz) = [ 1;   2;   4;   1;   2;   3;   5;   2;   3;   6;   1;
               4;   5;   7;   2;   4;   5;   6;   8;   3;   5;   6;
               9;   4;   7;   8;   5;   7;   8;   9;   6;   8;   9];

% 3 Blocks
nb     = nag_int(3);
nover  = 1;
lfill  = [nag_int(0); 0; 0];
dtol   = [  0;   0;   0];
pstrat = {'n'; 'n'; 'n'};
milu   = {'n'; 'n'; 'n'};

% Define diagonal block indices.
% In this example use blocks of mb consecutive rows and initialise
% assuming no overlap.
mb    = idivide(n+nb-1, nb);
istb  = zeros(nb+1, 1, nag_int_name);
indb  = zeros(3*n, 1, nag_int_name);
ipivp = zeros(3*n, 1, nag_int_name);
ipivq = zeros(3*n, 1, nag_int_name);
istb(1:nb) = [1:mb:nb*mb];
istb(nb+1) = n+1;
indb(1:n)  = [1:n];

% Modify indb and istb to account for overlap.
[istb, indb, ifail] = f11df_overlap(n, nz, irow, icol, nb, ...
                                    istb, indb, 3*n, nover);
if (ifail == -999)
  error('indb is too small, size of indb = %d', numel(indb));
end

% Calculate Factorisation
[a, irow, icol, ipivp, ipivq, istr, idiag, nnzc, npivm, ifail] = ...
f11df( ...
       n, nz, a, irow, icol, istb, indb, ...
       lfill, dtol, milu, ipivp, ipivq, 'pstrat', pstrat);

% RHS b and initial guess x
b = 100*ones(n,1);
x = zeros(n, 1);

% Solver algorithmic parameters
method = 'bicgstab';
m      = nag_int(2);
tol    = 1e-6;
maxitn = nag_int(100);

% Solve Ax = b using f11dg
[x, rnorm, itn, ifail] = f11dg( ...
                                method, nz, a, irow, icol, nb, istb, indb, ...
                                ipivp, ipivq, istr, idiag, b, m, tol, ...
                                maxitn, x);

fprintf('Converged in %d iterations\n', itn);
fprintf('Final redidual norm = %16.3d\n\n', rnorm);
disp('Solution');
disp(x);

% diary off

function [istb, indb, ifail] = f11df_overlap(n, nz, irow, icol, nb, ...
                                             istb, indb, lindb, nover)

  ifail = 0;

  % This function takes a set of row indices indb defining the diagonal
  % blocks to be used in f11df to define a block Jacobi or additive Schwarz
  % preconditioner, and expands them to allow for nover levels of overlap.
  % The pointer array istb is also updated accordingly, so that the returned
  % values of istb and indb can be passed to f11df to define overlapping
  % diagonal blocks.
  iwork = zeros(3*n+1, 1, nag_int_name);

  % Find the number of non-zero elements in each row of the matrix A, and
  % the start address of each row. Store the start addresses in
  % iwork(n+1,...,2*n+1).
  for k=1:nz
    iwork(irow(k)) = iwork(irow(k)) + 1;
  end
  iwork(n+1) = 1;
  for i = 1:n
    iwork(n+i+1) = iwork(n+i) + iwork(i);
  end

  % Loop over blocks
  for k=1:nb
    % Initialize marker array.
    iwork(1:n) = 0;

    % Mark the rows already in block k in the workspace array.
    for l = istb(k):istb(k+1)-1
      iwork(indb(l)) = 1;
    end

    % Loop over levels of overlap.
    for iover=1:nover
      % Initialize counter of new row indices to be added.
      ind = 0;

      % Loop over the rows currently in the diagonal block.
      for l = istb(k):istb(k+1)-1
        row = indb(l);

        % Loop over non-zero elements in row
        for i = iwork(n+row):iwork(n+row+1)-1

          % If the column index of the non-zero element is not in the
          % existing set for this block, store it to be added later, and
          % mark it in the marker array.
          if (iwork(icol(i))==0)
            iwork(icol(i)) = 1;
            ind = ind + 1;
            iwork(2*n+1+ind) = icol(i);
          end
        end
      end

      % Shift the indices in indb and add the new entries for block k.
      % Change istb accordingly.
      nadd = ind;
      if (istb(nb+1)+nadd-1>lindb) Then
        ifail = -999;
        return;
      end

      for i = istb(nb+1) - 1:-1:istb(k+1)
        indb(i+nadd) = indb(i);
      end
      n21 = 2*n + 1;
      ik = istb(k+1) - 1;
      indb(ik+1:ik+nadd) = iwork(n21+1:n21+nadd);
      istb(k+1:nb+1) = istb(k+1:nb+1) + nadd;
    end
  end
