function f11dr_example

% diary f11dr_example.r
fprintf('f11dr example results\n\n');

% Solve sparse system Ax = b using preconditioned CGS

% Sparse A and b
n    = nag_int(5);
m    = nag_int(2);
nz   = nag_int(16);
sparseA = [ 2  3   1    1;
            1 -1   1    2;
           -1  0   1    4;
            0  2   2    2;
           -2  1   2    3;
            1  0   2    5;
            0 -1   3    1;
            5  4   3    3;
            3 -1   3    4;
            1  0   3    5;
           -2  2   4    1;
           -3  1   4    4;
            0  3   4    5;
            4 -2   5    2;
           -2  0   5    3;
           -6  1   5    5];
a(1:nz)    = sparseA(:,1) + i*sparseA(:,2);
irow(1:nz) = nag_int(sparseA(:,3));
icol(1:nz) = nag_int(sparseA(:,4));
b = [ -3 +  3i;
     -11 +  5i;
      23 + 48i;
     -41 +  2i;
     -28 - 31i];

% Solver setup input argument initialization
method = 'CGS';
precon = 'P';
norm_p = 'I';
tol    = 1e-10;
maxitn = nag_int(1000);
anorm  = 0;
sigmax = 0;
monit  = nag_int(0);
lwork  = max([121+n*(3+m)+m*(m+5),120+7*n,120+(2*n+m)*(m+2)+2*n,120+10*n]);

% Initialize solver
[lwreq, work, ifail] = f11br( ...
                              method, precon, n, m, tol, maxitn, anorm, ...
                              sigmax, monit, lwork, 'norm_p', norm_p);

rdiag = zeros(n, 1);
% Calculate reciprocal diagonal matrix elements for preconditioner
if precon == 'P' || precon == 'p'
   dcount = zeros(n, 1, nag_int_name);

   for j = 1:nz
     if irow(j) == icol(j)
       dcount(irow(j)) = dcount(irow(j)) + 1;
       if a(j) ~= 0
         rdiag(irow(j)) = 1/a(j);
       else
         error('Matrix has a zero diagonal element');
       end
     end
   end

   for j = 1:n
     if dcount(j)==0
       error('Matrix has a missing diagonal element');
     elseif dcount(j) > 1
       error('Matrix has a multiple diagonal element');
     end
   end
end
% Other preconditioner inputs
omega = 1.4;
ckdrf = 'C';

% Solver inputs
irevcm = nag_int(0);
wgt    = zeros(n, 1);
x      = complex(zeros(n, 1));

% matrix-vector input
ckxnf = 'C';

% solve the linear system by revere communication
while (irevcm ~= 4)
  [irevcm, x, b, work, ifail] = f11bs( ...
                                       irevcm, x, b, wgt, work);

  if (irevcm == -1)
    % b = A^T x
    [b, ifail] = f11xn( ...
                        'T', a, irow, icol, ckxnf, x);
    ckxnf = 'N';
  elseif (irevcm == 1)
    % b = Ax
    [b, ifail] = f11xn( ...
                        'N', a, irow, icol, ckxnf, x);
    ckxnf = 'N';
  elseif (irevcm == 2)
    % SSOR preconditioning
    [b, ifail] = f11dr( ...
                        'N', a, irow, icol, rdiag, omega, ckdrf, x);
    ckdrf = 'N';
  end
end

if ifail == 0
   % Successful termination, get solution details from communication array
  [itn, stplhs, stprhs, anorm, sigmax, work, ifail] = ...
    f11bt(work);

  fprintf('Converged in %d iterations\n', itn);
  fprintf('Matrix norm         = %16.3e\n', anorm);
  fprintf('Final residual norm = %16.4e\n\n', stplhs);
  disp('Solution');
  disp(x);

end

% diary off
