function f11jd_example

% diary f11jd_example.r
fprintf('f11jd example results\n\n');

% Solve Ax = b by CG with SSOR preconditioning

% Matrix A and RHS b

n    = nag_int(7);
nz   = nag_int(16);
a = zeros(nz, 1);
irow = zeros(nz, 1, nag_int_name);
icol = irow;
a          = [4; 1; 5; 2; 2; 3;-1; 1; 4; 1;-2; 3; 2;-1;-2; 5];
irow(1:nz) = [1; 2; 2; 3; 4; 4; 5; 5; 5; 6; 6; 6; 7; 7; 7; 7];
icol(1:nz) = [1; 1; 2; 3; 2; 4; 1; 4; 5; 2; 5; 6; 1; 2; 3; 7];

b = [15; 18; -8; 21; 11; 10; 29];

% Initialise solver
method = 'CG';
precon = 'P';
tol    = 1e-6;
maxitn = nag_int(100);
anorm  = 0;
sigmax = 0;
maxits = nag_int(10);
monit = nag_int(0);
[lwreq, work, ifail] = ...
  f11gd( ...
         method, precon, n, tol, maxitn, anorm, sigmax, ...
         maxits, monit, 'sigcmp', 'N', 'norm_p', 'I');

% SSOR preconditioner
% Calculate reciprocal diagonal matrix elements.
iwork = zeros(n+1, 1, nag_int_name);
rdiag = zeros(n, 1);
for i=1:nz
  if irow(i) == icol(i)
    iwork(irow(i)) = iwork(irow(i)) + 1;
    if a(i) ~= 0
      rdiag(irow(i)) = 1/a(i);
    else
      error('Matrix has a zero diagonal element');
    end
  end
end
for i=1:n
  if iwork(i) == 0
    error('Matrix has a missing diagonal elemen');
  elseif iwork(i) > 2
    error('Matrix has a multiple diagonal element');
  end
end

% Solver inputs
irevcm = nag_int(0);
x      = zeros(n, 1);
wgt    = zeros(n, 1);

% Preconditioner inputs
omega  = 1;
check  = 'C';

% Solve the linear system
while irevcm ~= 4
  [irevcm, x, b, work, ifail] = f11ge( ...
                                       irevcm, x, b, wgt, work);
  if (irevcm == 1)
    % Compute matrix vector product
    [b, ifail] = f11xe( ...
                        a, irow, icol, 'N', x);
  elseif (irevcm == 2)
    % SSOR preconditioning
    [b, ifail] = f11jd( ...
                        a, irow, icol, rdiag, omega, check, x);
  end
end

% Get information about the computation
[itn, stplhs, stprhs, anorm, sigmax, its, sigerr, ifail] = ...
f11gf(work);
fprintf('Converged in %d iterations\n', itn);
fprintf('Final residual norm = %14.4e\n\n', stplhs);
disp('Solution');
disp(x);

% diary off
