function f12au_example

% diary f12au_example.r
fprintf('f12au example results\n\n');

nx = nag_int(10);
n = nx*nx;
nev = nag_int(4);
ncv = nag_int(10);
sigma = 0.4 + 0.6i;

% Construct the matrix A in banded form and store in AB.
% KU, KL are number of superdiagonals and subdiagonals within
% the band of matrices A and M.
kl = nx;
ku = nx;

% Construct ab and mb
ab = zeros(2*kl+ku+1,n);
mb = zeros(2*kl+ku+1,n);

% Main diagonal of A.
idiag = kl + ku + 1;
ab(idiag,1:n) = 4;
mb(idiag,1:n) = ab(idiag,1);

% First subdiagonal and superdiagonal of A.
isup = kl + ku;
isub = kl + ku + 2;
rho = 100;
h = 1/double(nx+1);
for i=1:nx
  lo = (i-1)*nx;
  for j=lo+1:lo+nx-1
    ab(isub,j+1) = -1 + 0.5*h*rho;
    ab(isup,j) = -1 - 0.5*h*rho;
  end
end
mb(isub,2:n)   = 1;
mb(isup,1:n-1) = 1;

% kl-th subdiagonal and ku-th super-diagonal.
isup = kl + 1;
isub = 2*kl + ku + 1;
for i = 1:nx - 1
  lo = (i-1)*nx;
  for j = lo + 1:lo + nx
    ab(isup,nx+j) = -1;
    ab(isub,j) = -1;
  end
end

resid = complex(zeros(100,1));

[icomm, comm, ifail] = f12at( ...
                              n, nev, ncv);
[icomm, comm, ifail] = f12ar( ...
                              'SHIFTED INVERSE', icomm, comm);
[icomm, comm, ifail] = f12ar( ...
                              'GENERALIZED', icomm, comm);

% Find eigenvalues closest in value to SIGMA and corresponding eigenvectors
[nconv, d, z, resid, v, comm, icomm, ifail] = ...
    f12au( ...
           kl, ku, complex(ab), complex(mb), sigma, resid, comm, icomm);

fprintf('\nRitz values closest to sigma:\n');
disp(d);

% diary off
