function f12fd_example

% diary f12fd_example.r
fprintf('f12fd example results\n\n');

n   = nag_int(100);
nev = nag_int(4);
ncv = nag_int(10);
imon = 0;

irevcm = nag_int(0);
resid  = zeros(n,1);
v      = zeros(n,ncv);
x      = zeros(n,1);
mx     = zeros(n,1);

sigma = 0;

% Setup and factorize A - sigma*B
h   = 1/double(n+1);
ad(1:n)  =  2/h - sigma*4*h/6;
adl(1:n) = -1/h - sigma*h/6;
adu(1:n) = adl(1:n);

[adl, ad, adu, adu2, ipiv, info] = f07cd( ...
                                          adl, ad, adu);

% Initialisation Step
[icomm, comm, ifail] = f12fa( ...
                              n, nev, ncv);

% Set Optional Parameters
[icomm, comm, ifail] = f12fd( ...
                              'Generalized', icomm, comm);
[icomm, comm, ifail] = f12fd( ...
                              'Shifted Inverse', icomm, comm);

% Solve
while (irevcm ~= 5)
  [irevcm, resid, v, x, mx, nshift, comm, icomm, ifail] = ...
    f12fb( ...
           irevcm, resid, v, x, mx, comm, icomm);
  if (irevcm == -1)
    % Solve (A-sigma*B)y = Bx
    mx = f12fd_Bx(n, x);
    [x, info] = f07ce( ...
                       'N', adl, ad, adu, adu2, ipiv, mx);
  elseif (irevcm == 1)
    % Solve (A-sigma*B)y = Bx, Bx in mx
     [x, info] = f07ce( ...
                       'N', adl, ad, adu, adu2, ipiv, mx);
  elseif (irevcm == 2)
    % y = Bx
    mx = f12fd_Bx(n, x);
  elseif (irevcm == 4 && imon == 1)
    [niter, nconv, ritz, rzest] = ...
    f12fe(icomm, comm);

    fprintf(['Iteration %2d, No. converged = %d, ', ...
             'norm of estimates = %10.2e\n'], ...
            niter, nconv, norm(rzest(1:nev),2));
  end
end

% Post-process to compute eigenvalues/vectors
[nconv, d, z, v, comm, icomm, ifail] = ...
f12fc( ...
       sigma, resid, v, comm, icomm);

fprintf('Smallest %d Eigenvalues are:\n',nconv);
disp(d(1:nconv));

% diary off

function [y] = f12fd_Bx(n,x)

  y = zeros(n,1);
  h = 1/(6*double(n+1));

  y(1) = 4*x(1) + x(2);
  for j=2:n-1
    y(j) = x(j-1) + 4*x(j) + x(j+1);
  end
  y(n) = x(n-1) + 4*x(n);
  y = y*h;
