function g01da_example

% diary g01da_example.r
fprintf('g01da example results\n\n');

n = nag_int(15);
etol = 0.001;

[pp, errest, ifail] = g01da(n, etol);

fprintf('Set size                = %5d\n', n);
fprintf('Error tolerance (input) = %13.3e\n', etol);
fprintf('Error estimate (output) = %13.3e\n', errest);
fprintf('Normal scores\n');
fprintf('   %10.3f%10.3f%10.3f%10.3f%10.3f\n',pp(1:n));

g01da_plot;

% diary off

function g01da_plot
  % This produces a Q-Q plot for a randomly generated set of data.
  % The normal scores have been calculated using g01da and the sample
  % quantiles obtained by sorting the observed data using m01ca.

  % Initialize the base generator
  seed = [nag_int(2324213)];
  genid = nag_int(1);
  subid = nag_int(1);

  [state, ifail] = g05kf( ...
                          genid, subid, seed);

  % Generate 50 variates from a Normal distribution
  n = nag_int(50);
  [state, x, ifail] = g05sk( ...
                             n, 0, 1, state);
  % Sort x
  m1 = nag_int(1);
  [x, ifail] = m01ca(x, m1, 'Ascending');

  % Calculate normal scores
  etol = 0.001;
  [pp, errest, ifail] = g01da(n, etol);

  fig1 = figure;
  hold on
  plot(pp,x,'+r',[-2.25 2.25],[-2.25 2.25],'black');
  axis([-2.5 2.5 -3 2.5]);
  xlabel('Normal scores');
  ylabel('sample Quantiles');
  title({'Q-Q plot for a random set of data','using exact normal scores'});
  hold off;
  % print(fig1,'-dpng','-r75','g01da_fig1.png');
  % print(fig1,'-deps','-r75','g01da_fig1.eps');
