function g01wa_example

% diary g01wa_example.r
fprintf('g01wa example results\n\n');

m   = nag_int([5,10,15]);
iwt = nag_int([0, 0, 2]);
wt{1}  = [1:double(m(1))];
wt{2}  = [1:double(m(2))];
wt{3}  = [ -3.0; -6.0; -5.0;  3.0; 21.0; 46.0; 67.0; 74.0;
           67.0; 46.0; 21.0;  3.0; -5.0; -6.0; -3.0];
data{1} = [-2170; -1770; -1660; -1360; -1100];
data{2} = [ -950;  -640;  -370;  -140;  -250; 
	    -510;  -620;  -730;  -880; -1130];
data{3} = [-1200;  -830;  -330;  -190;   210;
             170;   440;   440;   780;   880;
	    1220;  1260;  1140;   850;   640];
n = 30;

fprintf('\n Interval          Mean\n');
fprintf('------------------------\n');

% Loop over window lengths
for im = 1:numel(m);
  pn = nag_int(0);
  x{im} = 1820 + m(im)/2 + [1:n-m(im)+1];
  for b=1:3
    % Calculate the number of summaries we can produce
    nb = numel(data{b});
    nsummaries = max(0, nb + min(0, pn-m(im)+1));
    
    % Calculate summary statistics for this block of data
    if b==1
      [pn, rmean, rsd, rcomm, ifail] = ...
      g01wa( ...
	     m(im), data{b}, 'iwt', iwt(im), 'wt', wt{im}, 'wantsd', im<3);
    else
      [pn, rmean, rsd, rcomm, ifail] =  ...
      g01wa( ...
	     m(im), data{b}, 'iwt', iwt(im), 'wt', wt{im}, 'pn', pn, ...
             'rcomm', rcomm, 'wantsd', im<3);
    end

    % Number of results printed so far
    offset = max(0, pn-nb-m(im)+1);

    % Display the results for this block of data
    for i=1:nsummaries
      if im==3
	fprintf(' [%2d,%3d]%14.1f\n', i+offset, i+m(im)-1+offset, rmean(i));
      else
	y{im}(i+offset) = rmean(i);
	z{im}(i+offset) = rsd(i);
      end
    end
  end
end

fprintf('\nTotal number of observations : %d\n', pn);
fprintf('Length of window             : %d\n', m(3));

g01wa_plot(x,y,z,data);

% diary off

function g01wa_plot(x, y, z, data)

  d = [data{1}; data{2}; data{3}];
  fig1 = figure;
  plot(x{1},y{1},x{2},y{2},[1821:1850],d,'*');
  title({'Raw data and mean from a rolling window of', ...
          'changes in rate of Earth''s rotation (μs)'});
  xlabel('Year');
  ylabel('Change in day length');
  legend('m = 5', 'm = 10', 'Location', 'SouthEast');
  set(gca,'xtick',[1820:5:1850])
  set(gca,'ytick',[-2500:500:1500])
  set(gca, 'PlotBoxAspectRatio', [1,1/2,1])
  fig2 = figure;
  plot(x{1},z{1},x{2},z{2});
  title({'Standard deviation from a rolling window of', ...
	 'changes in rate of Earth''s rotation (μs)'});
  ylabel('Standard deviation');
  xlabel('Year');
  set(gca,'ytick',[100:100:700])
  set(gca, 'PlotBoxAspectRatio', [1,1/3,1])
  % print(fig1,'-dpng','-r75','g01wa_fig1.png');
  % print(fig1,'-deps','-r75','g01wa_fig1.eps');
  % print(fig2,'-dpng','-r75','g01wa_fig2.png');
  % print(fig2,'-deps','-r75','g01wa_fig2.eps');
