function g02ka_example

% diary g02ka_example.r
fprintf('g02ka example results\n\n');

% Data
x = [19.5, 43.1, 29.1;
     24.7, 49.8, 28.2;
     30.7, 51.9, 37.0;
     29.8, 54.3, 31.1;
     19.1, 42.2, 30.9;
     25.6, 53.9, 23.7;
     31.4, 58.5, 27.6;
     27.9, 52.1, 30.6;
     22.1, 49.9, 23.2;
     25.5, 53.5, 24.8;
     31.1, 56.6, 30.0;
     30.4, 56.7, 28.3;
     18.7, 46.5, 23.0;
     19.7, 44.2, 28.6;
     14.6, 42.7, 21.3;
     29.5, 54.4, 30.1;
     27.7, 55.3, 25.7;
     30.2, 58.6, 24.6;
     22.7, 48.2, 27.1;
     25.2, 51.0, 27.5];

[n,m] = size(x);
isx   = ones(m,1,nag_int_name);
ip    = nag_int(m);
y = [11.9; 22.8; 18.7; 20.1; 12.9; 21.7; 27.1; 25.4; 21.3; 19.3;
     25.4; 27.2; 11.7; 17.8; 12.8; 23.9; 22.6; 25.4; 14.8; 21.1];

% Parameters
h      = 0.5;
opt    = nag_int(1);
niter  = nag_int(25);
tol    = 0.0001;
orig   = nag_int(2);
optloo = nag_int(2);

% Fit ridge regression model
[h, niter, nep, b, vif, res, rss, df, perr, ifail] = ...
  g02ka( ...
         x, isx, ip, y, h, opt, niter, tol, orig, optloo);

% Display results
fprintf('Value of ridge parameter      : %10.4f\n\n', h);
fprintf('Sum of squares of residuals   : %14.4e\n', rss);
fprintf('Degrees of freedom            : %5d\n', df);
fprintf('Number of effective parameters: %10.4f\n', nep);
fprintf('\nParameter estimates\n');
ivar = double([1:ip+1]);
fprintf('%4d%11.4f\n',[ivar; b(ivar)']);
fprintf('\nNumber of iterations: %15d\n\n', niter);
if opt==1
  fprintf('Ridge parameter minimises GCV\n');
elseif opt==2
  fprintf('Ridge parameter minimises UEV\n');
elseif opt==3
  fprintf('Ridge parameter minimises FPE\n');
elseif opt==4
  fprintf('Ridge parameter minimises BIC\n');
end
fprintf('\nEstimated prediction errors:\n');
fprintf('GCV    = %10.4f\n', perr(1));
fprintf('UEV    = %10.4f\n', perr(2));
fprintf('FPE    = %10.4f\n', perr(3));
fprintf('BIC    = %10.4f\n', perr(4));
if optloo==2
  fprintf('LOO CV = %10.4f\n', perr(5));
end
fprintf('\nResiduals\n');
ivar = [1:n];
fprintf('%4d%11.4f\n',[ivar; res(ivar)']);
fprintf('\nVariance inflation factors\n');
ivar = double([1:ip]);
fprintf('%4d%11.4f\n',[ivar; vif(ivar)']);

% diary off
