function g05zq_example

% diary g05zq_example.r
fprintf('g05zq example results\n\n');

% Random Field variance
var = 0.5;
% Domain endpoints
xmin = -1;
xmax =  1;
ymin = -0.5;
ymax =  0.5;
% Number of sample points in x and y
ns = [nag_int(5), 5];
% Maximum dimensions of circulant matrix
maxm = [nag_int(81), 81];
% Scaling factor rho=1.
icorr = nag_int(2);

% Put covariance parameters (for cov2) in user
norm_p = nag_int(2);
l1 = 0.1;
l2 = 0.15;
nu = 1.2;
user = {norm_p; l1; l2; nu};
% cov2 is even
even = nag_int(1);

% Get square roots of the eigenvalues of the embedding matrix
[lam, xx, yy, m, approx, rho, icount, eig, user, ifail] = ...
  g05zq( ...
         ns, xmin, xmax, ymin, ymax, maxm, var, ...
         @cov2, even, 'icorr', icorr, 'user', user);

fprintf('Size of embedding matrix = %d\n\n', m(1)*m(2));

% Display approximation information if approximation used
if approx == 1
  fprintf('Approximation required\n\n');
  fprintf('rho = %10.5f\n', rho);
  fprintf('eig = %10.5f%10.5f%10.5f\n', eig(1:3));
  fprintf('icount = %d\n', icount);
else
  fprintf('Approximation not required\n\n');
end

% Display square roots of the eigenvalues of the embedding matrix
fprintf('Square roots of eigenvalues of embedding matrix:\n');
mm = m(1)*m(2);
mlam = reshape(lam(1:mm), m(1), m(2));
for i = 1:m(1)
  fprintf('%8.4f',mlam(i,:));
  fprintf('\n');
end

% diary off

function [gam, user] = cov2(x, y, user)
  norm_p = user{1};
  l1     = user{2};
  l2     = user{3};
  nu     = user{4};

  tl1 = abs(x)/l1;
  tl2 = abs(y)/l2;

  if norm_p == 1
    rnorm = tl1 +  tl2;
  else
    rnorm = sqrt(tl1^2+tl2^2);
  end

  gam = exp(-(rnorm^nu));
