
function g05zr_example

% diary g05zr_example.r
fprintf('g05zr example results\n\n');

% Use symmetric stable variogram
icov2 = nag_int(1);
params = [0.1; 0.15; 1.2];

% Random Field variance
var = 0.5;
% Domain endpoints
xmin = -1;
xmax =  1;
ymin = -0.5;
ymax =  0.5;
% Number of sample points in x and y
ns = [nag_int(5), 5];
% Maximum dimensions of circulant matrix
maxm = [nag_int(64), 64];
% Scaling factor, rhoo = 1.
icorr = nag_int(2);

% Get square roots of the eigenvalues of the embedding matrix
[lam, xx, yy, m, approx, rho, icount, eig, ifail] = ...
  g05zr( ...
         ns, xmin, xmax, ymin, ymax, maxm, var, ...
         icov2, params, 'icorr', icorr);

fprintf('Size of embedding matrix = %d\n\n', m(1)*m(2));

% Display approximation information if approximation used
if approx == 1
  fprintf('Approximation required\n\n');
  fprintf('rho = %10.5f\n', rho);
  fprintf('eig = %10.5f%10.5f%10.5f\n', eig(1:3));
  fprintf('icount = %d\n', icount);
else
  fprintf('Approximation not required\n\n');
end

% Display square roots of the eigenvalues of the embedding matrix
fprintf('Square roots of eigenvalues of embedding matrix:\n');
mlam = reshape(lam(1:m(1)*m(2)), m(1), m(2));
for i = 1:m(1)
  fprintf('%8.4f',mlam(i,:));
  fprintf('\n');
end

g05zr_plot;

% diary off

function g05zr_plot

  icov2 = nag_int(4);
  params = [0.1; 0.1];
  var = 1;
  % Domain endpoints
  xmin = 0;
  xmax = 1;
  ymin = 0;
  ymax = 1;
  % Number of sample points in x and y
  ns = [nag_int(100), 100];
  % Maximum dimensions of circulant matrix
  maxm = [nag_int(4096), 4096];
  icorr = nag_int(0);

  % Get square roots of the eigenvalues of the embedding matrix
  [lam, xx, yy, m, approx, rho, icount, eig, ifail] = ...
  g05zr( ...
         ns, xmin, xmax, ymin, ymax, maxm, var, ...
         icov2, params);

  % Initialize state array
  genid = nag_int(1);
  subid = nag_int(1);
  seed  = [nag_int(14965)];
  [state, ifail] = g05kf( ...
                          genid, subid, seed);

  % Compute 2 random field realisations
  s = nag_int(2);
  [state, z, ifail] = g05zs( ...
                             ns, s, m, lam, rho, state);

  fig1 = figure;
  zz = reshape(z(:,1),[100,100]);
  contourf(xx,yy,zz);
  axis equal;
  title({'First realization of Random Field', ...
	 'exponential variogram, corr. length = 0.1'});

  fig2 = figure;
  zz = reshape(z(:,2),[100,100]);
  contourf(xx,yy,zz);
  axis equal;
  title({'Second realization of Random Field', ...
	 'exponential variogram, corr. length = 0.1'});

  % print(fig1,'-dpng','-r75','g05zr_fig1.png');
  % print(fig1,'-deps','-r75','g05zr_fig1.eps');
  % print(fig2,'-dpng','-r75','g05zr_fig2.png');
  % print(fig2,'-deps','-r75','g05zr_fig2.eps');
