function g08ra_example

% diary g08ra_example.r
fprintf('g08ra example results\n\n');

% Single sample observations and design matrix
y = [1;  1;  3;  4;  2;  4;  1;  5;  4;  4;
     4;  4;  4;  1;  4;  5;  5;  4;  4;  3];
x = [1, 23;  1, 32;  1, 37;  1, 41;  1, 41;
     1, 48;  1, 48;  1, 55;  1, 55;  0, 56;
     1, 57;  1, 57;  1, 57;  0, 58;  1, 59;
     0, 59;  0, 60;  1, 61;  1, 62;  1, 62];

ns    = size(y,2);
ip    = size(x,2);
idist = nag_int(2);
nv    = nag_int(numel(y));
nmax  = nv;
tol   = 1e-05;

fprintf('Number of samples           = %3d\n', ns);
fprintf('Number of parameters fitted = %3d\n', ip);
fprintf('Distribution                = %3d\n', idist);
fprintf('Tolerance for ties          = %8.1e\n', tol);

[parvar, irank, zin, eta, vapvec, parest, ifail] = ...
  g08ra( ...
         nv, y, x, idist, nmax, tol);

% Display results
fprintf('\nScore statistic\n');
fprintf('%9.3f%9.3f\n', parest(1:ip));
fprintf('\nCovariance matrix of score statistic\n');
for j = 1:ip
  fprintf('%9.3f', parvar(1:j,j));
  fprintf('\n');
end
fprintf('\nParameter estimates\n');
fprintf('%9.3f', parest(ip+1:ip+ip));
fprintf('\n\nCovariance matrix of parameter estimates\n');
for j = 1:ip
  fprintf('%9.3f', parvar(j+1,1:j));
  fprintf('\n');
end

chisq = parest(2*ip+1);
fprintf('\nChi-squared statistic = %8.3f with %2d d.f.\n\n', chisq, ip);

sterr = reshape(parest(2*ip+2:end),[ip,2]);
fprintf('Standard errors of estimates and approximate z-statistics\n');
disp(sterr);

% diary off
