function g10ab_example

% diary g10ab_example.r
fprintf('g10ab example results\n\n');

x =  [ 5.2  8.8 10.5 10.6 10.4  1.8 12.7 15.6  5.8  1.9 ...
       2.2  4.8  7.9  5.2  0.9 11.8  7.9 11.5 10.6  8.5 ...
      11.1 12.8 11.3  1.0 14.5 11.9  8.1 13.8 15.5  9.8 ...
      11.0 12.4 11.1  5.1  4.8  4.2  6.9 13.2  9.9 12.5 ...
      13.2  8.9 10.8];
y =  [ 4.8  4.1  5.2  5.5  5.0  3.4  3.4  4.9  5.6  3.7 ...
       3.9  4.5  4.8  4.9  3.0  4.6  4.8  5.5  4.5  5.3 ...
       4.7  6.6  5.1  3.9  5.7  5.1  5.2  3.7  4.9  4.8 ...
       4.4  5.2  5.1  4.6  3.9  5.1  5.1  6.0  4.9  4.1 ...
       4.6  4.9  5.1];

% Reorder x, remove ties and weight accordingly
[n, x, y, wt, rss, ifail] = g10za( ...
                                   x, y);
x = x(1:n);
y = y(1:n);

rho  = [1 10 100];
nrho = numel(rho);

c    = zeros(n, 3);
comm = zeros(9*n+14, 1);
yhat = zeros(n,nrho);
rss  = zeros(nrho,1);
df   = zeros(nrho,1);

% Initialize and fit for rho(1)
mode = 'P';
[yhat(:,1), c, rss(1), df(1), res, h, comm, ifail] = ...
  g10ab(mode, x, y, rho(1), c, comm, 'wt', wt);

% Fit for subsequent rhos
mode = 'Q';
for j = 2:nrho
  [yhat(:,j), c, rss(j), df(j), res, h, comm, ifail] = ...
  g10ab( ...
         mode, x, y, rho(j), c, comm, 'wt', wt);
end

%  Display results
fprintf('Smoothing coefficient (rho) = ');
fprintf('  %8.2f', rho);
fprintf('\nResidual sum of squares     = ');
fprintf('%10.3f', rss);
fprintf('\nDegrees of freedom          = ');
fprintf('%10.3f', df);
fprintf('\n\n    x       y                            Fitted Values\n');
fprintf('%8.4f%8.4f%24.4f%10.4f%10.4f\n', [x y yhat]');

fig1 = figure;
plot(x,y,'+',x,yhat(:,1),x,yhat(:,2),x,yhat(:,3));
legend('Raw data', '\rho = 1', '\rho = 10', '\rho = 100', ...
       'Location','NorthWest');
xlabel('Age (years)');
ylabel('C-peptide concentration (pmol/ml)');
title({'Cubic smoothing spline', ...
       'Factors affecting insulin-dependent diabetis mellitus', ...
       'in children; Hastie and Tibshirani (1990)'});
% print(fig1,'-dpng','-r75','g10ab_fig1.png');
% print(fig1,'-deps','-r75','g10ab_fig1.eps');

% diary off
