function h02da_example

%   diary h02da_example.r
  fprintf('h02da example results\n\n');

  % Problem size
  n = nag_int(8);
  nclin = nag_int(5);
  ncnln = nag_int(2);

  varcon = zeros(n+nclin+ncnln, 1, nag_int_name);

  % Set variable types: continuous then binary
  varcon(1:4) = nag_int([0;0;0;0]);
  varcon(5:8) = nag_int([1;1;1;1]);

  bl = zeros(n,1);
  bu = zeros(n,1);

  % Set bounds of continuous variables
  bl(1:4) = 0;
  bu(1:4) = 1000;

  % Set linear constraints
  varcon(n+1)         = 3; % equality
  varcon(n+2:n+nclin) = 4; % inequality

  % Set Ax=d then Ax>=d
  a = zeros(nclin,n);
  a(1,1:4) = 1;
  a(2,[1,5]) = [-1,1];
  a(3,[2,6]) = [-1,1];
  a(4,[3,7]) = [-1,1];
  a(5,[4,8]) = [-1,1];

  d = zeros(nclin,1);
  d(1) = 1;

  % Set constraints supplied by confun, equality first
  varcon(n+nclin+1) = 3;
  varcon(n+nclin+2) = 4;

  iopts = zeros(200, 1, nag_int_name);
  opts = zeros(100, 1);

  % Initialize options
  [iopts, opts, ifail] = h02zk( ...
                                'Initialize = h02da', iopts, opts);

  % Optimisation parameters
  maxit =  nag_int(500);
  acc = 1e-6;

  x = zeros(n,1);
  % Initial estimate (binary variables need not be given)
  x(1:4) = 1;

  % Portfolio parameters
  user.p = 3;
  user.rho = 10;

  % Call the solver
  [ax, x, c, cjac, objgrd, objmip, user, ifail] = ...
  h02da( ...
         ncnln, a, d, bl, bu, varcon, x, @confun, @objfun, iopts, opts, ...
         'user', user);

  % Query the accuracy of the mixed integer QP solver
  [ivalue, accqp, cvalue, optype, ifail] = ...
    h02zl('QP Accuracy', iopts, opts);

  % Results
  [ifail] = x04ca('G', ' ', x, 'Final estimate');
  fprintf('\nMixed integer QP Accuracy: %g', accqp);
  fprintf('\nOptimised value: %g\n\n', objmip);


%   diary off

function [mode, c, cjac, user] = ...
         confun(mode, ncnln, n, varcon, x, cjac, nstate, user)
  c = zeros(ncnln, 1);

  if (mode == 0)
    % Constraints
    % Mean return at least rho:
    c(1) = 8*x(1) + 9*x(2) + 12*x(3) + 7*x(4) - user.rho;

    % Maximum of p assets in portfolio:
    c(2) = user.p - x(5) - x(6) - x(7) - x(8);
  else
    % Jacobian
    cjac(1,1:4) = [8,9,12,7];

    % c(2) does not include continuous variables which requires
    % that their derivatives are zero
    cjac(2,1:4) = 0;
  end

function [mode, objmip, objgrd, user] = ...
         objfun(mode, n, varcon, x, objgrd, nstate, user)
  objmip = 0;

  if (mode == 0)
    % Objective value
    objmip = x(1)*(4*x(1)+3*x(2)-x(3)) + ...
             x(2)*(3*x(1)+6*x(2)+x(3)) + ...
             x(3)*(x(2)-x(1)+10*x(3));
  else
    % Objective gradients for continuous variables
    objgrd(1) = 8*x(1) +  6*x(2) -  2*x(3);
    objgrd(2) = 6*x(1) + 12*x(2) +  2*x(3);
    objgrd(3) = 2*(x(2)-x(1)) + 20*x(3);
    objgrd(4) = 0;
  end
