function h05aa_example

% diary h05aa_example.r
fprintf('h05aa example results\n\n');

% Data required by the scoring function
n      = nag_int(40);
m      = nag_int(14);
[x,y]  = gen_data(n,m);

% Initialize parameters and get communication array lengths
irevcm = nag_int(0);
mincr  = nag_int(0);
ip     = nag_int(5);
drop   = nag_int(0);
lz     = nag_int(0);
mip    = m - ip;
z      = zeros(mip, 1, nag_int_name);
la     = nag_int(0);
nbest  = nag_int(3);
a      = zeros(max(nbest, m), 1, nag_int_name);
bscore = zeros(max(nbest, m), 1);
bz     = zeros(mip, nbest, nag_int_name);
mincnt = nag_int(-1);
gamma  = -1;
acc    = [0, 0];
icomm = zeros(2, 1, nag_int_name);
rcomm = zeros(0, 0);


warning('off', 'NAG:warning');
[irevcm, drop, lz, z, la, a, bscore, bz, icomm, rcomm, ifail] = ...
    h05aa(...
          irevcm, mincr, m, ip, drop, lz, z, la, a, ...
          bscore, bz, mincnt, gamma, acc, icomm, rcomm);
warning('on', 'NAG:warning');

% Ignore the warning message - required size of communication arrays now
% stored in icomm
rcomm = zeros(icomm(2), 1);
icomm = zeros(icomm(1), 1, nag_int_name);

% Initialization  call
cnt = 0;
[irevcm, drop, lz, z, la, a, bscore, bz, icomm, rcomm, ifail] = ...
    h05aa(...
          irevcm, mincr, m, ip, drop, lz, z, la, a, ...
          bscore, bz, mincnt, gamma, acc, icomm, rcomm);

% Reverse communication loop for best subset routine, terminates on irevcm = 0.
while not(irevcm == 0)
  % Calculate and return the score for the required models and keep track
  % of the number of subsets evaluated
  cnt = cnt + max(1,la);
  [bscore] = calc_subset_score(m, drop, lz, z, la, a, x, y, bscore);

  [irevcm, drop, lz, z, la, a, bscore, bz, icomm, rcomm, ifail] = ...
      h05aa(...
            irevcm, mincr, m, ip, drop, lz, z, la, a, ...
            bscore, bz, mincnt, gamma, acc, icomm, rcomm);
end

% Display the best subsets and corresponding scores. 
% h05aa returns a list of features excluded from the best subsets;
% this is inverted to give the set of features included in each subset.
fprintf('\n    Score        Feature Subset\n');
fprintf('    -----        --------------\n');
ibz = 1:m;
for i = 1:la
  mask = ones(1, m, nag_int_name);
  mask(bz(1:mip, i)) = 0;
  fprintf('%12.5e %5d %5d %5d %5d %5d\n', bscore(i), ibz(logical(mask)));
end

fprintf('\n%d subsets evaluated in total\n', cnt);

% diary off

function [bscore] = calc_subset_score(m, drop, lz, z, la, a, x, y, bscore)
  % Set up the initial feature set.
  % If drop = 0, this is the Null set (i.e. no features).
  % If drop = 1 then this is the full set (i.e. all features)
  if drop == 0
    isx = zeros(m, 1, nag_int_name);
  else
    isx = ones(m, 1, nag_int_name);
  end

  % Add (if drop = 0) or remove (if drop = 1) all the features specified in z
  inv_drop = not(drop);
  isx(z(1:lz)) = inv_drop;

  for i=1:max(la, 1)
    if (la > 0)
      if (i > 1)
        % Reset the feature altered at the last iteration
        isx(a(i-1)) = drop;
      end

      %  Add or drop the i'th feature in a
      isx(a(i)) = inv_drop;
    end

    ip = nag_int(sum(isx));

    % Fit the regression model
    rss = g02da('z', x, isx, ip, y);

    % Return the score (the residual sums of squares)
    bscore(i) = rss;
  end
  
function [x, y] = gen_data(n,m)
  x = zeros(n,m);
  genid = nag_int(3);
  subid = nag_int(1);
  seed(1) = nag_int(23124124);
  [state, ifail] = g05kf(genid, subid, seed);
  for i = 1:m
    [state, x(1:n,i), ifail] = g05sk( ...
                                      n, 0, sqrt(3), state);
  end
  [state, b, ifail] = g05sk(m, 1.5, 3, state);
  [state, y, ifail] = g05sk(n, 0, 1, state);
  y = x*b + y;
