function nag_best_subset_given_size_example

% diary nag_best_subset_given_size_example.r
fprintf('nag_best_subset_given_size example results\n\n');

m      = nag_int(14);
ip     = nag_int(5);
nbest  = nag_int(3);
mincr  = nag_int(0);
mincnt = nag_int(-1);
gamma  = -1;
acc    = [-1, -1];
mip    = m - ip;
z      = zeros(mip, 1, nag_int_name);
a      = zeros(max(nbest, m), 1, nag_int_name);
bz     = zeros(mip, nbest, nag_int_name);

% Data required by the scoring function
n = nag_int(40);
[x,y] = gen_data(n,m);

% Put x and y in user to be passed to f, along with a placeholder for the
% number of subsets evaluated
user = {x; y; 0};

% Call the forward communication best subset routine
[la, bscore, bz, user, ifail] = ...
    nag_best_subset_given_size(...
          mincr, m, ip, nbest, @f, mincnt, gamma, acc, 'user', user);

fprintf('\n    Score        Feature Subset\n');
fprintf('    -----        --------------\n');
% Display the best subsets and corresponding scores. nag_best_subset_given_size_revcomm returns
% a list of features excluded from the best subsets, so this is inverted
% to give the set of features included in each subset
ibz = 1:m;
for i = 1:la
  mask = ones(1, m, nag_int_name);
  for j = 1:mip
    mask(bz(j, i)) = 0;
  end
  fprintf('%12.5e %5d %5d %5d %5d %5d\n', bscore(i), ibz(logical(mask)));
end
fprintf('\n%d subsets evaluated in total\n', user{3});

% diary off

function [score, user, info] = f(m, drop, lz, z, la, a, user, info)
  % Set up the initial feature set.
  % If drop = 0, this is the Null set (i.e. no features).
  % If drop = 1 then this is the full set (i.e. all features)
  if drop == 0
    isx = zeros(m, 1, nag_int_name);
  else
    isx = ones(m, 1, nag_int_name);
  end

  % Add (if drop = 0) or remove (if drop = 1) all the features specified in z
  inv_drop = not(drop);
  for i = 1:lz
    isx(z(i)) = inv_drop;
  end

  score = zeros(max(la, 1), 1);
  for i=1:max(la, 1)
    if (la > 0)
      if (i > 1)
        % Reset the feature altered at the last iteration
        isx(a(i-1)) = drop;
      end

      %  Add or drop the i'th feature in a
      isx(a(i)) = inv_drop;
    end

    ip = nag_int(sum(isx));

    % Fit the regression model
    rss = nag_correg_linregm_fit('z', user{1}, isx, ip, user{2});

    % Return the score (the residual sums of squares)
    score(i) = rss;
  end

  % Keep track of the number of subsets evaluated
  user{3} = user{3} + max(1, la);

function [x, y] = gen_data(n,m)
  x = zeros(n,m);
  genid = nag_int(3);
  subid = nag_int(1);
  seed(1) = nag_int(23124124);
  [state, ifail] = nag_rand_init_repeat(genid, subid, seed);
  for i = 1:m
    [state, x(1:n,i), ifail] = nag_rand_dist_normal( ...
                                      n, 0, sqrt(3), state);
  end
  [state, b, ifail] = nag_rand_dist_normal(m, 1.5, 3, state);
  [state, y, ifail] = nag_rand_dist_normal(n, 0, 1, state);
  y = x*b + y;
