function nag_correg_mixeff_ml_example

% diary nag_correg_mixeff_ml_example.r
fprintf('nag_correg_mixeff_ml example results\n\n');

dat = [56, 1, 1, 1, 1;
       50, 1, 2, 1, 1;
       39, 1, 3, 1, 1;
       30, 2, 1, 1, 1;
       36, 2, 2, 1, 1;
       33, 2, 3, 1, 1;
       32, 3, 1, 1, 1;
       31, 3, 2, 1, 1;
       15, 3, 3, 1, 1;
       30, 4, 1, 1, 1;
       35, 4, 2, 1, 1;
       17, 4, 3, 1, 1;
       41, 1, 1, 2, 1;
       36, 1, 2, 2, 2;
       35, 1, 3, 2, 3;
       25, 2, 1, 2, 1;
       28, 2, 2, 2, 2;
       30, 2, 3, 2, 3;
       24, 3, 1, 2, 1;
       27, 3, 2, 2, 2;
       19, 3, 3, 2, 3;
       25, 4, 1, 2, 1;
       30, 4, 2, 2, 2;
       18, 4, 3, 2, 3];
[n,ncol] = size(dat);

% Number of levels in each variable
levels = [nag_int(1);4;3;2;3];

% Model information
yvid =  nag_int(1);
fvid = [nag_int(3);  4;  5];
rvid = [nag_int(3)];
svid =  nag_int(2);
cwid =  nag_int(0);
fint =  nag_int(1);
rint =  nag_int(1);

% Calculate lb
lb = (rint + sum(levels(rvid)))*prod(levels(svid)) + ...
     fint + sum(levels(fvid)) - numel(fvid);

% Variance component
vpr = [nag_int(1)];
nvpr = nag_int(numel(vpr));

% Initial gamma
gamma = [1; 1; 0];
% Fit the linear mixed effects regresion model
[nff, nrf, df, ml, b, se, gamma, warn, ifail] = ...
nag_correg_mixeff_ml( ...
       nvpr, levels, yvid, fvid, rvid, svid, cwid, vpr, ...
       dat, fint, rint, lb, gamma);

% Display results
if warn
   fprintf(['Warning: At least one variance component was ', ...
            'estimated to be negative and then reset to zero\n\n']);
end
fprintf('Fixed effects (Estimate and Standard Deviation)\n\n');
k = 1;
if fint==1
  fprintf('Intercept%15s%10.4f%10.4f\n', ' ',b(k), se(k));
  k = k + 1;
end
for i = 1:numel(fvid)
  for j = 1:levels(fvid(i))
    if levels(fvid(i))==1 || j>1
      fprintf('Variable %3d Level %3d: %10.4f%10.4f\n', i, j, b(k), se(k));
      k = k + 1;
    end
  end
end
fprintf('\nRandom Effects (Estimate and Standard Deviation\n\n');
if svid==0
  for i = 1:numel(rvid)
    for j = 1:levels(rvid(i))
      fprintf('Variable %4d Level %4d: %10.4f %10.4f\n', i, j, b(k), se(k));
      k = k + 1;
    end
  end
else
  for l = 1:levels(svid)
    if (rint==1)
      fprintf('Intercept for Subject Level %4d:%12s%10.4f%10.4f\n', ...
              l, ' ', b(k), se(k));
      k = k + 1;
    end
    for i = 1:numel(rvid)
      for j = 1:levels(rvid(i))
        fprintf('Subject Level %4d Variable %4d Level %4d: %10.4f%10.4f\n', ...
               l, i, j, b(k), se(k));
        k = k + 1;
      end
    end
  end
end
fprintf('\n Variance Components\n');
for i = 1:nvpr+rint
  fprintf('%4d%10.4f\n',i,gamma(i));
end
fprintf('\nsigma^2           = %10.4f\n', gamma(nvpr+rint+1));
fprintf('-2log likelihood  = %10.4f\n', ml);
fprintf('DF                = %16d\n', df);

% diary off
