function nag_eigen_real_gen_sparse_arnoldi_example

% diary nag_eigen_real_gen_sparse_arnoldi_example.r
fprintf('nag_eigen_real_gen_sparse_arnoldi example results\n\n');


% This example demonstrates the use of nag_eigen_real_gen_sparse_arnoldi to evaluate a number of
% eigenvalues of a sparse matrix closest to a given value.

% Use existing sparse matrix.
load('west0479.mat')
W = west0479;

[irow,icol,a] = find(W);
n = nag_int(size(W,1));
nnz = nag_int(size(irow,1));

% Add zero diagonals to be summed
irow(nnz+1:nnz+n) = (1:n);
icol(nnz+1:nnz+n) = (1:n);
a(nnz+1:nnz+n) = 0.0;
nnz = nnz + n;

% Use nag_sparse_real_gen_sort to get Compressed column storage array icolzp.
irow = nag_int(irow);
icol = nag_int(icol);
dup = 'S';
zero = 'K';
[nnz, a, icol, irow, icolzp, ifail] = ...
  nag_sparse_real_gen_sort(...
        n, nnz, a, icol, irow, dup, zero);

% Evaluate nev eigenvalues (w) closest to sigma.
nev = nag_int(20);
ncv = nag_int(60);
sigma = 4.0;
% Set some options via user array and function argument option.
% user(1) = print level, iuser(2) = iteration limit,
% user(3)>0 means shifted-invert mode
% user(4)>0 means print monitoring info
user = [0, 500, 1, 0];

[a, nconv, w, v, resid, user, ifail] = ...
  nag_eigen_real_gen_sparse_arnoldi(...
        n, a, icolzp, irow, nev, ncv, sigma, @monit, @option, 'user', user);

fprintf('\n The %d Ritz values closest to %8.2e are:\n', nconv, sigma);
disp(w(1:nconv));

fig1 = figure;
plot(w(1:nconv),'k+');
title('Eigenvalues of West0479 closest to (4,0)');
xlabel('real');
ylabel('imag');
% print(fig1,'-dpng','-r75','nag_eigen_real_gen_sparse_arnoldi_fig1.png');
% print(fig1,'-deps','-r75','nag_eigen_real_gen_sparse_arnoldi_fig1.eps');

% diary off

function [istat, user] = monit(ncv, niter, nconv, w, rzest, istat, user);
  if (user(4)>0)
    if (niter==1 && user(3)>0)
      fprintf('The following Ritz values (mu) are related to the\n');
      fprintf('true eigenvalues (lambda) by lambda = sigma + 1/mu\n\n');
    end
    fprintf('Iteration number %d\n', niter);
    fprintf(' Ritz values converged so far (%d):\n', nconv);
    for i = 1:nconv
      fprintf(' %4d (%13.5e,%13.5e)\n',  i, real(w(i)), imag(w(i)));
    end
    fprintf(' Next (unconverged) Ritz value:\n');
    fprintf(' %4d (%13.5e,%13.5e)\n', nconv+1, real(w(nconv+1)), ...
            imag(w(nconv+1)));
  end
  istat = nag_int(0);

function [icomm, comm, istat, user] = option(icomm, comm, istat, user);
  if (user(1)>0)
    [icomm, comm, ifail] = ...
      nag_sparseig_real_option(strcat('Print Level=', int2str(user(1))), icomm, comm);
    istat = max(istat,ifail);
  end
  if (user(2)>100)
    [icomm, comm, ifail] = ...
      nag_sparseig_real_option(strcat('Iteration Limit=', int2str(user(2))), icomm, comm);
    istat = max(istat,ifail);
  end
  if (user(3)>0)
    [icomm, comm, ifail] = ...
      nag_sparseig_real_option('Shifted Inverse Real', icomm, comm);
    istat = max(istat,ifail);
  end
