function nag_eigen_real_symm_sparse_arnoldi_example

% diary nag_eigen_real_symm_sparse_arnoldi_example.r
fprintf('nag_eigen_real_symm_sparse_arnoldi example results\n\n');


% This example demonstrates the use of nag_eigen_real_symm_sparse_arnoldi to evaluate a number of
% eigenvalues of a sparse symmetric matrix closest to a given value.

% Use existing sparse matrix to generate symmetric one.
load('west0479.mat')
W = west0479;
S = W * W';

%' Extract details of sparse matrix S.
[irow,icol,a] = find(S);
n   = nag_int(size(S,1));
nnz = nag_int(size(irow,1));

% zero out strictly upper triangle
for i = 1 : nnz 
  if (icol(i)>irow(i));
     a(i) = 0.0;
  end
end

% Use nag_sparse_real_gen_sort to remove zero entries.
irow = nag_int(irow);
icol = nag_int(icol);
dup  = 'R';
zero = 'R';
[nnz, a, icol, irow, icolzp, ifail] = ...
  nag_sparse_real_gen_sort(...
        n, nnz, a, icol, irow, dup, zero);

% Evaluate nev eigenvalues (w) closest to sigma.
nev   = nag_int(20);
ncv   = nag_int(60);
sigma = 50000;

[nconv, w, v, resid, user, ifail] = ...
    nag_eigen_real_symm_sparse_arnoldi(...
          n, a, irow, icol, nev, ncv, sigma, 'nag_eigen_arnoldi_monit_symm', @option);

fprintf('\n The %d Ritz values closest to %13.5e are:\n', nconv, sigma);

disp(w(1:nconv));

fig1 = figure;
plot(w(1:nconv),'k+');
title('Eigenvalues of WW^T (W=West0479) closest to 50000');
xlabel('index');
ylabel('eigenvalue');
% print(fig1,'-dpng','-r75','nag_eigen_real_symm_sparse_arnoldi_fig1.png');
% print(fig1,'-deps','-r75','nag_eigen_real_symm_sparse_arnoldi_fig1.eps');

% diary off

function [istat, user] = monit(ncv, niter, nconv, w, rzest, istat, user);
  istat = nag_int(0);

function [icomm, comm, istat, user] = option(icomm, comm, istat, user);
    [icomm, comm, ifail] = nag_sparseig_real_symm_option('Shifted Inverse', icomm, comm);
    istat = ifail;
