function nag_fit_1dspline_knots_example

% diary nag_fit_1dspline_knots_example.r
fprintf('nag_fit_1dspline_knots example results\n\n');

m = 14;
data = [  0.20  0.00  0.2;
          0.47  2.00  0.2;
          0.74  4.00  0.3;
          1.09  6.00  0.7;
          1.60  8.00  0.9;
          1.90  8.62  1.0;
          2.60  9.10  1.0;
          3.10  8.90  1.0;
          4.00  8.15  0.8;
          5.15  7.00  0.5;
          6.17  6.00  0.7;
          8.00  4.54  1.0;
         10.00  3.39  1.0;
         12.00  2.56  1.0];
x = data(:,1);
y = data(:,2);
w = data(:,3);
     
knots = [1.5  2.6   4   8];
nbar  = size(knots,2) + 1;
ncap7 = nbar + 7;
lamda = zeros(ncap7,1);
lamda(5:nbar+3) = knots;

[lamda, c, ss, ifail] = nag_fit_1dspline_knots( ...
                               x, y, w, lamda);

fprintf('\n  j       lamda(j+2)     b-spline coeff c(j)\n\n');
fprintf('%3d%35.4f\n', 1, c(1));
for j = 2:nbar+2
  fprintf('%3d%15.4f%20.4f\n',j,lamda(j+2),c(j));
end
fprintf('%3d%35.4f\n', nbar+3, c(nbar+3));
fprintf('\nResidual sum of squares = %12.2e\n\n', ss)
fprintf('Cubic spline approximation and residuals\n\n');
fprintf('       x          w          y         Fit     Residual\n');

k = 0;
for i = 1:m
  % data point evaluation
  k = k + 1;
  xp(k) = x(i);
  [fit(k), ifail] = nag_fit_1dspline_eval( ...
                           lamda, c, x(i));
  fprintf('%11.4f%11.4f%11.4f%11.4f%11.2e\n', ...
          x(i), w(i), y(i), fit(k), fit(k)-y(i));
  % mid point evaluation
  if i<m
    xh = (x(i)+x(i+1))/2;
    k = k + 1;
    xp(k) = xh;
    [fit(k), ifail] = nag_fit_1dspline_eval( ...
                             lamda, c, xh);
    fprintf('%11.4f%33.4f\n',xh, fit(k))
  end
end

fig1 = figure;
hold on
plot(x,y,'*')
plot(xp,fit);
xlabel('x');
title({'Weighted least-squares cubic spline approximation', ...
       'to a set of 14 data points'});
legend('data points','cubic spline fit');
hold off;
% print(fig1,'-dpng','-r75','nag_fit_1dspline_knots_fig1.png');
% print(fig1,'-deps','-r75','nag_fit_1dspline_knots_fig1.eps');

% diary off
