function nag_fit_2dcheb_lines_example

% diary nag_fit_2dcheb_lines_example.r
fprintf('nag_fit_2dcheb_lines example results\n\n');

% Fit cubic in x and quadratic in y
k = nag_int(3);
l = nag_int(2);

% 4 lines of data, 1 for each of 4 values of y
n = 4;
m = nag_int([8 7 7 6]);
mtot = sum(m);
x = [0.1     1.0     1.6     2.1     3.3     3.9     4.2     4.9 ...
     0.1     1.1     1.9     2.7     3.2     4.1     4.5         ...
     0.5     1.1     1.3     2.2     2.9     3.5     3.9         ...
     1.7     2.0     2.4     2.7     3.1     3.5 ];
f = [1.01005 1.10517 1.17351 1.23368 1.39097 1.47698 1.52196 1.63232 ...
     2.02010 2.23256 2.41850 2.61993 2.75426 3.01364 3.13662         ...
     3.15381 3.34883 3.41649 3.73823 4.00928 4.25720 4.43094         ...
     5.92652 6.10701 6.35625 6.54982 6.81713 7.09534 ];
w =  ones(mtot, 1);

y    = [0   1    2    4  ];  ymin = min(y); ymax = max(y);
xmin = [0   0.1  0.4  1.6];
xmax = [5   4.5  4    3.5];

nux  = [0];
nuy  = nux;

% Compute surface fit
[a, ifail] = nag_fit_2dcheb_lines( ...
                    m, k, l, x, y, f, w, xmin, xmax, nux, nuy);

fig1 = figure;
colors = {'Blue','Green','Red','Black'};
hold on;
% Evaluate fit 
mlast = nag_int(0);
for i = 1:n
  mfirst = mlast + 1;
  mlast  = mlast + m(i);
  [fit, ifail] = nag_fit_2dcheb_eval( ...
                        mfirst, k, l, x, xmin(i), xmax(i), y(i), ...
                        ymin, ymax, a, 'mlast', mlast);
  mm = mfirst:mlast;
  mfit(mm) = fit(mm);

  fprintf('\nLine number %d, y = %7.2f\n',i,y(i));
  fprintf('       x          f         fit      residual\n');
  sol = [x(mm); f(mm); fit(mm)'; fit(mm)'-f(mm)];
  fprintf('%11.4f%11.4f%11.4f%11.2e\n', sol);
  col = colors(:,i);
  plot(x(mm),fit(mm),'Color',char(col));
end
plot(x,f,'*','Color','Magenta');
hold off
title('Least-squares bi-variate polynomial fit');
xlabel('x');
ylabel('p(x,y=0,1,2,4)')
legend('line 1: y = 0','line 2: y = 1','line 3: y = 2',...
       'line 4: y = 4','data points','Location','NorthWest');

% print(fig1,'-dpng','-r75','nag_fit_2dcheb_lines_fig1.png');
% print(fig1,'-deps','-r75','nag_fit_2dcheb_lines_fig1.eps');

% diary off
