function nag_glopt_nlp_pso_example

% diary nag_glopt_nlp_pso_example.r
fprintf('nag_glopt_nlp_pso example results\n\n');

npar = nag_int(20);
bl = [-500; -500; -1e6; -1; -0.9];
bu = [ 500;  500; 10; 5e5; 0.9];

x_target_u = [-420.9687463599820; -420.9687463599820];
x_target_c = [-394.1470221120988; -433.48214189947606];
f_target_u = -837.9657745448674;
f_target_c = -731.70709230672696;
c_target_u = [0; 31644.05623568455; 0.07574889943398055];
c_target_c = [0; 0; 0];

xbest = zeros(2, 20);
fbest = zeros(20, 1);
cbest = zeros(3, 20);;

iopts = zeros(100, 1, nag_int_name);
opts  = zeros(100, 1);


% Initialize the option arrays for nag_glopt_nlp_pso
[iopts, opts, ifail] = nag_glopt_optset(...
                             'Initialize = nag_glopt_nlp_pso', iopts, opts);

% Query some default option values.
[ivalue, rvalue, cnorm, optype, ifail] = ...
  nag_glopt_optget(...
        'Constraint Norm', iopts, opts);
[maxits, rvalue, itstr, optype, ifail] = ...
  nag_glopt_optget(...
        'Maximum Iterations Completed', iopts, opts);
[ivalue, distol, cvalue, optype, ifail] = ...
  nag_glopt_optget(...
        'Distance Tolerance', iopts, opts);

itstr = strtrim(itstr);
fprintf('\nDefault Option Queries:\n\n');
fprintf('Constraint Norm              : %s\n', cnorm);
fprintf('Maximum Iterations Completed : %d (%s)\n', ivalue, itstr);
fprintf('Distance Tolerance           : %15.4e\n', distol);

fprintf('\n1. Solution without using coupled local minimizer.\n');

% Set various options to non-default values if required.
[iopts, opts, ifail] = nag_glopt_optset(...
                             'Repeatability = On', iopts, opts);
opstr = sprintf('Distance Tolerance = %32.16e', distol*0.1);
[iopts, opts, ifail] = nag_glopt_optset(...
                             opstr, iopts, opts);
[iopts, opts, ifail] = nag_glopt_optset(...
                             'Constraint Tolerance = 1.0e-4', iopts, opts);
[iopts, opts, ifail] = nag_glopt_optset(...
                             'Constraint Norm = Euclidean', iopts, opts);
opstr = sprintf('Target Objective Value = %32.16e', f_target_c);
[iopts, opts, ifail] = nag_glopt_optset(...
                             opstr, iopts, opts);
[iopts, opts, ifail] = nag_glopt_optset(...
                       'Target Objective Tolerance = 1.0e-4', iopts, opts);

% Call nag_glopt_nlp_pso to search for the global optimum.
[xb, fb, cb, xbest, fbest, cbest, iopts, opts, user, itt, inform, ifail] = ...
  nag_glopt_nlp_pso(...
        bl, bu, xbest, fbest, cbest, @objfun, @confun, 'nag_glopt_nlp_pso_dummy_monmod', ...
        iopts, opts);

switch ifail
  case {0,1}
    % nag_glopt_nlp_pso encountered no errors during operation,
    % and will have returned the best optimum found.
    display_result(x_target_u, x_target_c, f_target_u, f_target_c, ...
                   c_target_u, c_target_c, xb, fb, cb, itt, inform);
  case 3
    % An instruction to exit was received by nag_glopt_nlp_pso from objfun or monmod.
    % The exit flag will have been returned in inform.
    display_result(x_target_u, x_target_c, f_target_u, f_target_c, ...
                   c_target_u, c_target_c, xb, fb, cb, itt, inform);
  otherwise
    % An error was detected, and a warning has been displayed
end


fprintf('\n2. Solution using coupled local minimizer nag_opt_nlp1_solve.\n');

% Set the local minimizer to be nag_opt_nlp1_solve and set corresponding options
[iopts, opts, ifail] = ...
    nag_glopt_optset(...
          'Local Minimizer = nag_opt_nlp1_solve', iopts, opts);
[iopts, opts, ifail] = ...
  nag_glopt_optset(...
        'Local Interior Major Iterations = 5', iopts, opts);
[iopts, opts, ifail] = ...
  nag_glopt_optset(...
        'Local Interior Minor Iterations = 3', iopts, opts);
[iopts, opts, ifail] = ...
  nag_glopt_optset(...
        'Local Exterior Major Iterations = 30', iopts, opts);
[iopts, opts, ifail] = ...
  nag_glopt_optset(...
        'Local Exterior Minor Iterations = 15', iopts, opts);

% Get and increase Distance Tolerance
[ivalue, rvalue, cvalue, optype, ifail] = ...
      nag_glopt_optget(...
            'Distance Tolerance', iopts, opts);
opstr = sprintf('Distance Tolerance = %32.16e', rvalue*10);
[iopts, opts, ifail] = nag_glopt_optset(opstr, iopts, opts);

opstr = sprintf('Local Interior Tolerance = %32.16e', rvalue);
[iopts, opts, ifail] = nag_glopt_optset(...
                             opstr, iopts, opts);
opstr = sprintf('Local exterior Tolerance = %32.16e', rvalue*1e-4);
[iopts, opts, ifail] = nag_glopt_optset(...
                             opstr, iopts, opts);

% Call nag_glopt_nlp_pso to search for the global optimum.
[xb, fb, cb, xbest, fbest, cbest, iopts, opts, user, itt, inform, ifail] = ...
     nag_glopt_nlp_pso(bl, bu, xbest, fbest, cbest, @objfun, @confun, ...
           'nag_glopt_nlp_pso_dummy_monmod', iopts, opts);

switch ifail
  case {0,1}
    % nag_glopt_nlp_pso encountered no errors during operation,
    % and will have returned the best optimum found.
    display_result(x_target_u, x_target_c, f_target_u, f_target_c, ...
                   c_target_u, c_target_c, xb, fb, cb, itt, inform);
  case 3
    % An instruction to exit was received by nag_glopt_nlp_pso from objfun or monmod.
    % The exit flag will have been returned in inform.
    display_result(x_target_u, x_target_c, f_target_u, f_target_c, ...
                   c_target_u, c_target_c, xb, fb, cb, itt, inform);
  otherwise
    % An error was detected, and a warning has been displayed
end

% diary off

function [mode, c, cjac, user] = ...
    confun(mode, ncon, ndim, ldcj, needc, x, cjac, nstate, user)
  c = zeros(ncon, 1);

  % Test nstate to determine whether the local minimizer is being called
  % for the first time from a new start point
  if (nstate == 1)
    % Set any constant elements of the Jacobian matrix.
    cjac(1, 1) =  3;
    cjac(1, 2) = -2;
  end

  % mode determines whether constraints, derivatives, or both are required
  if mode==0 || mode==2
    % Constraint values are required, however only those for which needc
    % is non-zero need be set.
    for k = 1:double(ncon)
       if (needc(k)>0)
          switch k
          case (1)
             c(k) = 3.0*x(1) - 2.0*x(2);
          case (2)
             c(k) = x(1)^2 - x(2)^2 + 3.0*x(1)*x(2);
          case (3)
             c(k) = cos((x(1)/200.0)^2+(x(2)/100.0));
          otherwise
             c(k) = 0;
          end
       end
    end
  end

  if mode==1 || mode==2
    %Constraint derivatives (cjac) are required.
    for k = 1:double(ncon)
        switch k
        case (1)
          % Constant derivatives set when nstate=1 remain throughout
          % the local minimization.
        case (2)
          % If the constraint derivatives are known and are readily
          % calculated, populate cjac when required.
           cjac(k,1) =  2.0*x(1) + 3*x(2);
           cjac(k,2) = -2.0*x(2) + 3*x(1);
        otherwise
          % Any elements of cjac left unaltered will be approximated
          % using finite differences when required.
        end
     end
  end

function [mode, objf, vecout, user] = ...
          objfun(mode, ndim, x, objf, vecout, nstate, user)

  % Test nstate to indicate what stage of computation has been reached.
  switch nstate
    case (2)
      % objfun is called for the very first time.
    case (1)
      % objfun is called on entry to a NAG local minimiser.
    case (0)
      % This will be the normal value of NSTATE.
    otherwise
      % This is extremely unlikely, and indicates that an error has
      % occurred on the system
      mode = nag_int(-1);
      error('*** Error detected in objfun');
  end

  % Test mode to determine whether to calculate objf and/or objgrd.
  evalobjf = false;
  evalobjg = false;
  switch mode
    case {0,5}
      % Only the value of the objective function is needed.
      evalobjf = true;
    case {1,6}
      % Only the values of the NDIM gradients are required.
      evalobjg = true;
    case {2,7}
      % Both the objective function and the NDIM gradients are required.
      evalobjf = true;
      evalobjg = true;
    otherwise
      mode = nag_int(-1);
      error('*** Illegal value of mode (%d) in objfun', mode);
  end

  if evalobjf
    % Evaluate the objective function.
    objf = sum(x(1:double(ndim)).*sin(sqrt(abs(x(1:double(ndim))))));
  end

  if evalobjg
    % Calculate the gradient of the objective function,
    % and return the result in vecout.
    vecout = sqrt(abs(x));
    for i=1:double(ndim)
      vecout(i) = sin(vecout(i)) + 0.5*vecout(i)*cos(vecout(i));
    end
  end

function [] = display_result(x_target_u, x_target_c, f_target_u, ...
                                   f_target_c, c_target_u, c_target_c, ...
                                   xb, fb, cb, itt, inform)

  % Display final counters.
  fprintf('\nAlgorithm Statistics\n--------------------\n');
  fprintf('Total complete iterations             : %d\n', itt(1));
  fprintf('Complete iterations since improvement : %d\n', itt(2));
  fprintf('Total particles converged to xb       : %d\n', itt(3));
  fprintf('Total improvements to global optimum  : %d\n', itt(4));
  fprintf('Total function evaluations            : %d\n', itt(5));
  fprintf('Total particles re-initialized        : %d\n', itt(6));
  fprintf('Total constraints violated            : %d\n\n', itt(7));

  % Display why finalization occurred.
  switch inform
  case 0
     fprintf('Solution Status : An error was detected by nag_glopt_bnd_pso\n');
  case 1
     fprintf('Solution Status : Target value achieved\n');
  case 2
     fprintf('Solution Status : Minimum swarm standard deviation obtained\n');
  case 3
     fprintf('Solution Status : Sufficient particles converged\n');
  case 4
     fprintf('Solution Status : No improvement in preset iteration limit\n');
  case 5
     fprintf('Solution Status : Maximum complete iterations attained\n');
  case 6
     fprintf('Solution Status : Maximum function evaluations exceeded\n');
  case 7
     fprintf('Solution Status : Constrained point located\n');
  otherwise
     fprintf('User termination case:  %d\n', inform);
  end

  % Display final objective value and location.
  fprintf('\n Known unconstrained objective minimum     : %9.3f\n', f_target_u);
  fprintf(' Best Known constrained objective minimum  : %9.3f\n', f_target_c);
  fprintf(' Achieved objective value                  : %9.3f\n\n', fb);

  title = 'Comparison between known and achieved optima.';
  clabs = {'x_target_u'; 'x_target_c'; 'xb        '};
  [ifail] = nag_file_print_matrix_real_gen_comp('G', 'N', [x_target_u, x_target_c, xb], 'f11.2', title, ...
                  'I', clabs, 'C', clabs, nag_int(80), nag_int(0));
  fprintf('\n');

  title   = 'Comparison between scaled constraint violations.';
  clabs   = {'c_target_u'; 'c_target_c'; 'cb        '};
  c_scale = [2490; 750000; 0.1];
  c_arr   = [c_target_u./c_scale, c_target_c./c_scale, cb./c_scale];
  [ifail] = nag_file_print_matrix_real_gen_comp(...
                  'G', 'N', c_arr, 'f11.5', title, ...
                  'I', clabs, 'C', clabs, nag_int(80), nag_int(0));
fprintf('\n');
