function nag_interp_4d_scat_shep_eval_example

% diary nag_interp_4d_scat_shep_eval_example.r
fprintf('nag_interp_4d_scat_shep_eval example results\n\n');

genid = nag_int(1);
subid = nag_int(1);
seed  = [nag_int(1762543)];
seed2 = [nag_int(43331)];


% Initialize the generator to a repeatable sequence
[state, ifail] = nag_rand_init_repeat(genid, subid, seed);

% Generate the data points x
[state, x, ifail] = nag_rand_dist_uniform01(nag_int(120), state);

% Put x into the right shape for the e01 routines
x = reshape(x, 4, 30);

% Create function values
c1 = cos(6*x(1,:));
c2 = cos(6*x(2,:));
c3 = 6 + 6*(3*x(3,:)-1).^2;
c4 = 1.25 + cos(5.4*x(4,:));
f  = c4.*c1.*c2./c3;


% Generate the interpolant
nq = nag_int(0);
nw = nag_int(0);
[iq, rq, ifail] = nag_interp_4d_scat_shep(x, f, nw, nq);

% Generate repeatable evaluation points
[state, ifail] = nag_rand_init_repeat(genid, subid, seed2);
[state, xe, ifail] = nag_rand_dist_uniform01(nag_int(32), state);
xe = reshape(xe, 4, 8);

% Evaluate the interpolant
[q, qx, ifail] = nag_interp_4d_scat_shep_eval(x, f, iq, rq, xe);

% Evaluate function at xe
c1  = cos(6*xe(1,:));
c2  = cos(6*xe(2,:));
c3  = 6 + 6*(3*xe(3,:)-1).^2;
c4  = 1.25 + cos(5.4*xe(4,:));
fun = c4.*c1.*c2./c3;

fprintf('\n i |  f(i)       q(i)   |f(i)-q(i)|\n');
fprintf('---|--------------------+---------+\n');
for i=1:8
  fprintf(' %d |%8.4f  %8.4f  %8.4f \n', i, fun(i), q(i), abs(fun(i)-q(i)));
end

% diary off
