function nag_lapack_dsbev_example

% diary nag_lapack_dsbev_example.r
fprintf('nag_lapack_dsbev example results\n\n');

% Symmetric band matrix A, stored on symmetric banded format
uplo = 'U';
kd = nag_int(2);
n  = nag_int(5);
ab = [0, 0, 3, 4, 5;
      0, 2, 3, 4, 5;
      1, 2, 3, 4, 5];

% Calculate all eigenvalues and eigenvectors
jobz = 'Vectors';
[abf, w, z, info] = nag_lapack_dsbev( ...
                           jobz, uplo, kd, ab);

% Normalize eigenvectors: largest element positive
for j = 1:n
  [~,k] = max(abs(z(:,j)));
  if z(k,j) < 0;
    z(:,j) = -z(:,j);
  end
end                            

disp('Eigenvalues');
disp(w);
disp('Eigenvectors');
disp(z);

% Eigenvalue error bound
errbnd = nag_machine_precision*max(abs(w(1)),abs(w(end)));
% Eigenvector condition numbers
[rcondz, info] = nag_lapack_ddisna( ...
		        'Eigenvectors', n, n, w);

% Eigenvector error bounds
zerrbd = errbnd./rcondz;

disp('Error estimate for the eigenvalues');
fprintf('%12.1e\n',errbnd);
disp('Error estimates for the eigenvectors');
fprintf('%12.1e',zerrbd);
fprintf('\n');

% diary off
