function nag_lapack_dtrevc_example

% diary nag_lapack_dtrevc_example.r
fprintf('nag_lapack_dtrevc example results\n\n');

n = nag_int(4);
a = [ 5.14, 0.91,  0.00, -32.80;
      0.91, 0.20,  0.00,  34.50;
      1.90, 0.80, -0.40,  -3.00;
     -0.33, 0.35,  0.00,   0.66];

% Balance a
[a, ilo, ihi, scale, info] = nag_lapack_dgebal( ...
                                    'Both', a);

% Reduce a to upper Hessenberg form
[H, tau, info] = nag_lapack_dgehrd( ...
                        ilo, ihi, a);

% Form Q
[Q, info] = nag_lapack_dorghr( ...
                   ilo, ihi, H, tau);

% Calculate the eigenvalues and Schur factorisation of A
[H, wr, wi, Z, info] = nag_lapack_dhseqr( ...
                              'Schur Form', 'Vectors', ilo, ihi, H, Q);

w = wr + i*wi;
disp('Eigenvalues:');
disp(w);

% Calculate the eigenvectors of A
[select, ~, VR, m, info] = ...
nag_lapack_dtrevc( ...
       'Right', 'Backtransform', false, H, zeros(1), Z, n);

% Back scale to get eigenvectors of A
[VR, info] = nag_lapack_dgebak( ...
                    'Both', 'Right', ilo, ihi, scale, VR);

% Normalize eigenvectors: largest element positive
for j = 1:n
  [~,k] = max(abs(VR(:,j)));
  VR(:,j) =VR(:,j)/norm(VR(:,j));
  if VR(k,j) < 0;
    VR(:,j) = -VR(:,j);
  end
end                            

disp('Eigenvectors:');
disp(VR);

% diary off
