function nag_lapack_zggesx_example

% diary nag_lapack_zggesx_example.r
fprintf('nag_lapack_zggesx example results\n\n');

% Matrix pair (A,B)
A = [ -21.10 - 22.50i,  53.5 - 50.5i,  -34.5 + 127.5i,   7.5 +  0.5i;
       -0.46 -  7.78i,  -3.5 - 37.5i,  -15.5 +  58.5i, -10.5 -  1.5i;
        4.30 -  5.50i,  39.7 - 17.1i,  -68.5 +  12.5i,  -7.5 -  3.5i;
        5.50 +  4.40i,  14.4 + 43.3i,  -32.5 -  46i,   -19.0 - 32.5i];
B = [ 1   - 5i,    1.6 + 1.2i, -3 + 0i,  0   - 1i;
      0.8 - 0.6i,  3.0 - 5.0i, -4 + 3i, -2.4 - 3.2i;
      1   + 0i,    2.4 + 1.8i, -4 - 5i,  0   - 3i;
      0   + 1i,   -1.8 + 2.4i,  0 - 4i,  4   - 5i];

% Generalized Schur form (S,T) of (A,B), generalized eigenvalues
% and Schur vectors Q and Z with sorting: select eigenvalues for which
% the diagonal of T is at least 6 times bigger than the corresponding
% diagonal of S.
jobvsl = 'Vectors (left)';
jobvsr = 'Vectors (right)';
sortp = 'Sort';
selctg = @(a, b) (abs(a) < 6*abs(b));
sense = 'Both reciprocal condition numbers';
[S, T, sdim, alpha, beta, VSL, VSR, rconde, rcondv, info] = ...
  nag_lapack_zggesx( ...
         jobvsl, jobvsr, sortp, selctg, sense, A, B);

fprintf('Number of selected eigenvalues = %4d\n\n', sdim);
disp('Generalized eigenvalues');
eigs = alpha./beta;
disp(eigs(1:sdim));

fprintf('%s\n%s\n%s = %8.1e, %s = %8.1e\n\n', ...
        'Reciprocals of left and right projection norms onto', ...
        'the deflating subspaces for the selected eigenvalues', ...
        'rconde(1)', rconde(1), 'rconde(2)', rconde(2));
fprintf('%s\n%s\n%s = %8.1e, %s = %8.1e\n\n', ...
        'Reciprocals condition numbers for the left and right', ...
        'deflating subspaces', 'rcondv(1)', rcondv(1), ...
        'rcondv(2)', rcondv(2));

anorm = norm(A,2);
bnorm = norm(B,2);
abnorm = sqrt(anorm^2+bnorm^2);
fprintf('%s = %8.1e\n', ...
        'Approximate asymptotic error bound for selected eigenvalues   ', ...
        nag_machine_precision*abnorm/rconde(1));
fprintf('%s = %8.1e\n', ...
        'Approximate asymptotic error bound for the deflating subspaces', ...
        nag_machine_precision*abnorm/rcondv(2));

% diary off
