function nag_lapack_zggrqf_example

% diary nag_lapack_zggrqf_example.r
fprintf('nag_lapack_zggrqf example results\n\n');

% Find x that minimizes norm(c-Ax) subject to Bx = d .
m = nag_int(6);
n = nag_int(4);
p = nag_int(2);
a = [ 0.96-0.81i -0.03+0.96i -0.91+2.06i -0.05+0.41i;
     -0.98+1.98i -1.20+0.19i -0.66+0.42i -0.81+0.56i;
      0.62-0.46i  1.01+0.02i  0.63-0.17i -1.11+0.60i;
      0.37+0.38i  0.19-0.54i -0.98-0.36i  0.22-0.20i;
      0.83+0.51i  0.20+0.01i -0.17-0.46i  1.47+1.59i;
      1.08-0.28i  0.20-0.12i -0.07+1.23i  0.26+0.26i];
b = complex([ 1 0 -1  0;
              0 1  0 -1]); 
c = [-2.54+0.09i;
      1.65-2.26i;
     -2.11-3.96i;
      1.82+3.30i;
     -6.41+3.77i;
      2.07+0.66i];
d = complex([0;0]);

% Compute the generalized RQ factorization of (B,A) as
% A = ZRQ, B = TQ
[TQ, taub, ZR, taua, info] = nag_lapack_zggrqf(b, a);

% Set Qx = y. The problem reduces to:
% minimize (Ry - Z^Hc) subject to Ty = d

% Update c = Z^H*c -> minimize (Ry-c)
[cup, info] = nag_lapack_zunmqr( ...
                     'Left','Conjugate Transpose',ZR,taua,c);

% Solve Ty = d for last p elements
T12 = complex(triu(TQ(1:p,n-p+1:n)));

[y2, info] = nag_lapack_ztrtrs( ...
                    'Upper', 'No transpose', 'Non-unit', T12, d);

% (from Ry-c) R11*y1 + R12*y2 = c1 --> R11*y1 = c1 - R12*y2
% Update c1
c1 = cup(1:n-p) - ZR(1:n-p,n-p+1:n)*y2;

% Solve R11*y1 = c1 for y1
  R11 = complex(triu(ZR(1:n-p,1:n-p)));
[y1, info] = nag_lapack_ztrtrs( ...
                    'Upper', 'No transpose', 'Non-unit', R11, c1);

% Contruct y and backtransform for x = Q^Hy
y = [y1;y2];
[~, x, info] = nag_lapack_zunmrq( ...
                      'Left', 'Conjugate Transpose', TQ, taub, y);

disp('Constrained least squares solution');
disp(x);

res = a*x - c;
fprintf('Square root of the residual sum of squares\n%11.2e\n', ...
        norm(res));

% diary off
