function nag_lapack_zheev_example

% diary nag_lapack_zheev_example.r
fprintf('nag_lapack_zheev example results\n\n');

% upper triangular part of Hernitian matrix A
n = nag_int(4);
uplo = 'Upper';
a = [ 1,       2 - 1i,  3 - 1i,  4 - 1i;
      0 + 0i,  2 + 0i,  3 - 2i,  4 - 2i;
      0 + 0i,  0 + 0i,  3 + 0i,  4 - 3i;
      0 + 0i,  0 + 0i,  0 + 0i,  4 + 0i];

jobz = 'Vectors';
[z, w, info] = nag_lapack_zheev( ...
		      jobz, uplo, a);

disp('Eigenvalues');
disp(w);

% Normalize
for i = 1:n
  [~,k] = max(abs(real(z(:,i)))+abs(imag(z(:,i))));
  z(1:n,i) = z(1:n,i)*conj(z(k,i))/abs(z(k,i));
end
[ifail] = nag_file_print_matrix_complex_gen( ...
                 'General', ' ', z, 'Eigenvectors');

% Eigenvalue error bound
errbnd = nag_machine_precision*max(abs(w(1)),abs(w(end)));
% Eigenvector condition numbers
[rcondz, info] = nag_lapack_ddisna( ...
		        'Eigenvectors', n, n, w);

% Eigenvector error bounds
zerrbd = errbnd./rcondz;

fprintf('\n');
disp('Error estimate for the eigenvalues');
fprintf('%12.1e\n',errbnd);
disp('Error estimates for the eigenvectors');
fprintf('%12.1e',zerrbd);
fprintf('\n');

% diary off
