function nag_lapack_zhegvd_example

% diary nag_lapack_zhegvd_example.r
fprintf('nag_lapack_zhegvd example results\n\n');

% Upper triangular parts of Hermitian matrix A and positive definite matrix B
uplo = 'Upper';
n = nag_int(4);
a = [-7.36 + 0i,  0.77 - 0.43i, -0.64 - 0.92i,  3.01 - 6.97i;
      0    + 0i,  3.49 + 0i,     2.19 + 4.45i,  1.90 + 3.73i;
      0    + 0i,  0    + 0i,     0.12 + 0i,     2.88 - 3.17i;
      0    + 0i,  0    + 0i,     0    + 0i,    -2.54 + 0i];
b = [ 3.23 + 0i,  1.51 - 1.92i,  1.90 + 0.84i,  0.42 + 2.50i;
      0    + 0i,  3.58 + 0i,    -0.23 + 1.11i, -1.18 + 1.37i;
      0    + 0i,  0    + 0i,     4.09 + 0i,     2.33 - 0.14i;
      0    + 0i,  0    + 0i,     0    + 0i,     4.29 + 0i];

anorm = norm(a+a'-diag(diag(a)),1);
bnorm = norm(b+b'-diag(diag(b)),1);

% Generalized eigenvalues and eigenvectors for problem ABz = lambda z.
itype = nag_int(2);
jobz = 'Vectors';
[Z, L, w, info] = nag_lapack_zhegvd( ...
                         itype, jobz, uplo, a, b);

% Normalize: largest elements are real (preserving Z^HBZ = I)
for i = 1:n
  [~,k] = max(abs(real(Z(:,i)))+abs(imag(Z(:,i))));
  Z(:,i) = Z(:,i)*conj(Z(k,i))/abs(Z(k,i));
end

disp('Eigenvalues');
disp(w');
disp('Eigenvectors');
disp(Z);

% Get reciprocal condition number
[rcond, info] = nag_lapack_ztrcon( ...
                       '1', uplo, 'N', L);

rcondb = rcond^2;
fprintf('Estimate of reciprocal condition number for B\n%12.1e\n', rcondb);

% Eigenvector condition numbers
[rcondz, info] = nag_lapack_ddisna( ...
		        'Eigenvectors', n, n, w);

% Error bounds
t1 = 1/rcond;
t2 = nag_machine_precision*t1;
t3 = anorm*bnorm;
errbnd(1:n) = nag_machine_precision*t3 + (nag_machine_precision/rcondb).*abs(w);
zerrbd(1:n) = t1*t2 + (t2*t3)./rcondz;

fprintf('\n');
disp('Error estimate for the eigenvalues');
fprintf('%12.1e',errbnd);
fprintf('\n\n');
disp('Error estimates for the eigenvectors');
fprintf('%12.1e',zerrbd);
fprintf('\n');

% diary off
