function nag_lapack_zhsein_example

% diary nag_lapack_zhsein_example.r
fprintf('nag_lapack_zhsein example results\n\n');

% Complex matrix A
a = [ -3.97 - 5.04i, -4.11 + 3.70i, -0.34 + 1.01i,  1.29 - 0.86i;
       0.34 - 1.50i,  1.52 - 0.43i,  1.88 - 5.38i,  3.36 + 0.65i;
       3.31 - 3.85i,  2.50 + 3.45i,  0.88 - 1.08i,  0.64 - 1.48i;
      -1.10 + 0.82i,  1.81 - 1.59i,  3.25 + 1.33i,  1.57 - 3.44i];

% Reduce (all of) A to upper Hessenberg Form
n   = nag_int(4);
ilo = nag_int(1);
ihi = n;
[H, tau, info] = nag_lapack_zgehrd(ilo, ihi, a);

% Form Q
[Q, info] = nag_lapack_zunghr(ilo, ihi, H, tau);

% Schur factorize H = Y*T*Y^H
job   = 'Schur form';
compz = 'Vectors';
[~, w, ~, info] = nag_lapack_zhseqr( ...
                         job, compz, ilo, ihi, H, Q);

disp('Eigenvalues of A');
disp(w);

% Calculate eigenvectors of H for negative real part eigenvalues
select = (real(w) < 0);

job = 'Right';
eigsrc = 'QR';
initv = 'No initial vectors';
vl = [];
vr = complex(zeros(n,n));
[~, ~, VR, m, ifaill, ifailr, info] = ...
nag_lapack_zhsein( ...
       job, eigsrc, initv, select, H, w, vl, vr, n);

% Eigenvectors of A = Q*VR
side = 'Left';
trans = 'No transpose';
[z, info] = nag_lapack_zunmhr( ...
                   side, trans, ilo, ihi, H, tau, VR);

% Normalize eigenvectors: largest elements are real
for i = 1:m
  [~,k] = max(abs(real(z(:,i)))+abs(imag(z(:,i))));
  z(:,i) = z(:,i)*conj(z(k,i))/abs(z(k,i));
end

disp('Eigenvectors corresponding to eigenvalues with negative real part');
% Normalize eigenvectors before printing
disp(z/diag(z(1,:)));

% diary off
