function nag_lapack_ztgsna_example

% diary nag_lapack_ztgsna_example.r
fprintf('nag_lapack_ztgsna example results\n\n');

% Generalized Schur form pair (S,T)
n = nag_int(4);
S = [ 4 + 4i,  1 + 1i,  1 + 1i,  2 - 1i;
      0 + 0i,  2 + 1i,  1 + 1i,  1 + 1i;
      0 + 0i,  0 + 0i,  2 - 1i,  1 + 1i;
      0 + 0i,  0 + 0i,  0 + 0i,  6 - 2i];
T = [2,  1 + 1i,  1 + 1i,  3 - 1i;
      0 + 0i,  1 + 0i,  2 + 1i,  1 + 1i;
      0 + 0i,  0 + 0i,  1 + 0i,  1 + 1i;
      0 + 0i,  0 + 0i,  0 + 0i,  2 + 0i];

% Obtain scaled eigenvectors from Schur form
job    = 'Both';
howmny = 'All';
select = [false];
Q = complex(eye(n));
Z = Q;

[VL, VR, m, info] = nag_lapack_ztgevc( ...
                           job, howmny, select, S, T, Q, Z, n);

% Estimate condition numbers for eigenvalues and right eigenvectors
[rconde, rcondv, m, info] = ...
  nag_lapack_ztgsna( ...
         job, howmny, select, S, T, VL, VR, n);

disp('Reciprocal condition numbers for eigenvalues of (S,T)');
fprintf('%11.1e',rconde);
fprintf('\n\n');
disp('Reciprocal condition numbers for right eigenvectors of (S,T)');
fprintf('%11.1e',rcondv);
fprintf('\n\n');

% Calculate approximate error estimates
        
snorm = norm(S,1);
tnorm = norm(T,1);
stnorm = sqrt(snorm^2 + tnorm^2);

disp('Approximate error estimates for eigenvalues of (S,T)')
erre = nag_machine_precision*stnorm./rconde;
fprintf('%11.1e',erre);
fprintf('\n\n');
disp('Approximate error estimates for right eigenvectors of (S,T)')
errv = nag_machine_precision*stnorm./rcondv;
fprintf('%11.1e',errv);
fprintf('\n');

% diary off
