function nag_lapack_zungbr_example

% diary nag_lapack_zungbr_example.r
fprintf('nag_lapack_zungbr example results\n\n');

% Two cases of performing Singular Value Decomposition
% Case 1: m > n
ex1;
% Case 2: m < n
ex2;

% diary off

function ex1

  m = nag_int(6);
  n = nag_int(4);
  a = [ 0.96 - 0.81i, -0.03 + 0.96i, -0.91 + 2.06i, -0.05 + 0.41i;
       -0.98 + 1.98i, -1.20 + 0.19i, -0.66 + 0.42i, -0.81 + 0.56i;
        0.62 - 0.46i,  1.01 + 0.02i,  0.63 - 0.17i, -1.11 + 0.60i;
       -0.37 + 0.38i,  0.19 - 0.54i, -0.98 - 0.36i,  0.22 - 0.20i;
        0.83 + 0.51i,  0.20 + 0.01i, -0.17 - 0.46i,  1.47 + 1.59i;
        1.08 - 0.28i,  0.20 - 0.12i, -0.07 + 1.23i,  0.26 + 0.26i];
  
  % Reduce A to bidiagonal form
  [B, d, e, tauq, taup, info] =  ...
  nag_lapack_zgebrd(a);

  % Form PT explicitly (n-by-n, k = m)
  vect = 'P';
  [PT, info] = nag_lapack_zungbr( ...
                      vect, m, B, taup, 'm', n, 'n', n);

  % Form Q explicitly (m-by-n, k = n)
  vect = 'Q';
  [Q, info] = nag_lapack_zungbr( ...
                     vect, n, B, tauq, 'm', m, 'n', n);
  
  % Compute SVD of A using its bidiagonal form.
  c = [];
  uplo = 'Upper';
  [s, ~, VT, U, c, info] = nag_lapack_zbdsqr( ...
                                  uplo, d, e, PT, Q, c);

  disp('Example 1: singular values');
  disp(s(1:n)');
  disp('Example 1: right singular vectors, by row');
  disp(VT(1:n,:));
  disp('Example 1: left singular vectors, by column');
  disp(U);

function ex2
  m = nag_int(3);
  n = nag_int(4);
  a = [0.28 - 0.36i  0.50 - 0.86i -0.77 - 0.48i  1.58 + 0.66i;
      -0.50 - 1.10i -1.21 + 0.76i -0.32 - 0.24i -0.27 - 1.15i;
       0.36 - 0.51i -0.07 + 1.33i -0.75 + 0.47i -0.08 + 1.01i];
 
  % Reduce A to bidiagonal form
  [B, d, e, tauq, taup, info] =  ...
  nag_lapack_zgebrd(a);

  % Form P^T explicitly (m-by-n, k = m)
  vect = 'P';
  [PT, info] = nag_lapack_zungbr( ...
                      vect, m, B, taup, 'm', m, 'n', n);
  % Form Q explicitly (m-by-m, k = n)
  vect = 'Q';
  [Q, info] = nag_lapack_zungbr( ...
                     vect, n, B, tauq, 'm', m, 'n', m);
  
  % Compute SVD of A using its bidiagonal form.
  c = [];
  uplo = 'Lower';
  [s, ~, VT, U, c, info] = nag_lapack_zbdsqr( ...
                                  uplo, d, e, PT, Q, c, 'n', m);

  disp('Example 2: singular values');
  disp(s(1:m)');
  disp('Example 2: right singular vectors, by row');
  disp(VT);
  disp('Example 2: left singular vectors, by column');
  disp(U(:,1:m));
