function nag_ode_bvp_coll_sys_example

% diary nag_ode_bvp_coll_sys_example.r
fprintf('nag_ode_bvp_coll_sys example results\n\n');

% Set up initial values.
n     = nag_int(2);
k1max = 8;
kpmax = 15;
x0    = -1.0;
x1    = 1.0;
c     = zeros(k1max, n);

fprintf('  KP   K1   Chebyshev coefficients\n');
for kp = nag_int(10:5:kpmax)
  for k1 = nag_int(4:2:k1max)
    [c, ifail] = nag_ode_bvp_coll_sys(...
                        n, @cf, @bc, x0, x1, k1, kp);
    % Output results.
    fprintf('%4d ',kp, k1);
    for jord = 1:n
      for kind = 1:k1
        fprintf('%7.4f ',c(kind,jord));
        if mod(kind, 8) == 0 && kind ~= k1
          fprintf('\n          ');
        end
      end
      fprintf('\n          ');
    end
    fprintf('\n');
  end
end
% Now prepare to evaluate and plot the last solution.
fprintf('\n');
k1   = 8;
k1m1 = nag_int(k1-1);
m    = 9;
ia1  = nag_int(1);
x    = zeros(m, 1);
y    = zeros(m, n+1);

fprintf(['Last computed solution evaluated at %1d equally spaced ', ...
    'points\n\n'], m);
fprintf('    X       Y(1)      Y(2)\n');
for i = 1:m
  x(i) = (x0*(m-i) + x1*(i-1))/(m-1);
  fprintf('%8.4f  ', x(i));

  for jord = 1:n
    % Calling nag_fit_1dcheb_eval2 to evaluate the polynomial from its Chebyshev
    % representation.
    [y(i, jord), ifail] = nag_fit_1dcheb_eval2(...
                                 k1m1, x0, x1, c(:,jord), ia1, x(i));
    fprintf('%8.4f  ',y(i, jord));
  end
  fprintf('\n');

  % Calculate the absolute error at this point.
  y(i,n+1) = abs(y(i,1) - 1 + cos(x(i))/cos(1));
end
% Plot results.
fig1 = figure;
display_plot(x, y, fig1);
% print(fig1,'-dpng','-r75','nag_ode_bvp_coll_sys_fig1.png');
% print(fig1,'-deps','-r75','nag_ode_bvp_coll_sys_fig1.eps');

% diary off

function [j, rhs] = bc(i)
  % Define the boundary conditions.
  rhs = 0;
  if (i == 1)
    j = nag_int(1);
  else
    j = nag_int(-1);
  end

function result = cf(ii, jj, x)
  % Define the differential equation to be solved.
  if (jj == ii)
    result = 0;
  elseif (ii == 1 && jj == 2)
    result = 1;
  elseif (ii == 2 && jj == 1)
    result = -1;
  elseif (ii == 1)
    result = 0;
  else
    result = 1;
  end

function display_plot(x, y, fig1)
  axes1 = axes('Parent',fig1,'YMinorTick','on','YTick',[-2 -1 0 1 2],...
               'YColor',[0 0.447 0.741],...
               'XMinorTick','on');
  hold(axes1,'on');
  ylabel('y and y''');
  xlabel('x');
  title({'Two-point BVP for ODE System using Chebyshev series',...
         ' with Collocation & Least-Squares'});

  % Create multiple lines using matrix input to plot
  plot1 = plot(x,y(:,1:2),'Parent',axes1);
  set(plot1(1),'DisplayName','y''','Marker','+','LineStyle','--');
  set(plot1(2),'DisplayName','y','Marker','x');
  
  % Create axes
  axes2 = axes('Parent',fig1,'HitTest','off','Color','none',...
               'YMinorTick','on',...
               'YTick',[0 5e-07 1e-06 1.5e-06 2e-06],...
               'YColor',[0.85 0.325 0.098],...
               'YAxisLocation','right',...
               'Position',[0.13 0.11 0.775 0.815]);
  hold(axes2,'on');
  ylabel('Absolute Error');

  plot(x,y(:,3),'Parent',axes2,'DisplayName','y error','Marker','*',...
       'LineStyle',':','Color',[0.85 0.325 0.098]);

  legend1 = legend(axes1,'show');
  set(legend1,'Position',[0.17 0.77 0.09 0.11]);
