function nag_ode_bvp_fd_lin_gen_example

% diary nag_ode_bvp_fd_lin_gen_example.r
fprintf('nag_ode_bvp_fd_lin_gen example results\n\n');

% For communication with f.
global eps;

% Set up initial values before calling NAG routine for the first time.
mnp = nag_int(70);
a = 0;
b = 1;
tol = 0.001;
c = [1, 0; 0, 0];
d = [0, 0; 1, 0];
gam = [0; 1];
x = zeros(1,mnp);
np = nag_int(0);
n = nag_int(2);
xkeep = zeros(1,2);
npkeep = zeros(2);
ykeep = zeros(1,2);
espkeep = zeros(2);

for i = 1:2
    eps = 10^(-i);
    [c, d, gam, x, y, np, ifail] = ...
        nag_ode_bvp_fd_lin_gen(a, b, tol, @f, @g, c, d, gam, x, np, 'n', n, 'mnp', mnp);
    % save results.
    for j = 1:np 
        xkeep(j,i) = x(j);
        ykeep(j,i) = y(1,j);
    end
    % Store the number of points and epsilon for use in plot.
    npkeep(i) = np;
    espkeep(i) = eps;
    fprintf(' for eps = %7.4f, number of points required, np = %d\n', eps, np);
end

% Plot results.
fig1 = figure;
plot(xkeep(1:npkeep(1),1), ykeep(1:npkeep(1),1), '-+', ...
     xkeep(1:npkeep(2),2), ykeep(1:npkeep(2),2), ':x');
set(gca, 'YLim', [0 1.1]);
set(gca, 'XLim', [-0.01 1.01]);
title('Solution to Simple Second-order Problem');
xlabel('x');
ylabel('Solution');
legend(['\epsilon = ', num2str(espkeep(1))], ...
       ['\epsilon = ', num2str(espkeep(2))], 'Location', 'East');
% print(fig1,'-dpng','-r75','nag_ode_bvp_fd_lin_gen_fig1.png');
% print(fig1,'-deps','-r75','nag_ode_bvp_fd_lin_gen_fig1.eps');

% diary off

function f = f(x)
% For communication with main routine.
global eps;

% Evaluate matrix.
f=zeros(2,2);
f(1,2) = 1;
f(2,2) = -1/eps;

function g = g(x)
% Evaluate vector.
g=zeros(2,1);
