function nag_ode_bvp_shoot_bval_example

% diary nag_ode_bvp_shoot_bval_example.r
fprintf('nag_ode_bvp_shoot_bval example results\n\n');

% Initialize variables and arrays.
u = [0, 0; 0.5, 0.46; 1.15, -1.2];
v = [0, 0; 0, 1; 1, 1];
a = 0;
b = 5;
m1 = 6;
n = 3;
ykeep = zeros(m1,3);
xkeep = zeros(1,m1);

for id = 3:4
    tol = 5*10^(-id);

    [uOut, soln, w, ifail] = nag_ode_bvp_shoot_bval(u, v, a, b, tol, @fcn, nag_int(m1));
    if (ifail ~= 0)
        % Problems in integration, or incorrect parameters.  Print
        % message and exit.
        error('Warning: nag_ode_bvp_shoot_bval returned with ifail = %d ',ifail);
    else
        % Output results.
        fprintf('Results with tol = %1.3e\n\n',tol);
        fprintf('X-value and final solution\n\n');
        for i = 1:m1
            disp((sprintf('%d   %8.4f   %8.4f   %8.4f', i-1,...
                soln(1,i),soln(2,i),soln(3,i))));
            % Store results for plotting.
            ykeep(i,1) = soln(1,i);
            ykeep(i,2) = soln(2,i);
            ykeep(i,3) = soln(3,i);
            xkeep(i) = i-1;
        end
    end
    disp(' ');
end
% Plot results.
fig1 = figure;
display_plot(xkeep, ykeep)
% print(fig1,'-dpng','-r75','nag_ode_bvp_shoot_bval_fig1.png');
% print(fig1,'-deps','-r75','nag_ode_bvp_shoot_bval_fig1.eps');

% diary off

function f = fcn(x,y)
% Evaluate the derivatives.
f = zeros(3,1);
f(1) = tan(y(3));
f(2) = -0.032*tan(y(3))/y(2) - 0.02*y(2)/cos(y(3));
f(3) = -0.032/y(2)^2;

function  display_plot(xkeep, ykeep)
% Plot curves.
plot(xkeep, ykeep(:,1), '-+',...
     xkeep, ykeep(:,2), '--x',...
     xkeep, ykeep(:,3), ':*');
% Add title.
title({'Solution of Two-point Boundary-value Problem',...
    ['using Runge-Kutta-Merson and Newton Correction in a ', ...
    'Shooting Method']});
% Label the axes.
xlabel('x');
ylabel('Solution');
% Add legend.
legend('height','angle','velocity','location','Best');
