function nag_ode_bvp_shoot_genpar_algeq_example

% diary nag_ode_bvp_shoot_genpar_algeq_example.r
fprintf('nag_ode_bvp_shoot_genpar_algeq example results\n\n');

% For communication with prsol.
global ykeep ncall xkeep;

% Initialize variables and arrays.
p   = [1.2; 0.032; 2.5;0.2];
n1  = nag_int(3);
pe  = [0.001; 0.001; 0.001; 0.001];
pf  = [1e-06; 1e-06; 1e-06; 1e-06];
e   = [1e-05; 1e-05; 1e-05];
dp  = [0; 0; 0; 0];
swp = zeros(3, 6);
icount = nag_int(0);
ymax   = 0;
ncall  = 0;
ykeep  = zeros(1,3);
xkeep  = zeros(1,1);

fprintf('   x        y(1)        y(2)        y(3)\n')

[p, pf, dp, swp, w, ifail] = ...
  nag_ode_bvp_shoot_genpar_algeq(...
        p, n1, pe, pf, e, dp, swp, icount, @range, @bc, @fcn, ...
        @eqn, @constr, ymax, @monit, @prsol);

% Plot results.
fig1 = figure;
display_plot(xkeep,ykeep)
% print(fig1,'-dpng','-r75','nag_ode_bvp_shoot_genpar_algeq_fig1.png');
% print(fig1,'-deps','-r75','nag_ode_bvp_shoot_genpar_algeq_fig1.eps');

% diary off

function z = prsol(z, y, n)
  % Obtains values for solution and stores them for plotting.

  % For communication with main routine.
  global ykeep ncall xkeep;
  ncall = ncall+1;
  fprintf(' %6.4f',z);
  xkeep(ncall,1) = z;
  fprintf('%10.4f  ', y(1:n));
  fprintf('\n');
  z = z+0.5;
  if (abs(z-5) < 0.25)
    z = 5;
  end
  ykeep(ncall,:) = y;

function x = range(npoint, p, m)
  % Specify the break points.
  x = zeros(npoint,1);
  x(1) = 0;
  x(2) = p(3);
  x(3) = 5;

function [g1, g2] = bc(p, m, n)
  % Specify boundary conditions.
  g1 = [0, 0.5, p(1)];
  g2 = [0, 0.45, -1.2];

function f = fcn(x, y, n, p, m, interval)
  % Evaluate derivative vector.
  f = zeros(n,1);
  f(1) = tan(y(3));
  if (interval == 1)
    f(2) = -0.032d0*tan(y(3))/y(2) - 0.02d0*y(2)/cos(y(3));
    f(3) = -0.032d0/y(2)^2;
  else
    f(2) = -p(2)*tan(y(3))/y(2) - p(4)*y(2)/cos(y(3));
    f(3) = -p(2)/y(2)^2;
  end

function eq = eqn(q,p,m)
  % Specify additional algebraic equations.
  eq = zeros(q,1);
  eq(1) = 0.02 - p(4) - 1.0d-5*p(3);

function result = constr(p,n)
  % Check constraints.
  result = true;
  for i=1:n
    if p(i) < 0
      result = false;
    end
  end
  if p(3) > 5
    result = false;
  end

function monit(istate, iflag, ifail1, p, m, f, pnorm, pnorm1, eps, d)
  % Allows monitoring of parameter values during solution (not used here).

function display_plot(xkeep,ykeep)
  % Plot the results.
  plot(xkeep, ykeep(:,1), '-+', ...
       xkeep, ykeep(:,2), '--x', ...
       xkeep, ykeep(:,3), ':*');
  % Add title.
  title('Projectile in Two Media Problem');
  % Label the axes.
  xlabel('x');
  ylabel('Solution');
  % Add a legend.
  legend('height','velocity','angle','Location','SouthWest');

  % Now get the range of the x and y values, and use them to make nice axes.
  xmin = min(xkeep);
  xmax = max(xkeep);
  ymin = round(min(min(ykeep))-1);
  ymax = round(max(max(ykeep))+1);
  axis([xmin xmax ymin ymax]);

  stem(xkeep, ykeep, ymin, ymax, 'determined interface');

function stem(x, y, ymin, ymax, label)
  % Find the x bin that the crossover point between y2 and y3 lies in.
  for i = 1:length(x)
    if y(i,2) > y(i,3)
      break
    end
  end
  % Use interpolation to find the crossover point more accurately.
  m(1:2) = (y(i,2:3) - y(i-1,2:3))/(x(i)-x(i-1));
  c(1:2) = y(i,2:3) - m(1:2)*x(i);
  xx = (c(2)-c(1))/(m(1)-m(2));
  hold on
  % Draw line, and add label.
  plot([xx,xx],[ymin,ymax],'k-s');
  text('Position', [xx-0.1,0.5], 'String', label, 'Rotation', 90.0)
