function nag_ode_ivp_adams_rootdiag_example

% diary nag_ode_ivp_adams_rootdiag_example.r
fprintf('nag_ode_ivp_adams_rootdiag example results\n\n');

neqf   = nag_int(2);
neqg   = nag_int(2);
vectol = true;
atol   = [1e-06;  1e-06];
rtol   = [0.0001; 0.0001];
onestp = false;
crit   = true;
tcrit  = 10;
hmax   = 0;
maxstp = nag_int(0);
lrwork = 23*(1+neqf) + 14*neqg;
alterg = false;
sophst = true;

lrwork = 23*(1+neqf) + 14*neqg;
rwork  = zeros(lrwork,1);
liwork = 21 + 4*neqg;
iwork  = zeros(liwork, 1, nag_int_name);

% Adams method setup
[statefOut, altergOut, rwork, iwork, ifail] = ...
  nag_ode_ivp_adams_setup(...
        'Start', neqf, vectol, atol, rtol, onestp, crit, tcrit, hmax, ...
        maxstp, neqg, alterg, sophst, rwork, iwork);

% Integrate from t = 0 to 10
t    = 0;
tout = 10;
y    = [0; 1];

[t, y, root, rwork, iwork, ifail] = ...
    nag_ode_ivp_adams_roots(...
          @fcn, t, y, tout, @g, neqg, rwork, iwork);

% Display solution and root statistics.
fprintf('Solution and root functions at t = %7.4f is:\n',t);

[index, itype, events, resids, ifail] = nag_ode_ivp_adams_rootdiag(neqg, rwork, iwork);

fprintf('\n y :');
fprintf('%10.4f',y);
fprintf('\n g :');
fprintf('%10.4f',resids);
fprintf('\n\nRoot Diagnostics:\n');
fprintf('  Index of event function,  k : %d\n',index);
fprintf('  Type of root                : ');
if itype==1
   fprintf('simple root\n');
elseif itype==2
   fprintf('root of even multiplicity\n');
elseif itype==3
   fprintf('root of odd multiplicity\n');
else
   fprintf('root of high multiplicity or cluster?\n');
end
fprintf('  event function sign change  : ');
if events(index)==-1
   fprintf('positive to negative\n');
elseif events(index)==1
   fprintf('negative to positive\n');
else events(index)==2
   fprintf('no change in sign\n');
end

% diary off

function f = fcn(neqf, x, y)
  f=zeros(neqf,1);
  f(1)=y(2);
  f(2)=-y(1);

function result = g(neqf, x, y, yp, k)
  if (k == 1)
    result = yp(1)+0.5;
  else
    result = y(1)-sqrt(0.5);
  end
