function nag_ode_ivp_rkts_onestep_example

% diary nag_ode_ivp_rkts_onestep_example.r
fprintf('nag_ode_ivp_rkts_onestep example results\n\n');

% Set initial conditions and input
method = nag_int(2);
tstart = 0;
tend = 2*pi;
yinit = [0;1];
hstart = 0;
thresh = [1e-08; 1e-08];
n = nag_int(2);
tol0 =  1.0E-4;
ynow = zeros(20, n);
tnow = zeros(20, 1);
err1 = zeros(20, 2);
err2 = zeros(20, 2);

% Set control for output
tol = 10.0*tol0;

% Run through the calculation twice with two tolerance values
for i = 1:2

  tol = tol*0.1;

  % Call setup function
  [iwsav, rwsav, ifail] = nag_ode_ivp_rkts_setup(tstart, tend, yinit, tol, thresh, method);


  tnow(1) = tstart;
  ynow(1,:) = yinit;
  j=1;
  while tnow(j) < tend
    j=j+1;
    [tnow(j), ynow(j, :), ypnow, user, iwsav, rwsav, ifail] = ...
      nag_ode_ivp_rkts_onestep(@f, n, iwsav, rwsav);

    err1(j, i) =  ynow(j, 1)-sin(tnow(j));
    err2(j, i) =  ynow(j, 2)-cos(tnow(j));

  end

  fprintf('\nCalculation with TOL = %8.1e:\n\n', tol);
  [fevals, stepcost, waste, stepsok, hnext, iwsav, ifail] = nag_ode_ivp_rkts_diag(iwsav, rwsav);
  fprintf('  Number of evaluations of f = %d\n', fevals);

  % Store the t values for use in plotting errors
  if i == 1
    tnow1 = tnow;
  end

  % Store value of j
  npts(i) = j;
end

% Plot results
fig1 = figure;
title({['First-order ODEs solution by single stepping'],...
          ['Medium-order Runge-Kutta Method, Two Tolerances']});
hold on;
axis([0 10 -1.2 1.2]);
xlabel('t');
ylabel('Solution (y, y'')');
plot(tnow(1:npts(2)), ynow(1:npts(2), 1), '-xr');
text(ceil(tnow(npts(2))), ynow(npts(2), 1), 'y', 'Color', 'r');
plot(tnow(1:npts(2)), ynow(1:npts(2), 2), '-xg');
text(ceil(tnow(npts(2))), ynow(npts(2), 2), 'y''', 'Color', 'g');
% Plot errors with a different (log) scale
ax1 = gca;
ax2 = axes('Position',get(ax1,'Position'),...
           'XAxisLocation','bottom','YAxisLocation','right',...
           'YScale','log','Color','none','XColor','k','YColor','k');
hold on;
axis([0 10 1e-9 1e-4]);
ylabel('abs(Error)');
plot(ax2, tnow1(1:npts(1)), abs(err1(1:npts(1), 1)), '-*b');
text(ceil(tnow1(npts(1))), err1(npts(1), 1) - 1e-5, ...
     'y-error (tol=0.001)', 'Color', 'b');
plot(ax2, tnow, abs(err1(:, 2)), '-sm');
text(ceil(tnow(npts(2))), err1(npts(2), 2), 'y-error (tol=0.0001)', ...
     'Color', 'm');
% print(fig1,'-dpng','-r75','nag_ode_ivp_rkts_onestep_fig1.png');
% print(fig1,'-deps','-r75','nag_ode_ivp_rkts_onestep_fig1.eps');
hold off;

% diary off

function [yp, user] = f(t, n, y, user)
  yp = [y(2); -y(1)];
