function nag_ode_ivp_rkts_reset_tend_example

% diary nag_ode_ivp_rkts_reset_tend_example.r
fprintf('nag_ode_ivp_rkts_reset_tend example results\n\n');

% Set initial conditions and input
method = nag_int(-3);
tstart = 0;
tend   = 6*pi;
ecc    = 0.7;
yinit  = [1-ecc; 0; 0; sqrt((1+ecc)/(1-ecc))];
hstart = 0;
thresh = [1e-10; 1e-10; 1e-10; 1e-10];
n      = nag_int(4);
npts   = 96;
tol0   =  1.0E-4;
ysave  = zeros(npts+1, n);
tsave  = zeros(npts+1, 1);

% Set control for output
tol  = 10.0*tol0;
tinc = (tend-tstart)/npts;




% We run through the calculation twice with two tolerance values
for i = 1:2

  tol = tol*0.1;

  tendnu = tstart + tinc;

  % Call setup function
  [iwsav, rwsav, ifail] = nag_ode_ivp_rkts_setup(tstart, tendnu, yinit, tol, thresh, method);

  fprintf('\nCalculation with TOL = %8.1e\n\n', tol);
  fprintf('    t         y1        y2        y3        y4\n');
  fprintf(' %6.3f   %7.3f   %7.3f   %7.3f   %7.3f\n', tstart, yinit);

  tsave(1) = tstart;
  ysave(1, :) = yinit;
  tnow = tstart;
  j=1;
  while tnow < tend
    [tnow, ynow, ypnow, user, iwsav, rwsav, ifail] = ...
      nag_ode_ivp_rkts_onestep(@f, n, iwsav, rwsav);

    if tnow >= tendnu
      % Just print some of the results
      if rem(j, 16) == 0
        fprintf(' %6.3f   %7.3f   %7.3f   %7.3f   %7.3f\n', tnow, ynow);
      end
      j = j+1;
      tsave(j) = tnow;
      ysave(j, :) = ynow;
      if tnow < tend
        tendnu = tendnu + tinc;
        [iwsav, rwsav, ifail] = nag_ode_ivp_rkts_reset_tend(tendnu, iwsav, rwsav);
      end
    end
  end

  [fevals, stepcost, waste, stepsok, hnext, iwsav, ifail] = nag_ode_ivp_rkts_diag(iwsav, rwsav);
  fprintf('Cost of the integration in evaluations of f is %d\n', fevals);

end

% Plot results
fig1 = figure;
hold on;
xlabel('Orbit - x');
ylabel('Orbit - y');
plot(ysave(:,1), ysave(:, 2), '-xr');
% Plot errors with a different scale
ax1 = gca;
ax2 = axes('Position',get(ax1,'Position'),...
           'XAxisLocation','top',...
           'YAxisLocation','right',...
           'Color','none',...
           'XColor','k','YColor','k');
hold on;
axis([-0.25 0.05 -0.1 0.1]);
xlabel('x Deviation from True Ellipse');
ylabel('y Deviation from True Ellipse');
d = dev(ysave(:,1), ysave(:, 2));
plot(ax2, d.*cos(tsave), d.*sin(tsave),'-*b');
text(-0.05, 0, 'Deviation', 'Color', 'b');
% print(fig1,'-dpng','-r75','nag_ode_ivp_rkts_reset_tend_fig1.png');
% print(fig1,'-deps','-r75','nag_ode_ivp_rkts_reset_tend_fig1.eps');

% diary off

function [yp, user] = f(t, n, y, user)
  r = sqrt(y(1)^2+y(2)^2);
  yp = [y(3); y(4); -y(1)/r^3; -y(2)/r^3];

function [d] = dev(x, y)
  ecc = 0.7;
  d =  abs((x+ecc).*(x+ecc)+y.*y/(ecc*ecc)-1);
