function nag_ode_ivp_stiff_contin_example

% diary nag_ode_ivp_stiff_contin_example.r
fprintf('nag_ode_ivp_stiff_contin example results\n\n');

% Initialize variables and arrays for setup routine
n      = nag_int(3);
ord    = nag_int(11);
sdy    = nag_int(ord + 3);
petzld = false;
con    = zeros(6);
tcrit  = 0;
hmin   = 1.0e-10;
hmax   = 10;
h0     = 0;
maxstp = nag_int(200);
mxhnil = nag_int(5);
lrwork = 50 + 4*n;
rwork  = zeros(lrwork);
[const, rwork, ifail] = nag_ode_ivp_stiff_blend(n, sdy, ord, con, tcrit, hmin, hmax, h0, ...
                              maxstp, mxhnil, 'Average-L2', rwork);

% Setup for banded Jacbian using nag_ode_ivp_stiff_bandjac_setup
ml     = nag_int(1);
mu     = nag_int(2);
nwkjac = nag_int(15);
[rwork, ifail] = nag_ode_ivp_stiff_bandjac_setup(n, n, 'Analytical', ml, mu, nwkjac, n, rwork);

% Initialize variables and arrays for integration
t      = 0;
tout   = 5;
y      = [1; 0; 0];
rtol   = [0.0001];
atol   = [1e-07; 1e-08; 1e-07];
itol   = nag_int(2);
inform = zeros(23, 1, nag_int_name);
ysave  = zeros(n, sdy);
wkjac  = zeros(nwkjac, 1);
jacpvt = zeros(n, 1, nag_int_name);
itask  = nag_int(1);
itrace = nag_int(0);

% Integrate ODE from t=0 to t=tout, no monitoring, using nag_ode_ivp_stiff_exp_bandjac
[t, y, ydot, rwork, inform, ysave, wkjac, jacpvt, ifail] = ...
    nag_ode_ivp_stiff_exp_bandjac(t, tout, y, rwork, rtol, atol, itol, inform, @fcn, ysave, @jac, ...
          wkjac, jacpvt, 'nag_ode_ivp_stiff_exp_fulljac_dummy_monit', itask, itrace);

fprintf('Solution y and derivative y'' at t = %7.4f is:\n',t);
fprintf('\n %10s %10s\n','y','y''');
for i=1:n
  fprintf(' %10.4f %10.4f\n',y(i),ydot(i));
end

% Call to nag_ode_ivp_stiff_contin to alow continuation of integration up to t=10.
[rwork, ifail] = nag_ode_ivp_stiff_contin(n, tcrit, h0, hmin, hmax, maxstp, mxhnil, rwork);

tout = 10;
% Integrate ODE from t to t=tout, no monitoring, using nag_ode_ivp_stiff_exp_bandjac
[t, y, ydot, rwork, inform, ysave, wkjac, jacpvt, ifail] = ...
    nag_ode_ivp_stiff_exp_bandjac(t, tout, y, rwork, rtol, atol, itol, inform, @fcn, ysave, @jac, ...
          wkjac, jacpvt, 'nag_ode_ivp_stiff_exp_fulljac_dummy_monit', itask, itrace);

fprintf('Solution y and derivative y'' at t = %7.4f is:\n',t);
fprintf('\n %10s %10s\n','y','y''');
for i=1:n
  fprintf(' %10.4f %10.4f\n',y(i),ydot(i));
end

% diary off

function [f, ires] = fcn(neq, t, y, ires)
% Evaluate derivative vector.
f = zeros(3,1);
f(1) = -0.04d0*y(1) + 1.0d4*y(2)*y(3);
f(2) =  0.04d0*y(1) - 1.0d4*y(2)*y(3) - 3.0d7*y(2)*y(2);
f(3) =                                  3.0d7*y(2)*y(2);

function p = jac(neq, t, y, h, d, ml, mu, pIn)
% Evaluate the Jacobian.
p = zeros(ml+mu+1, neq);
hxd = h*d;
p(1,1) = 1.0d0 - hxd*(-0.04d0);
p(2,1) = -hxd*(1.0d4*y(3));
p(3,1) = -hxd*(1.0d4*y(2));
p(1,2) = -hxd*(0.04d0);
p(2,2) = 1.0d0 - hxd*(-1.0d4*y(3)-6.0d7*y(2));
p(3,2) = -hxd*(-1.0d4*y(2));
p(1,3) = -hxd*(6.0d7*y(2));
p(2,3) = 1.0d0 - hxd*(0.0d0);
