function nag_opt_miqp_mps_read_example

% diary nag_opt_miqp_mps_read_example.r
fprintf('nag_opt_miqp_mps_read example results\n\n');

% Open the data file for reading to get max sizes for problem
nin     = nag_int(7);
mode    = nag_int(0);
[ifail] = nag_file_open(nin, 'nag_opt_miqp_mps_read.opt', mode);

% Call nag_opt_miqp_mps_read in query mode
n       = nag_int(0);
m       = nag_int(0);
nz      = nag_int(0);
ncolh   = nag_int(0);
nnzh    = nag_int(0);
lintvar = nag_int(-1);
mpslst  = nag_int(0);
pnames  = {'        ', '        ', '        ', '        ', '        '};
[n, m, nz, ncolh, nnzh, lintvar, iobj, a, irowa, iccola, bl, bu, pnames, ...
  nname, crname, h, irowh, iccolh, minmax, intvar, ifail] = ...
  nag_opt_miqp_mps_read(...
        nin, n, m, nz, ncolh, nnzh, lintvar, mpslst, pnames);

% Close the data file
[ifail] = nag_file_close(nin);

% Re-open the data file for reading the problem
[ifail] = nag_file_open(nin, 'nag_opt_miqp_mps_read.opt', mode);

% Call nag_opt_miqp_mps_read to read the problem after setting max sizes
maxm     = m;
maxn     = n;
maxnnz   = nz;
maxnnzh  = nnzh;
maxncolh = ncolh;
[n, m, nz, ncolh, nnzh, lintvar, iobj, a, irowa, iccola, bl, bu, pnames, ...
  nname, crname, h, irowh, iccolh, minmax, intvar, ifail] = ...
  nag_opt_miqp_mps_read(...
        nin, n, m, nz, ncolh, nnzh, lintvar, mpslst, pnames);

% Close the data file
[ifail] = nag_file_close(nin);

% Data has been read, set up and run the solver

% Initialise workspace
[cw, iw, rw, ifail] = nag_opt_qpconvex2_sparse_init;

% Turn off printing and listing
[cw, iw, rw, ifail] = nag_opt_qpconvex2_sparse_option_string('NoList', cw, iw, rw);
[cw, iw, rw, ifail] = nag_opt_qpconvex2_sparse_option_string('Print Level = 0', cw, iw, rw);

% We have no explicit objective vector so set LENC = 0; the
% objective vector is stored in row IOBJ of ACOL.
lenc   = nag_int(0);
objadd = 0;
start  = 'c';

c      = zeros(lenc, 1);
helast = zeros(n+m, 1, nag_int_name);
hs     = zeros(n+m, 1, nag_int_name);
x      = zeros(n+m, 1);
for i=1:n+m
  x(i) = min(max(0, bl(i)),bu(i));
end
ns     = nag_int(0);

% Store the non zeros of H, iccolh and irowh in user for use by qphx
user = {h, iccolh, irowh};

% Call nag_opt_qpconvex2_sparse_solve to solve the problem
[hs, x, pi, rc, ns, ninf, sinf, obj, user, cw, iw, rw, ifail] = ...
  nag_opt_qpconvex2_sparse_solve(...
        start, @qphx, m, n, lenc, ncolh, iobj, objadd, pnames{1}, a, irowa, ...
        iccola, bl, bu, c, crname, helast, hs, x, ns, cw, iw, rw, 'user', user);
fprintf('Number of variables : %9d\n',n);
fprintf('Number of contraints: %9d\n',3);
fprintf('Minimum value       : %9.4f\n\n',obj);
fprintf('Variable values at minimum:\n');
fprintf('    %9.4f %9.4f %9.4f\n',x(1:n));
fprintf('Linear contrained values:\n');
fprintf('    %9.4f %9.4f %9.4f\n',x(n+1:n+3));

% diary off

function [hx, user] = qphx(ncolh, x, nstate, user)
  hx = zeros(ncolh, 1);

  h      = user{1};
  iccolh = user{2};
  irowh  = user{3};

  for icol = 1:ncolh
    start = iccolh(icol);
    endd  = iccolh(icol+1)-1;

    for idx = start:endd
      irow     = irowh(idx);
      hx(irow) = hx(irow) + x(icol)*h(idx);
      if irow ~= icol
        hx (icol) = hx(icol) + x(irow)*h(idx);
      end
    end
  end
