function nag_pde_1d_parab_convdiff_dae_example

% diary nag_pde_1d_parab_convdiff_dae_example.r
fprintf('nag_pde_1d_parab_convdiff_dae example results\n\n');

fprintf('example 1\n\n');
ex1;
fprintf('\nexample 2\n\n');
ex2;

% diary off

function ex1
  npde  = nag_int(2);
  npts  = nag_int(141);
  ncode = nag_int(2);
  neqn  = npde*npts+ncode;
  ts    = 0;
  tout  = 0.5;
  x     = zeros(npts,1);
  xi    = [0; 1];
  rtol  = [0.00025];
  atol  = [1e-05];
  itol  = nag_int(1);
  norm_p = '1';
  laopt  = 'S';
  algopt = zeros(30,1);
  algopt(1)  = 1;
  algopt(29) = 0.1;
  algopt(30) = 1.1;
  rsave  = zeros(11000, 1);
  isave  = zeros(15700, 1, nag_int_name);
  itask  = nag_int(1);
  itrace = nag_int(0);
  ind    = nag_int(0);

  % Initialise mesh
  dx = 1/(double(npts)-1);
  x  = [0:dx:1];

  u = exact(ts, npde, x, npts);
  u = reshape(u, [2*npts,1]);
  u(neqn-1) = u(1) - u(2);
  u(neqn)   = u(neqn-2) + u(neqn - 3);

  [ts, u, rsave, isave, ind, ifail] = ...
  nag_pde_1d_parab_convdiff_dae( ...
         npde, ts, tout, @pdedef, @numflx1, @bndary1, u, x, ncode, ...
         @odedef, xi, rtol, atol, itol, norm_p, laopt, ...
         algopt, rsave, isave, itask, itrace, ind);
  
  xout = x(1:20:npts);
  nop = length(xout);

  uout(1, :) = u(1:20*npde:npts*npde);
  uout(2, :) = u(2:20*npde:npts*npde);

  ue = exact(tout, npde, xout, nop);

  fprintf('\n%8s%12s%12s%12s%12s\n','x','U_1','u_1','U_2','u_2');
  for i=1:nop
    fprintf('%12.4f%12.4f%12.4f%12.4f%12.4f\n', ...
            xout(i), uout(1,i), ue(1,i), uout(2,i), ue(2,i));
  end

  fig1 = figure;
  plot(x,u(1:2:npts*npde),x,u(2:2:npts*npde));
  title('Convection-diffusion solution at t=0.5');
  xlabel('x');
  ylabel('u_1, u_2');
  legend('u_1','u_2','location','NorthWest');
  % print(fig1,'-dpng','-r75','nag_pde_1d_parab_convdiff_dae_fig1.png');
  % print(fig1,'-deps','-r75','nag_pde_1d_parab_convdiff_dae_fig1.eps');

function ex2
  npde  = nag_int(3);
  npts  = nag_int(141);
  ncode = nag_int(0);
  nxi   = nag_int(0);
  neqn  = npde*npts+ncode;
  ts    = 0;
  x     = zeros(npts,1);
  xi    = [];
  rtol  = [0.0005];
  atol  = [0.005];
  itol  = nag_int(1);
  norm_p = '2';
  laopt  = 'B';
  algopt = zeros(30,1);
  algopt(1) = 2;
  algopt(6) = 2;
  algopt(7) = 2;
  algopt(13) = 0.005;
  rsave  = zeros(21000, 1);
  isave  = zeros(25700, 1, nag_int_name);
  itask  = nag_int(1);
  itrace = nag_int(0);
  ind    = nag_int(0);

  % Initialise mesh
  dx = 1/(double(npts)-1);
  x  = [0:dx:1];

  u = uvinit(x);

  for j=1:2
    tout = 0.1*j;
    [ts, u, rsave, isave, ind, ifail] = ...
    nag_pde_1d_parab_convdiff_dae( ...
           npde, ts, tout, @pdedef, @numflx2, @bndary2, u, x, ncode, ...
           @odedef, xi, rtol, atol, itol, norm_p, laopt, ...
           algopt, rsave, isave, itask, itrace, ind,'nxi',nxi);

    density = u(1,:);
    velocity = u(2,:)./density;
    pressure = 0.4*density.*(u(3,:)./density-velocity.^2/2);
    if j==1
      fig2 = figure;
    else
      fig3 = figure;
    end
    plot(x,density,x,velocity,x,pressure);
    text = sprintf('Shock tube problem, t = %3.1f',tout);
    title(text);
    xlabel('x');
    legend('density','velocity','pressure')
  end

  nsteps = 50*((isave(1)+25)/50);
  nfuncs = 50*((isave(2)+25)/50);
  njacs = isave(3);
  niters = isave(5);
  fprintf('\n Number of time steps           (nearest 50) = %6d\n',nsteps);
  fprintf(' Number of function evaluations (nearest 50) = %6d\n',nfuncs);
  fprintf(' Number of Jacobian evaluations (nearest  1) = %6d\n',njacs);
  fprintf(' Number of iterations           (nearest  1) = %6d\n',niters);

  % print(fig2,'-dpng','-r75','nag_pde_1d_parab_convdiff_dae_fig2.png');
  % print(fig2,'-deps','-r75','nag_pde_1d_parab_convdiff_dae_fig2.eps');
  % print(fig3,'-dpng','-r75','nag_pde_1d_parab_convdiff_dae_fig3.png');
  % print(fig3,'-deps','-r75','nag_pde_1d_parab_convdiff_dae_fig3.eps');


function [g, ires] = bndary1(npde, npts, t, x, u, ncode, v, vdot, ibnd, ires)
  g = zeros(npde, 1);
  np1 = npts-1;
  if (ibnd == 0)
    ue = exact(t,npde,x(1),1);
    g(1) = u(1,1) + u(2,1) - ue(1,1) - ue(2,1);
    dudx = (u(1,2)-u(2,2)-u(1,1)+u(2,1))/(x(2)-x(1));
    g(2) = vdot(1) - dudx;
  else
    ue = exact(t,npde,x(npts),1);
    g(1) = u(1,npts) - u(2,npts) - ue(1,1) + ue(2,1);
    dudx = (u(1,npts)+u(2,npts)-u(1,np1)-u(2,np1))/(x(npts)-x(np1));
    g(2) = vdot(2) + 3*dudx;
  end

function [u] = exact(t,npde,x,npts)
  u = zeros(npde,npts);
  pi2 = 2*pi;
  for i = 1:double(npts)
   f = exp(pi*(x(i)-3*t))*sin(pi2*(x(i)-3*t));
   g = exp(-pi2*(x(i)+t))*cos(pi2*(x(i)+t));
   u(1,i) = f + g;
   u(2,i) = f - g;
  end

function [flux, ires] = numflx1(npde, t, x, ncode, v, uleft, uright, ires)
  flux = zeros(npde, 1);
  flux(1) = (3*uleft(1)-uright(1)+3*uleft(2)+uright(2))/2;
  flux(2) = (3*uleft(1)+uright(1)+3*uleft(2)-uright(2))/2;

function [r, ires] = odedef(npde, t, ncode, v, vdot, nxi, xi, ucp, ucpx, ...
                            ucpt, ires)
  r = zeros(ncode, 1);
  if (ires ~= -1)
    r(1) = v(1) - ucp(1,1) + ucp(2,1);
    r(2) = v(2) - ucp(1,2) - ucp(2,2);
  end

function [p, c, d, s, ires] = pdedef(npde, t, x, u, ux, ncode, v, vdot, ires)
  p = eye(npde);
  c = ones(npde,1);
  d = zeros(npde, 1);
  s = d;

function [g, ires] = bndary2(npde, npts, t, x, u, ncode, v, vdot, ibnd, ires)
  
  if (ibnd == 0)
    g(1) = u(1,1) - 1;
    g(2) = u(2,1);
    g(3) = u(3,1) - 2.5;
  else
    g(1) = u(1,npts) - 1/8;
    g(2) = u(2,npts);
    g(3) = u(3,npts) - 1/4;
  end

function [flux, ires] = numflx2(npde, t, x, ncode, v, uleft, uright, ires)

  % Use Rho scheme rputine nag_pde_1d_parab_euler_roe (Osher scheme nag_pde_1d_parab_euler_osher is an alternative)
  gamma = 1.4;
  [flux, ifail] = nag_pde_1d_parab_euler_roe( ...
                         uleft, uright, gamma);

function [u] = uvinit(x)
  n = size(x,2);
  u = zeros(3,n);
  for i = 1:n
    if x(i)<1/2
      u(1,i) = 1;
      u(3,i) = 2.5;
    elseif x(i)== 1/2
      u(1,i) = (1+1/8)/2;
      u(3,i) = (2.5+1/4)/2;
    else
      u(1,i) = 1/8;
      u(3,i) = 1/4;
    end
  end
