function nag_pde_1d_parab_convdiff_example

% diary nag_pde_1d_parab_convdiff_example.r
fprintf('nag_pde_1d_parab_convdiff example results\n\n');

fprintf('Example 1\n');
ex1;
fprintf('\n\nExample 2\n');
ex2;

% diary off

function ex1
  ts   = 0;
  tout = 0.1;
  npde = 2;
  npts = 101;
  x    = [0:0.01:1];
  u    = exact(ts,npde,x,npts);
  acc  = [1e-04;  1e-05];
  tsmax  = 0;
  rsave  = zeros(6695, 1);
  isave  = zeros(226, 1, nag_int_name);
  itask  = nag_int(1);
  itrace = nag_int(0);
  ind    = nag_int(0);

  [ts, u, rsave, isave, ind, ifail] = ...
  nag_pde_1d_parab_convdiff( ...
         ts, tout, @pdedef, @numflx1, @bndary1, u, x, acc, ...
         tsmax, rsave, isave, itask, itrace, ind);

  uerr  = exact(ts,npde,x,npts) - u;
  l2err = norm(uerr);

  fprintf(' At t = %7.4f the L2-error in computed solution = %7.4f\n', ...
          ts, l2err);
  
  nsteps = 5*((isave(1)+2)/5);
  nfuncs = 50*((isave(2)+25)/50);
  njacs = 10*((isave(3)+5)/10);
  niters = 50*((isave(5)+25)/50);
  fprintf('\n Number of time steps           (nearest 5)  = %6d\n',nsteps);
  fprintf(' Number of function evaluations (nearest 50) = %6d\n',nfuncs);
  fprintf(' Number of Jacobian evaluations (neaerst 10) = %6d\n',njacs);
  fprintf(' Number of iterations           (nearest 50) = %6d\n',niters);

  fig1 = figure; 
  plot(x,u(1,:),x,u(2,:));
  title({['First order system, solution at t = 0.1 using'],...
         ['Roe''s approximate Reimann solver']});
  xlabel('x');
  ylabel('u,v');
  legend('u(x,t) approx','v(x,t) approx','Location','NorthWest');
  % print(fig1,'-dpng','-r75','nag_pde_1d_parab_convdiff_fig1.png');
  % print(fig1,'-deps','-r75','nag_pde_1d_parab_convdiff_fig1.eps');

function ex2
  ts   = 0;
  tout = 1;
  npde = 1;
  npts = 151;
  dx   = 2/(npts-1); 
  x    = [-1:dx:1];
  u(1,:) = x + 4;
  acc  = [1e-05;  1e-05];
  tsmax  = 0.02;
  rsave  = zeros(6695, 1);
  isave  = zeros(1000, 1, nag_int_name);
  itask  = nag_int(1);
  itrace = nag_int(0);
  ind    = nag_int(0);

  [ts, u, rsave, isave, ind, ifail] = ...
  nag_pde_1d_parab_convdiff( ...
         ts, tout, @pdedef, @numflx2, @bndary2, u, x, acc, ...
         tsmax, rsave, isave, itask, itrace, ind);

  nsteps = 5*((isave(1)+2)/5);
  nfuncs = 50*((isave(2)+25)/50);
  njacs = 10*((isave(3)+5)/10);
  niters = 50*((isave(5)+25)/50);
  fprintf('\n Number of time steps           (nearest 5)  = %6d\n',nsteps);
  fprintf(' Number of function evaluations (nearest 50) = %6d\n',nfuncs);
  fprintf(' Number of Jacobian evaluations (neaerst 10) = %6d\n',njacs);
  fprintf(' Number of iterations           (nearest 50) = %6d\n',niters);

  fig2 = figure; 
  plot(x,u(1,:));
  title({['Burger''s equation, solution at t = 0.1 using'],...
         ['Roe''s approximate Reimann solver']});
  xlabel('x');
  ylabel('u,v');
  % print(fig2,'-dpng','-r75','nag_pde_1d_parab_convdiff_fig2.png');
  % print(fig2,'-deps','-r75','nag_pde_1d_parab_convdiff_fig2.eps');


function [p, c, d, s, ires] = pdedef(npde, t, x, u, ux, ires)
    %  simple hyperbolic system.
    p = eye(npde);
    c = zeros(npde, 1); d = c; s = c;
    c(1) = 0.01; 
    d(1) = ux(1);
    s(1) = u(1);

function [flux,ires] = numflx1(npde, t, x, uleft, uright, ires)
  flux = zeros(npde,1);
  flux(1) = (-2*uright(1)+6*uleft(1)+uright(2)+3*uleft(2))/4;
  flux(2) = ( 2*uright(1)+6*uleft(1)-uright(2)+3*uleft(2))/2;

function [flux,ires] = numflx2(npde, t, x, uleft, uright, ires)
  if (x>=0)
    flux(1) = x*uleft(1);
  else
    flux(1) = x*uright(1);
  end

function [g, ires] = bndary1(npde, npts, t, x, u, ibnd, ires)
  np1 = npts-1;
  np2 = npts-2;
  g   = zeros(npde,1);
  if (ibnd == 0)
    ue   = exact(t,npde,x(1),1);
    c    = (x(2)-x(1))/(x(3)-x(2));
    exu1 = (1+c)*u(1,2) - c*u(1,3);
    exu2 = (1+c)*u(2,2) - c*u(2,3);
    g(1) = 2*u(1,1) + u(2,1) - 2*ue(1,1) - ue(2,1);
    g(2) = 2*u(1,1) - u(2,1) - 2*exu1 + exu2;
  else
    ue   = exact(t,npde,x(npts),1);
    c    = (x(npts)-x(np1))/(x(np1)-x(np2));
    exu1 = (1+c)*u(1,2) - c*u(1,3);
    exu2 = (1+c)*u(2,2) - c*u(2,3);
    g(1) = 2*u(1,1) - u(2,1) - 2*ue(1,1) + ue(2,1);
    g(2) = 2*u(1,1) + u(2,1) - 2*exu1 - exu2;
  end

function [g, ires] = bndary2(npde, npts, t, x, u, ibnd, ires)
  np1 = npts-1;
  np2 = npts-2;
  g   = zeros(npde,1);
  if (ibnd == 0)
    g(1) = u(1,1) - 3;
  else
    g(1) = u(1,1) - 5;
  end

% exact solution (for comparison and b.c. purposes)
function u = exact(t,npde,x,npts)
  u = zeros(npde,npts);
  for i = 1:double(npts)
    x1 = x(i) + t;
    x2 = x(i) - 3*t;
    s1 = sin(2*pi*x1^2)/2;
    s2 = sin(2*pi*x2^2)/2;
    
    u(1,i) = (exp(x1)+exp(x2))/2 + (s2-s1)/2 + 2*t^2 - 2*x(i)*t;
    u(2,i) = exp(x2)-exp(x1) + (s2+s1) + x(i)^2 + 5*t^2 - 2*x(i)*t;
  end
