function nag_pde_1d_parab_dae_keller_example

% diary nag_pde_1d_parab_dae_keller_example.r
fprintf('nag_pde_1d_parab_dae_keller example results\n\n');

npde   = nag_int(2);
ts     = 0.0001;
tout   = 0.2;
x      = [0:0.05:1];
u(1:2:42) = exp(ts*(1-x))-1;
u(2:2:42) = -ts*exp(ts*(1-x));
u(43)  = ts; 
nleft  = nag_int(1);
ncode  = nag_int(1);
xi     = [1];
rtol   = [0.0001];
atol   = [0.0001];
itol   = nag_int(1);
normt  = 'A';
laopt  = 'F';
algopt = zeros(30,1);
algopt(1)  = 1;
algopt(13) = 0.005;
rsave  = zeros(4000, 1);
isave  = zeros(100, 1, nag_int_name);
itask  = nag_int(1);
itrace = nag_int(0);
ind    = nag_int(0);
for i_t = 1:5
  tout = 2^i_t/10;

  [ts, u, rsave, isave, ind, ifail] = ...
  nag_pde_1d_parab_dae_keller(...
         npde, ts, tout, @pdedef, @bndary, u, x, nleft, ncode, ...
        @odedef, xi, rtol, atol, itol, normt, laopt, algopt, ...
        rsave, isave, itask, itrace, ind);

  fprintf('\nThe solution at t = %7.4f is:\n',ts);
  fprintf('%3s%12s%9s ','x','u_1(x,t)','u_2(x,t)');
  fprintf('%5s%12s%9s ','x','u_1(x,t)','u_2(x,t)');
  fprintf('%5s%12s%9s\n','x','u_1(x,t)','u_2(x,t)');
  for i=1:7
    fprintf('%5.2f%9.4f%9.4f ',x(i),u(2*i-1),u(2*i));
    fprintf('%8.2f%9.4f%9.4f ',x(i+7),u(2*i+13),u(2*i+14));
    fprintf('%8.2f%9.4f%9.4f\n',x(i+14),u(2*i+27),u(2*i+28));
  end
end

fig1 = figure;
plot(x,u(1:2:41),x,u(2:2:42));
title({'Two PDEs coupled with ODE solution at t=3.2',...
       'using Keller Box and BDF'});
xlabel('x');
ylabel('u');
legend('u_1(x,t=3.2)','u_2(x,t=3.2)');

% print(fig1,'-dpng','-r75','nag_pde_1d_parab_dae_keller_fig1.png');
% print(fig1,'-deps','-r75','nag_pde_1d_parab_dae_keller_fig1.eps');

% diary off

function [res, ires] = pdedef(npde, t, x, u, ut, ux, ncode, v, vdot, ires)
  res = zeros(npde, 1);
  if (ires == -1)
    res(1) = v(1)*v(1)*ut(1) - x*u(2)*v(1)*vdot(1);
    res(2) = 0;
  else
    res(1) = v(1)*v(1)*ut(1) - x*u(2)*v(1)*vdot(1) - ux(2);
    res(2) = u(2) - ux(1);
  end

function [res, ires] = bndary(npde, t, ibnd, nobc, u, ut, ncode, v, vdot, ires)
  res = zeros(nobc, 1);
  if (ibnd == 0)
    if (ires == -1)
      res(1) = 0;
    else
      res(1) = u(2) + v(1)*exp(t);
    end
  else
    if (ires == -1)
      res(1) = v(1)*vdot(1);
    else
      res(1) = u(2) + v(1)*vdot(1);
    end
  end


function [f, ires] = odedef(npde, t, ncode, v, vdot, nxi, xi, ucp, ucpx, ...
                            ucpt, ires)
  f = zeros(ncode, 1);
  if (ires == -1)
    f(1) = vdot(1);
  else
    f(1) = vdot(1) - v(1)*ucp(1,1) - ucp(2,1) - 1 - t;
  end
