function nag_quad_md_adapt_multi_example

% diary nag_quad_md_adapt_multi_example.r
fprintf('nag_quad_md_adapt_multi example results\n\n');

% Initialize variables and arrays.
nfun = nag_int(10);
ndim = 4;
absreq = 0;
relreq = 1e-3;
mincls = nag_int(0);
ircls = 2^ndim+2*ndim*ndim+2*ndim+1;
maxcls = nag_int(ircls);
lenwrk = 6*ndim+9*nfun+(ndim+nfun+2)*(1+maxcls/ircls);
ncall = 1;
a = zeros(1, ndim);
b = ones(1, ndim);
wrkstr = zeros(lenwrk, 1);
mkeep = zeros(1,1);
fkeep = zeros(1,1);
akeep = zeros(1,1);

% Factor for scaling the maxcls parameter (which controls the maximum
% number of times f is to be called by nag_quad_md_adapt_multi).
if (ndim < 10)
    mulfac = 2^ndim;
else
    mulfac = 2*(ndim^3);
end

% Turn off warnings from NAG routines, catching them instead.
wstat = warning();
warning('OFF');
% First call: the initial parameter values are set up to demonstrate the
% continuation facility of the routine; it should exit with ifail = 1
% indicating that maxcls is too small.
% Loop until ifail is not 1 or 3 (i.e. maxcls is big enough).
ifail = nag_int(1);
while (ifail == 1 || ifail == 3)
    [mincls, wrkstr, finest, absest, ifail] = ...
        nag_quad_md_adapt_multi(a, b, mincls, maxcls, nfun, @f, absreq, relreq, wrkstr);

    % Is it big enough yet?  Output appropriate message.
    if ifail ~= 0
        fprintf('maxcls = %7d mincls = %7d  ifail = %d \n\n', maxcls, ...
                mincls, ifail);
    else
        fprintf('Success: %7d user function calls in last call\n\n', mincls);
    end

    % Store results for plotting.
    fkeep(1:nfun,ncall) = finest(1:nfun);
    akeep(1:nfun,ncall) = absest(1:nfun);
    mkeep(ncall,1) = mincls;

    % Prepare for next call of routine.
    mincls = nag_int(-1);
    maxcls = maxcls * mulfac;
    ncall = ncall + 1;
    if ifail ~= 0
        fprintf('  .. increasing maxcls to %7d\n\n', maxcls);
    end
end
warning(wstat);

% Turn NAG warnings back to origianl setting.
% Plot results.
fig1 = figure;
display_plot(fkeep, mkeep, 'Integrand Number');
% print(fig1,'-dpng','-r75','nag_quad_md_adapt_multi_fig1.png');
% print(fig1,'-deps','-r75','nag_quad_md_adapt_multi_fig1.eps');

fig2 = figure;
display_plot(akeep, mkeep, 'Estimated Error');
% print(fig2,'-dpng','-r75','nag_quad_md_adapt_multi_fig2.png');
% print(fig2,'-deps','-r75','nag_quad_md_adapt_multi_fig2.eps');

% diary off

function f = f(ndim, z, nfun)
% Evaluate the integrands.
f = zeros(nfun,1);
sm = 0;
for n=1:ndim
    sm = sm + double(n)*z(n);
end
for k=1:nfun
    f(k) = log(sm)*sin(double(k)+sm);
end

function display_plot(data, labels, ylabelString)
% Formatting for title and axis labels.
% Use bars to plot intregral data and points on logscale for errors.
if strncmp(ylabelString, 'Integrand', 9)
    bar(data,'group');
    title({'Multi-dimensional Integrals of ten Integrands with ', ...
        'Various Numbers of Function Calls'});
    set(gca, 'YTick', [-0.5 -0.3 -0.1 0.1 0.3 0.5]);
else
    plot(data,'*');
    title({'Errors in Computing Integrals of ten Integrands with ', ...
        'Various Numbers of Function Calls'});
    set(gca, 'YScale', 'log');
    set(gca, 'XLim', [0.5 10.5]);
end
xlabel('Integral Index');
ylabel(ylabelString);
% Add a legend.
legend([num2str(labels(1)) ' calls'], ...
    [num2str(labels(2)) ' calls'], ...
    [num2str(labels(3)) ' calls'], 'Location', 'Best')
