function nag_rand_bb_inc_init_example

% diary nag_rand_bb_inc_init_example.r
fprintf('nag_rand_bb_inc_init example results\n\n');

% Get information required to set up the bridge
[bgord,t0,tend,ntimes,intime,nmove,move] = get_bridge_init_data();

% Make the bridge construction bgord
[times, ifail] = nag_rand_bb_make_bridge_order( ...
                        t0, tend, intime, move, 'bgord', bgord);

% Initialize the Brownian bridge generator
[rcommb, ifail] = nag_rand_bb_init( ...
                         t0, tend, times);
[rcommd, ifail] = nag_rand_bb_inc_init( ...
                         t0, tend, times);

% Get additional information required by the bridge generator
[npaths,d,start,a,term,c] = get_bridge_gen_data();
dif = term - start;

% Generate the Z values
[z] = get_z(npaths, d, a, ntimes);

% Call the Brownian bridge generator routine
[zb, bb, ifail] = nag_rand_bb( ...
                         npaths, start, term, z, c, rcommb, 'a', a);
[zd, bd, ifail] = nag_rand_bb_inc( ...
                         npaths, dif, z, c, rcommd, 'a', a);

% Display the results
unscaled = zeros(1, d);
for n = 1:npaths
  fprintf('Weiner Path %d, %d time steps, %d dimensions\n', n, ntimes+1, d);
  fprintf('     Output of nag_rand_bb    Output of nag_rand_bb_inc    Sum of nag_rand_bb_inc\n');

  cum = start;
  unscaled = bd(1:d, n)*(intime(1)-t0);
  cum = cum + unscaled;
  fprintf('%2d %8.4f %8.4f  %8.4f %8.4f  %8.4f %8.4f\n', ...
          1, bb(1:d,n), bd(1:d,n), cum(1:d));

  jd1 = 1;
  for j = 2:ntimes
    jd1 = jd1 + d;
    unscaled = bd(jd1:j*d, n)*(intime(j)-intime(j-1));
    cum = cum + unscaled;
    fprintf('%2d %8.4f %8.4f  %8.4f %8.4f  %8.4f %8.4f\n', ...
            j, bb(jd1:j*d,n), bd(jd1:j*d,n), cum(1:d));
  end

  j = ntimes+1;
  jd1 = ntimes*d + 1;
  unscaled = bd(jd1:j*d, n)*(tend-intime(j-1));
  cum = cum + unscaled;
  fprintf('%2d %8.4f %8.4f  %8.4f %8.4f  %8.4f %8.4f\n\n', ...
          j, bb(jd1:j*d,n), bd(jd1:j*d,n), cum(1:d));
end

% diary off

function [bgord,t0,tend,ntimes,intime,nmove,move] = get_bridge_init_data()
  % Set the basic parameters for a Wiener process
  t0 = 0;
  n = 10;
  ntimes = nag_int(n);

  % We want to generate the Wiener process at these time points
  intime = 1:n + t0;
  tend = t0 + n + 1;

  nmove = nag_int(0);
  move  = zeros(nmove, 1, nag_int_name);
  bgord = nag_int(3);

function [npaths,d,start,a,term,c] = get_bridge_gen_data();
  % Set the basic parameters for a free Wiener process
  npaths = nag_int(2);
  d = 2;
  a = nag_int(0);
  start = [0; 2];
  term  = [1; 0];

  % We want the following covariance matrix
  c = [ 6, -1;
       -1,  5];

  % Cholesky factorize of the covariance matrix c
  [c, info] = nag_lapack_dpotrf('l', c);

function [z] = get_z(npaths, d, a, ntimes)
  idim = d*(ntimes+1-a);

  % We now need to generate the input pseudorandom points

  % First initialize the base pseudorandom number generator
  state = initialize_prng(nag_int(6), nag_int(0), [nag_int(1023401)]);

  % Generate the pseudorandom points from N(0,1)
  [state, z, ifail] = nag_rand_dist_normal( ...
                             nag_int(idim*npaths), 0, 1, state);

  z = transpose(reshape(z, npaths, idim));

function [state] = initialize_prng(genid, subid, seed)
  % Initialize the generator to a repeatable sequence
  [state, ifail] = nag_rand_init_repeat( ...
                          genid, subid, seed);
