function nag_rand_bb_make_bridge_order_example

% diary nag_rand_bb_make_bridge_order_example.r
fprintf('nag_rand_bb_make_bridge_order example results\n\n');

% Get information required to set up the bridge
[bgord,t0,tend,ntimes,intime,nmove,move] = get_bridge_init_data();

% Make the bridge construction bgord
[times, ifail] = nag_rand_bb_make_bridge_order( ...
                        t0, tend, intime, move, 'bgord', bgord);

% Initialize the Brownian bridge generator
[rcomm, ifail] = nag_rand_bb_init( ...
                        t0, tend, times);

% Get additional information required by the bridge generator
[npaths,d,start,a,term,c] = get_bridge_gen_data();

% Generate the Z values
[z] = get_z(npaths, d, a, ntimes);

% Call the Brownian bridge generator routine
[z, b, ifail] = nag_rand_bb( ...
                       npaths, start, term, z, c, rcomm, 'a', a);

% Display the results
for i = 1:npaths
  fprintf('Weiner Path %d, %d time steps, %d dimensions\n', i, ntimes+1, d);
  w = transpose(reshape(b(:,i), d, ntimes+1));

  ifail = nag_file_print_matrix_real_gen('G', ' ', w, '');

  fprintf('\n');
end

% diary off

function [bgord,t0,tend,ntimes,intime,nmove,move] = get_bridge_init_data()
  % Set the basic parameters for a Wiener process
  t0 = 0;
  ntimes = nag_int(10);

  % We want to generate the Wiener process at these time points
  intime = 1.71*double(1:ntimes) + t0;
  tend = t0 + 1.71*double(ntimes + 1);

  % We suppose the following 3 times are very important and should be
  % constructed first. Note: these are indices into intime
  nmove= nag_int(3);
  move = [nag_int(3), 5, 4];
  bgord = nag_int(3);

function [npaths,d,start,a,term,c] = get_bridge_gen_data();
  % Set the basic parameters for a non-free Wiener process
  npaths = nag_int(2);
  d = 3;
  a = nag_int(0);
  start = zeros(d, 1);
  term  = zeros(d, 1);

  % As a = 0, term need not be initialized

  % We want the following covariance matrix
  c = [ 6,   1,  -0.2; 
        1,   5,   0.3;  
       -0.2, 0.3, 4  ];

  % Cholesky factorize of the covariance matrix c
  [c, info] = nag_lapack_dpotrf('l', c);

function [z] = get_z(npaths, d, a, ntimes)
  idim = d*(ntimes+1-a);

  % We now need to generate the input pseudorandom points

  % First initialize the base pseudorandom number generator
  state = initialize_prng(nag_int(6), nag_int(0), [nag_int(1023401)]);

  % Scrambled quasi-random sequences preserve the good discrepancy
  % properties of quasi-random sequences while counteracting the bias
  % some applications experience when using quasi-random sequences.
  % Initialize the scrambled quasi-random generator.
  [iref, state] = initialize_scrambled_qrng(nag_int(1), nag_int(2), ...
                                            idim, state);

  % Generate the quasi-random points from N(0,1)
  xmean = zeros(idim, 1);
  std   = ones(idim, 1);
  [z, iref, ifail] = nag_rand_quasi_normal( ...
                            xmean, std, npaths, iref);
  z = z';

function [state] = initialize_prng(genid, subid, seed)
  % Initialize the generator to a repeatable sequence
  [state, ifail] = nag_rand_init_repeat( ...
                          genid, subid, seed);


function [iref, state] = initialize_scrambled_qrng(genid,stype,idim,state)
  iskip = nag_int(0);
  nsdigits = nag_int(32);
  [iref, state, ifail] = nag_rand_quasi_init_scrambled( ...
                                genid, stype, nag_int(idim), ...
                                iskip, nsdigits, state);
