function nag_rand_field_fracbm_generate_example

% diary nag_rand_field_fracbm_generate_example.r
fprintf('nag_rand_field_fracbm_generate example results\n\n');

% Upper bound for interval
xmax = 2;
% Number of sample points
ns   = nag_int(10);
% Scaling factor, rho = 1.
icorr = nag_int(2);

% Set fixed problem specifications for simulating fractional Brownian motion
h     = 0.35;
icov1 = nag_int(14);
np    = nag_int(2);
xmin  = 0;
var   = 1;
params = [h, xmax/double(ns)];

% Get square roots of the eigenvalues of the embedding matrix
[lam, xx, m, approx, rho, icount, eig, ifail] = ...
  nag_rand_field_1d_predef_setup( ...
         ns, xmin, xmax, var, icov1, params, ...
         'icorr', icorr, 'maxm', nag_int(2048));

fprintf('\nSize of embedding matrix = %d\n\n', m);

% Display approximation information if approximation used
if approx == 1
  fprintf('Approximation required\n\n');
  fprintf('rho = %10.5f\n', rho);
  fprintf('eig = %10.5f%10.5f%10.5f\n', eig(1:3));
  fprintf('icount = %d\n', icount);
else
  fprintf('Approximation not required\n\n');
end

% Initialize state array
genid = nag_int(1);
subid = nag_int(1);
seed  = [nag_int(14965)];
[state, ifail] = nag_rand_init_repeat( ...
                        genid, subid, seed);

% Compute s fractional Brownian Motion realisations.
s    = nag_int(5);
[state, z, yy, ifail] = nag_rand_field_fracbm_generate( ...
                               ns, s, xmax, h, lam(1:m), rho, state);

% Display random field results
% Set row labels to mesh points (column label is realisation number).
rlabs = cell(ns+1, 1);
for i=1:ns+1
  rlabs{i} = sprintf('%6.1f', yy(i));
end

% Matrix printing parameters
mtitle = 'Fractional Brownian motion realisations (x coordinate first):';
matrix = 'General';
diag   = 'Non-unit';
fmt    = 'f10.5';
rlabel = 'Character';
clabel = 'Integer';
clabs  = {' '};
ncols  = nag_int(80);
indent = nag_int(0);

[ifail] = nag_file_print_matrix_real_gen_comp( ...
                 matrix, diag, z, fmt, mtitle, rlabel, rlabs, clabel, ...
                 clabs, ncols, indent);

% diary off
