function nag_sparse_complex_gen_precon_jacobi_example

% diary nag_sparse_complex_gen_precon_jacobi_example.r
fprintf('nag_sparse_complex_gen_precon_jacobi example results\n\n');

% Solve sparse system Ax = b iteratively using Jacobi preconditioner

% Define sparse matrix A and RHS B
nz   = nag_int(24);
n    = nag_int(8);
a = [ 2 + 1i, -1 + 1i,  1 - 3i,  4 + 7i, -3 + 0i,  2 + 4i, ...
     -7 - 5i,  2 + 1i,  3 + 2i, -4 + 2i,  0 + 1i,  5 - 3i, ...
     -1 + 2i,  8 + 6i, -3 - 4i, -6 - 2i,  5 - 2i,  2 + 0i, ...
      0 - 5i, -1 + 5i,  6 + 2i, -1 + 4i,  2 + 0i,  3 + 3i];
b = [ 7 + 11i;
      1 + 24i;
    -13 - 18i;
    -10 +  3i;
     23 + 14i;
     17 -  7i;
     15 -  3i;
     -3 + 20i];
irow = nag_int(...
               [1; 1; 1; 2; 2; 2; 3; 3; 4; 4; 4; 4;
                5; 5; 5; 6; 6; 6; 7; 7; 7; 8; 8; 8]);
icol = nag_int(...
               [1; 4; 8; 1; 2; 5; 3; 6; 1; 3; 4; 7;
                2; 5; 7; 1; 3; 6; 3; 5; 7; 2; 6; 8]);

b = [  7 + 11i;
       1 + 24i;
     -13 - 18i;
     -10 +  3i;
      23 + 14i;
      17 -  7i;
      15 -  3i;
      -3 + 20i];


% Iterative Solution Setup
%      Input parameters
method = 'TFQMR';
precon = 'p';
m      = nag_int(2);
tol    = 1e-6;
maxitn = nag_int(20);
anorm  = 0;
sigmax = 0;
monit  = nag_int(1);
lwork = nag_int(300);

% Initialise Solver
[lwreq, work, ifail] = nag_sparse_complex_gen_basic_setup( ...
                              method, precon, n, m, tol, maxitn, anorm, ...
                              sigmax, monit, lwork, 'norm_p', '1');

% Solver inputs
irevcm = nag_int(0);
x = complex(zeros(n, 1));
wgt= zeros(n, 1);

% Preconditioner inputs
store = 'Non Hermitian';
trans = 'N';
init  = 'I';
niter = nag_int(2);
check = 'Check';
diag  = complex(zeros(n, 1));

% Solve by reverse communication
while (irevcm ~= 4)
  [irevcm, x, b, work, ifail] = nag_sparse_complex_gen_basic_solver( ...
                                       irevcm, x, b, wgt, work);

  if (irevcm == -1)
    % b = A^Tx
    [b, ifail] = nag_sparse_complex_gen_matvec('T', a, irow, icol, 'N', x);
  elseif (irevcm == 1)
    % b = Ax
    [b, ifail] = nag_sparse_complex_gen_matvec('N', a, irow, icol, 'N', x);
  elseif (irevcm == 2)
    % Jacobi preconditioner
    [b, diag, ifail] = nag_sparse_complex_gen_precon_jacobi( ...
                              store, trans, init, niter, ...
                              a, irow, icol, check, x, diag);
  elseif (irevcm == 3)
    [itn, stplhs, stprhs, anorm, sigmax, work, ifail] = ...
    nag_sparse_complex_gen_basic_diag(work);
    fprintf('\nMonitoring at iteration number %d\n', itn);
    fprintf('residual norm: %14.4e\n', stplhs);
  end
end

% Obtain information about the computation
[itn, stplhs, stprhs, anorm, sigmax, work, ifail] = ...
nag_sparse_complex_gen_basic_diag(work);
fprintf('\nNumber of iterations for convergence: %d\n', itn);
fprintf('Residual norm:                           %14.4e\n', stplhs);
fprintf('Right-hand side of termination criteria: %14.4e\n', stprhs);
fprintf('i-norm of matrix a:                      %14.4e\n', anorm);
fprintf('\n   Solution Vector\n');
disp(x);
fprintf('\n   Residual Vector\n');
disp(b);

% diary off
