function nag_sparse_complex_herm_solve_ilu_example

% diary nag_sparse_complex_herm_solve_ilu_example.r
fprintf('nag_sparse_complex_herm_solve_ilu example results\n\n');

% Solve sparse Hermitian system Ax = b using CG method with
% Incomplete Cholesky preconditioning (IC)

% Define A and b 
n  = nag_int(9);
nz = nag_int(23);
a    = zeros(3*nz,1);
irow = zeros(3*nz, 1, nag_int_name);
icol = zeros(3*nz, 1, nag_int_name);
a(1:nz) = [ 6 + 0.i; -1 + 1.i;  6 + 0.i;  0 + 1.i;
            5 + 0.i;  5 + 0.i;  2 - 2.i;  4 + 0.i;
            1 + 1.i;  2 + 0.i;  6 + 0.i; -4 + 3.i;
            0 + 1.i; -1 + 0.i;  6 + 0.i; -1 - 1.i;
            0 - 1.i;  9 + 0.i;  1 + 3.i;  1 + 2.i;
           -1 + 0.i;  1 + 4.i;  9 + 0.i];
b       = [ 8 + 54i;-10 - 92i; 25 + 27i; 26 - 28i;
           54 + 12i; 26 - 22i; 47 + 65i; 71 - 57i;
           60 + 70i];
irow(1:nz) = nag_int([1;2;2;3;3;4;5;5;6;6;6;7;7;7;7;8;8;8;9;9;9;9;9]);
icol(1:nz) = nag_int([1;1;2;2;3;4;1;5;3;4;6;2;5;6;7;4;6;8;1;5;6;8;9]);

% Calculate incomplete Cholesky factorization
lfill  = nag_int(0);
dtol   = 0;
mic    = 'N';
dscale = 0;
ipiv   = zeros(n, 1, nag_int_name);

[a, irow, icol, ipiv, istr, nnzc, npivm, ifail] = ...
  nag_sparse_complex_herm_precon_ilu( ...
         nz, a, irow, icol, lfill, dtol, mic, dscale, ipiv);

% Solve Ax = b
method = 'CG';
tol    = 1e-06;
maxitn = nag_int(100);
x      = complex(zeros(n, 1));

[x, rnorm, itn, ifail] = ...
  nag_sparse_complex_herm_solve_ilu( ...
         method, nz, a, irow, icol, ipiv, istr, b, tol, maxitn, x);

fprintf('Converged in %d iterations\n', itn);
fprintf('Final redidual norm = %16.3d\n\n', rnorm);
disp('Solution');
disp(x);

% diary off
