function nag_sparse_real_gen_basic_setup_example

% diary nag_sparse_real_gen_basic_setup_example.r
fprintf('nag_sparse_real_gen_basic_setup example results\n\n');

% Solve sparse system Ax = b iteratively using ILU preconditioner

% Define sparse matrix A and RHS B
n    = nag_int(8);
nz   = nag_int(24);
a    = zeros(3*nz, 1);
irow = zeros(3*nz, 1, nag_int_name);
icol = irow;
irow(1:nz) = nag_int(...
             [1; 1; 1; 2; 2; 2; 3; 3; 4; 4; 4; 4;
              5; 5; 5; 6; 6; 6; 7; 7; 7; 8; 8; 8]);
icol(1:nz) = nag_int(...
             [1; 4; 8; 1; 2; 5; 3; 6; 1; 3; 4; 7;
              2; 5; 7; 1; 3; 6; 3; 5; 7; 2; 6; 8]);
a(1:nz)    = [2;-1; 1; 4;-3; 2;-7; 2; 3;-4; 5; 5;
             -1; 8;-3;-6; 5; 2;-5;-1; 6;-1; 2; 3];
b          = [6; 8;-9;46;17;21;22;34];

% ILU preconditioner
%     Input parameters
lfill = nag_int(0);
dtol  = 0;
milu  = 'No modification';
ipivp = zeros(n, 1, nag_int_name);
ipivq = zeros(n, 1, nag_int_name);
% Compute preconditioner
[a, irow, icol, ipivp, ipivq, istr, idiag, nnzc, npivm, ifail] = ...
    nag_sparse_real_gen_precon_ilu(...
          nz, a, irow, icol, lfill, dtol, milu, ipivp, ipivq);

% Iterative Solution Setup
%      Input parameters
method = 'BICGSTAB';
precon = 'Preconditioned';
lpoly  = nag_int(1);
tol    = sqrt(nag_machine_precision);
maxitn = nag_int(20);
anorm  = 0;
sigmax = 0;
monit  = nag_int(1);
lwork  = nag_int(6000);
% Setup iterative method
[lwreq, work, ifail] = ...
  nag_sparse_real_gen_basic_setup(...
        method, precon, n, lpoly, tol, maxitn, anorm, sigmax, ...
        monit, lwork, 'norm_p', '1');

% Reverse communication loop calling iterative solver
irevcm = nag_int(0);
u      = zeros(8, 1);
v      = b;
wgt    = zeros(8, 1);

while (irevcm ~= 4)

  [irevcm, u, v, work, ifail] = nag_sparse_real_gen_basic_solver(...
                                      irevcm, u, v, wgt, work);

  if (irevcm == -1)
    % v = A^Tu
    [v, ifail] = nag_sparse_real_gen_matvec(...
                       'T', a(1:nz), irow(1:nz), icol(1:nz), 'N', u);
  elseif (irevcm == 1)
    % v = Au
    [v, ifail] = nag_sparse_real_gen_matvec(...
                       'N', a(1:nz), irow(1:nz), icol(1:nz), 'N', u);
  elseif (irevcm == 2)
    % Solve ILU v = u  
    [v, ifail] = nag_sparse_real_gen_precon_ilu_solve('N', a, irow, icol, ipivp, ipivq, istr, idiag, 'N', u);
    if (ifail ~= 0)
      irevcm = 6;
    end
  elseif (irevcm == 3)
    % Monitoring stage
    [itn, stplhs, stprhs, anorm, sigmax, ifail] = ...
      nag_sparse_real_gen_basic_diag(work);
    fprintf('\nMonitoring at iteration number %2d\n',itn);
    fprintf('residual norm:              %14.4e\n', stplhs);
    fprintf('\n   Solution Vector  Residual Vector\n');
    for i = 1:n
      fprintf('%16.4f %16.2e\n', u(i), v(i));
    end
  end
end

% Get information about the computation
[itn, stplhs, stprhs, anorm, sigmax, ifail] = ...
      nag_sparse_real_gen_basic_diag(work);

fprintf('\nNumber of iterations for convergence:     %4d\n', itn);
fprintf('Residual norm:                           %14.4e\n', stplhs);
fprintf('Right-hand side of termination criteria: %14.4e\n', stprhs);
fprintf('i-norm of matrix a:                      %14.4e\n', anorm);
fprintf('\n   Solution Vector  Residual Vector\n');
for i = 1:n
  fprintf('%16.4f %16.2e\n', u(i), v(i));
end

% diary off
