function nag_sparse_real_gen_precon_ssor_solve_example

% diary nag_sparse_real_gen_precon_ssor_solve_example.r
fprintf('nag_sparse_real_gen_precon_ssor_solve example results\n\n');

% Sparse matrix A
n    = nag_int(5);
m    = nag_int(2);
nz   = nag_int(16);
a    = zeros(3*nz, 1);
irow = zeros(3*nz, 1, nag_int_name);
icol = irow;
a(1:nz)    = [2; 1;-1;-3;-2; 1; 1; 5; 3; 1;-2;-3;-1; 4;-2;-6];
irow(1:nz) = [1; 1; 1; 2; 2; 2; 3; 3; 3; 3; 4; 4; 4; 5; 5; 5];
icol(1:nz) = [1; 2; 4; 2; 3; 5; 1; 3; 4; 5; 1; 4; 5; 2; 3; 5];

% Solver setup initializations 
method = 'rgmres';
precon = 'P';
tol    = 1e-10;
maxitn = nag_int(1000);
anorm  = 0;
sigmax = 0;
monit  = nag_int(0);
lwork  = max([n*(m+3)+m*(m+5)+101,7*n+100,(2*n+m)*(m+2)+n+100,10*n+100]);

% Initialize solver
[lwreq, work, ifail] = ...
  nag_sparse_real_gen_basic_setup( ...
         method, precon, n, m, tol, maxitn, anorm, sigmax, monit, lwork, ...
         'norm_p', 'I');

% RHS b and initial guess x
b = [0; -7; 33; -19; -28];
x = zeros(n, 1);

% Calculate reciprocal diagonal matrix elements for SSOR preconditioning
rdiag = zeros(n, 1);
if strcmpi(precon, 'P')
  dcount = zeros(n, 1, nag_int_name);

  for i = 1:nz
    if irow(i) == icol(i)
      dcount(irow(i)) = dcount(irow(i)) + 1;
      if a(i) ~= 0
        rdiag(irow(i)) = 1/a(i);
      else
        error('Matrix has a zero diagonal element');
      end
    end
  end

  for i = 1:n
    if dcount(i) == 0
      error('Matrix has a missing diagonal element');
    elseif dcount(i) >= 2
      error('Matrix has a multiple diagonal element');
    end
  end
end
% Preconditioner input argument initializations
trans  = 'N';
omega  = 1.1;
ckddf = 'C';

% Solver input argument initialization
irevcm = nag_int(0);
wgt = zeros(n, 1);

% Matrix-vector input argument
ckxaf = 'C';

% Solve the linear system by reverse communication
while irevcm ~= 4
  [irevcm, x, b, work, ifail] = nag_sparse_real_gen_basic_solver( ...
                                       irevcm, x, b, wgt, work);

  if (irevcm == -1)
    % Compute transposed matrix-vector product
    [b, ifail] = nag_sparse_real_gen_matvec( ...
                        'T', a(1:nz), irow(1:nz), icol(1:nz), ckxaf, x);
    ckxaf = 'N';
  elseif (irevcm == 1)
    % Compute matrix-vector product
    [b, ifail] = nag_sparse_real_gen_matvec( ...
                        'N', a(1:nz), irow(1:nz), icol(1:nz), ckxaf, x);
    ckxaf = 'N';
  elseif (irevcm == 2)
    % SSOR preconditioning
    [b, ifail] = nag_sparse_real_gen_precon_ssor_solve( ...
                        trans, a(1:nz), irow(1:nz), icol(1:nz), rdiag,...
                        omega, ckddf, x);
    ckddf = 'N';
  end
end

% Get information about the computation
[itn, stplhs, stprhs, anorm, sigmax, ifail] = ...
    nag_sparse_real_gen_basic_diag(work);
fprintf('\nConverged in %d iterations\n', itn);
fprintf('Matrix norm         = %16.3e\n', anorm);
fprintf('Final residual norm = %16.3e\n\n', stplhs);
disp('Solution');
disp(x);

% diary off
