function nag_sparseig_real_symm_init_example

% diary nag_sparseig_real_symm_init_example.r
fprintf('nag_sparseig_real_symm_init example results\n\n');

nx  = nag_int(10);
n   = nx^2;
nev = nag_int(4);
ncv = nag_int(10);
imon = 0;

irevcm = nag_int(0);
resid  = zeros(n,1);
v      = zeros(n,ncv);
x      = zeros(n,1);
mx     = zeros(n,1);

% Initialisation Step
[icomm, comm, ifail] = nag_sparseig_real_symm_init( ...
                              n, nev, ncv);

% Set Optional Parameters
[icomm, comm, ifail] = nag_sparseig_real_symm_option( ...
                              'SMALLEST MAGNITUDE', icomm, comm);

% Solve
while (irevcm ~= 5)
  [irevcm, resid, v, x, mx, nshift, comm, icomm, ifail] = ...
    nag_sparseig_real_symm_iter( ...
           irevcm, resid, v, x, mx, comm, icomm);
  if (irevcm == 1 || irevcm == -1)
    x = nag_sparseig_real_symm_init_Ax(nx, x);
  elseif (irevcm == 4 && imon==1)
    [niter, nconv, ritz, rzest] = nag_sparseig_real_symm_monit(icomm, comm);
    fprintf(['Iteration %2d, No. converged = %d, ', ...
             'norm of estimates = %10.2e\n'], ...
            niter, nconv, norm(rzest(1:nev),2));
  end
end

% Post-process to compute eigenvalues/vectors
sigma = 0;
[nconv, d, z, v, comm, icomm, ifail] = ...
  nag_sparseig_real_symm_proc( ...
         sigma, resid, v, comm, icomm);

fprintf('Smallest %d Eigenvalues are:\n',nconv);
disp(d(1:nconv));

% diary off

function [y] = nag_sparseig_real_symm_init_Ax(nx, x)

  y = zeros(nx*nx,1);

  h2 = 1/double((nx+1)^2);

  y(1:nx) = nag_sparseig_real_symm_init_Bx(nx, x(1:nx));
  y(1:nx) = y(1:nx) - x(nx+1:2*nx);

  for j=2:nx-1
    lo = (j-1)*nx +1;
    hi = j*nx;

    y(lo:hi) = nag_sparseig_real_symm_init_Bx(nx, x(lo:hi));
    y(lo:hi) = y(lo:hi) - x(lo-nx:lo-1) - x(hi+1:hi+nx);
  end

  lo = (nx-1)*nx +1;
  hi = nx*nx;
  y(lo:hi) = nag_sparseig_real_symm_init_Bx(nx, x(lo:hi));
  y(lo:hi) = y(lo:hi) - x(lo-nx:lo-1);
  y = y/h2;


function [y] = nag_sparseig_real_symm_init_Bx(nx,x)

  y = zeros(nx,1);

  dd = 4;
  dl = -1;
  du = -1;

  y(1) = dd*x(1) + du*x(2);
  for j=2:nx-1
    y(j) = dl*x(j-1) + dd*x(j) + du*x(j+1);
  end
  y(nx) = dl*x(nx-1) + dd*x(nx);
