function nag_sparseig_real_symm_monit_example

% diary nag_sparseig_real_symm_monit_example.r
fprintf('nag_sparseig_real_symm_monit example results\n\n');

n = nag_int(100);
nev = nag_int(4);
ncv = nag_int(10);

irevcm = nag_int(0);
resid = zeros(n,1);
v     = zeros(n,ncv);
x     = zeros(n,1);
mx    = zeros(n,1);
imon = 1;

sigma = 1;

% Setup and factorize A - sigma*B
h   = 1/double(n+1);
ad(1:n)  =  2/h - sigma*4*h/6;
adl(1:n) = -1/h - sigma*h/6;
adu(1:n) = adl(1:n);

[adl, ad, adu, adu2, ipiv, info] = nag_lapack_dgttrf( ...
                                          adl, ad, adu);

% Initialisation Step
[icomm, comm, ifail] = nag_sparseig_real_symm_init( ...
                              n, nev, ncv);

% Set Optional Parameters
[icomm, comm, ifail] = nag_sparseig_real_symm_option( ...
                              'Generalized', icomm, comm);
[icomm, comm, ifail] = nag_sparseig_real_symm_option( ...
                              'Buckling', icomm, comm);

% Solve
while (irevcm ~= 5)
  [irevcm, resid, v, x, mx, nshift, comm, icomm, ifail] = ...
    nag_sparseig_real_symm_iter( ...
           irevcm, resid, v, x, mx, comm, icomm);
   if (irevcm == -1)
    % Solve (A-sigma*B)y = Ax
    mx = nag_sparseig_real_symm_monit_Ax(n, x);
    [x, info] = nag_lapack_dgttrs( ...
                       'N', adl, ad, adu, adu2, ipiv, mx);
  elseif (irevcm == 1)
    % Solve (A-sigma*B)y = Ax, Ax in mx
     [x, info] = nag_lapack_dgttrs( ...
                       'N', adl, ad, adu, adu2, ipiv, mx);
  elseif (irevcm == 2)
    % y = Ax
    mx = nag_sparseig_real_symm_monit_Ax(n, x);
  elseif (irevcm == 4 && imon == 1)
    [niter, nconv, ritz, rzest] = ...
    nag_sparseig_real_symm_monit(icomm, comm);

    fprintf(['Iteration %2d, No. converged = %d, ', ...
             'norm of estimates = %10.2e\n'], ...
            niter, nconv, norm(rzest(1:nev),2));
  end
end

% Post-process to compute eigenvalues/vectors
[nconv, d, z, v, comm, icomm, ifail] = ...
nag_sparseig_real_symm_proc( ...
       sigma, resid, v, comm, icomm);

fprintf('\nThe %d Eigenvalues closest to %7.2f are:\n',nconv, sigma);
disp(d(1:nconv));

% diary off

function [y] = nag_sparseig_real_symm_monit_Ax(n,x)

  y = zeros(n,1);
  h = 1/double(n+1);

  dd = 2;
  dl = -1;
  du = -1;

  y(1) = dd*x(1) + du*x(2);
  for j=2:n-1
    y(j) = dl*x(j-1) + dd*x(j) + du*x(j+1);
  end
  y(n) = dl*x(n-1) + dd*x(n);
  y = y/h;
