function nag_stat_normal_scores_approx_example

% diary nag_stat_normal_scores_approx_example.r
fprintf('nag_stat_normal_scores_approx example results\n\n');

n = nag_int(10);
[pp, ifail] = nag_stat_normal_scores_approx(n);

fprintf('Sample size = %5d\n', n);
fprintf('Normal scores\n');
fprintf('              %10.4f%10.4f%10.4f%10.4f%10.4f\n',pp(1:n));

nag_stat_normal_scores_approx_plot;

% diary off

function nag_stat_normal_scores_approx_plot
  % This produces a Q-Q plot for a randomly generated set of data.
  % The normal scores have been calculated using nag_stat_normal_scores_approx and the sample
  % quantiles obtained by sorting the observed data using nag_sort_realvec_sort.

  % Initialize the Mersenne Twister generator
  seed = [nag_int(6324213)];
  genid = nag_int(3);
  subid = nag_int(0);

  [state, ifail] = nag_rand_init_repeat( ...
                          genid, subid, seed);

  % Generate 50 variates from a Student t distribution with 5 df
  n  = nag_int(50);
  df = nag_int(5);
  [state, x, ifail] = nag_rand_dist_students_t( ...
                             n, df, state);
  % Sort x
  m1 = nag_int(1);
  [x, ifail] = nag_sort_realvec_sort(x, m1, 'Ascending');

  % Calculate normal scores
  [pp, ifail] = nag_stat_normal_scores_approx(n);

  fig1 = figure;
  hold on
  plot(pp,x,'+r',[-2.25 2.25],[-2.25 2.25],'black');
  axis([-2.5 2.5 -3 3]);
  xlabel('Normal scores');
  ylabel('sample Quantiles');
  title({'Q-Q plot for a random set of data','using exact normal scores'});
  hold off;
  % print(fig1,'-dpng','-r75','nag_stat_normal_scores_approx_fig1.png');
  % print(fig1,'-deps','-r75','nag_stat_normal_scores_approx_fig1.eps');
