function nag_sum_invlaplace_crump_example

% diary nag_sum_invlaplace_crump_example.r
fprintf('nag_sum_invlaplace_crump example results\n\n');

% Initialize variables and arrays.
mxterm = nag_int(200);
alphab = -0.5;
n = 1;
trures = zeros(1, n);
trurel = zeros(1, n);

% First, make two preliminary calls to the routine to verify that the
% chosen alphab value is suitable.  Use t = 1 for these calls.
t(1) = 1;
disp(['Test with t(1) = ',num2str(t(1))]);

% Parameter values for a close to alphab.
tfac = 7.5;
relerr = 1e-2;
fprintf(['\nmxterm = %3.0f   tfac = %3.2f   alphab = %3.2f ', ...
         'relerr = %3.0e \n\n'], mxterm,tfac,alphab,relerr);

[valinv, errest, nterms, na, alow, ahigh, nfeval, ifail] = ...
nag_sum_invlaplace_crump( ...
       @fun, t, relerr, alphab, 'tfac', tfac, 'mxterm',mxterm);

% Calculate results and output them.
trures(1) = exp(double(-t(1)/2));
trurel(1) = abs((valinv(1)-trures(1))/trures(1));
disp('t     Result     exp(t/2)   Relative error   Error estimate')
fprintf('%1.1f   %8.4f    %8.4f      %8.4f         %8.4f\n',...
        t(1), valinv(1), trures(1), trurel(1), errest(1));
fprintf(['\nnterms = %3d   nfeval = %3d   alow = %3.2f ', ...
         'ahigh = %3.2f   ifail = %3d\n\n'], nterms, nfeval, alow, ...
        ahigh, ifail);

% Parameter values for larger a.
tfac = 0.8;
relerr = 1e-3;
disp(['Test with t(1) = ',num2str(t(1))]);
fprintf(['\nmxterm = %3d   tfac = %3.2f   alphab = %3.2f ', ...
         'relerr = %3.0e \n\n'], mxterm, tfac, alphab, relerr);

[valinv, errest, nterms, na, alow, ahigh, nfeval, ifail] = ...
nag_sum_invlaplace_crump( ...
       @fun, t, relerr, alphab, 'tfac', tfac, 'mxterm',mxterm);

% Calculate results and output them.
trures(1) = exp(double(-t(1)/2));
trurel(1) = abs((valinv(1)-trures(1))/trures(1));
disp('t     Result     exp(t/2)   Relative error   Error estimate')
fprintf('%1.1f   %8.4f    %8.4f      %8.5f         %8.5f\n',...
        t(1), valinv(1), trures(1), trurel(1), errest(1));
fprintf(['\nnterms = %3d   nfeval = %3d   alow = %3.2f ', ...
         'ahigh = %3.2f   ifail = %3d\n\n'], nterms, nfeval, alow, ...
        ahigh, ifail);

% Now calculate the inverse Laplace transform for several t values.
n = 5;
t = [1:5];
disp('Compute inverse')
fprintf(['\nmxterm = %3d   tfac = %3.2f   alphab = %3.2f ', ...
         'relerr = %3.0e \n\n'], mxterm, tfac, alphab, relerr);

[valinv, errest, nterms, na, alow, ahigh, nfeval, ifail] = ...
nag_sum_invlaplace_crump( ...
       @fun, t, relerr, alphab, 'tfac', tfac, 'mxterm',mxterm);

% Calculate results and output them.
disp('t    Result    exp(t/2)   Relative error  Error estimate')
for i = 1:n
  trures(i) = exp(-t(i)/2);
  trurel(i) = abs((valinv(i)-trures(i))/trures(i));
  fprintf('%1.0f   ',t(i));
  fprintf(' %4.3f      %4.3f      %4.3e      %4.3e\n',...
          valinv(i),trures(i),trurel(i),errest(i));
end
fprintf(['\nmxterm = %3d   tfac = %3.2f   alphab = %3.2f ', ...
         'relerr = %3.0e \n\n'], mxterm, tfac, alphab, relerr);

% Plot results.
fig1 = figure;
display_plot(t,trures,valinv,trurel,errest);
% print(fig1,'-dpng','-r75','nag_sum_invlaplace_crump_fig1.png');
% print(fig1,'-deps','-r75','nag_sum_invlaplace_crump_fig1.eps');

% diary off

function [fr,fi] = fun(preal,pimag)
  % Evaluate the real & imaginary parts of the user function.
  z = complex(1)/complex(preal+0.5,pimag);
  fr = real(z);
  fi = imag(z);

function display_plot(t, trures, valinv, trurel, errest)
  % Use a log plot for the second curve.
  [haxes, hline1, hline2] = plotyy(t, valinv, t, trurel, 'plot', 'semilogy');
  % Add the third curve (as points).
  hold on
  hpoints = plot(t, trures, '+', 'MarkerEdgeColor', 'r');
  % Set the axis limits and the tick specifications to beautify the plot.
  set(haxes(1), 'YLim', [0 0.8]);
  set(haxes(1), 'YMinorTick', 'on');
  set(haxes(1), 'YTick', [0.0 0.2 0.4 0.6 0.8]);
  set(haxes(2), 'YLim', [1e-7 5e-3]);
  set(haxes(2), 'YMinorTick', 'on');
  set(haxes(2), 'YTick', [1e-7 1e-6 1e-5 1e-4]);
  for iaxis = 1:2
    % These properties must be the same for both sets of axes.
    set(haxes(iaxis), 'XLim', [1 5]);
    set(haxes(iaxis), 'XTick', [1 1.5 2 2.5 3 3.5 4 4.5 5]);
  end
  set(gca, 'box', 'off'); % no ticks on opposite axes.
  % Add title.
  title('Inverse Laplace Transform of 1/(p+0.5)');
  % Label the x axis.
  xlabel('t');
  % Label the left and right y axes.
  ylabel(haxes(1),'f(t)');
  set(haxes(2),'YLim',[1e-8 Inf]);
  ylabel(haxes(2),'Error');
  % Add the fourth curve (plotted against the log axis).
  axes(haxes(2));
  hold on;
  hline3 = semilogy(t, errest);
  % Set some features of the three lines.
  set(hline1, 'Linewidth', 0.5, 'MarkerFaceColor', 'auto');
  set(hline2, 'Linewidth', 0.5, 'Marker', 'x', 'Color','Magenta');
  set(hline3, 'Linewidth', 0.5, 'Marker', '*', 'MarkerFaceColor', 'auto');
  % Add legend.
  legend([hpoints hline1 hline2 hline3], 'result', 'exp(-t/2)', ...
    'relative error', 'estimated error', 'Location','SouthWest');
