function [infos, config, rbutton_state] = load_Domains(filename)
   
    doc = xmlread(filename);
    
    domainsList = struct ('boundaries', struct('coor',{},'edge',{},'grad',{},'reverse',{},'name',{},'type',{}), 'boundary', struct('coor',{},'edge',{}), 'mesh' , struct('coor', {}, 'conn', {} ), 'activeBoundary', {1}, 'showMesh',{false}, 'changed',{false}, 'name', {} );
    constraintsList = struct('name',{},'expression',{},'type',{},'tag',{});
    
    
    conditions = doc.getElementsByTagName('Condition');
    for k=0:conditions.getLength-1
       curCondition = conditions.item(k);
       id = str2double( curCondition.getAttribute('Id') );
       constraintsList(id).name = char(curCondition.getAttribute('Name'));
       constraintsList(id).type = str2double(curCondition.getAttribute('Type'));
       constraintsList(id).expression = char(curCondition.getAttribute('Expression'));
       constraintsList(id).tag = char(curCondition.getAttribute('Tag'));
       
        
        
    end    
        
    domains = doc.getElementsByTagName('Domain');
    for k=0:domains.getLength-1
        thisDomain = domains.item(k);
        

        
        domainsList(k+1).name = char(thisDomain.getAttribute('Name'));
        
        boundaries = thisDomain.getElementsByTagName('Boundary');
        for i=0:boundaries.getLength-1
           thisBoundary = boundaries.item(i);

           domainsList(k+1).boundaries(i+1).name = char(thisBoundary.getAttribute('Name'));
           domainsList(k+1).boundaries(i+1).reverse = str2double(thisBoundary.getAttribute('Reverse'));


           points = thisBoundary.getElementsByTagName('Point');
           for j=0:points.getLength-1;
              point = points.item(j);
              x = str2double(point.getAttribute('x'));
              y = str2double(point.getAttribute('y'));
              domainsList(k+1).boundaries(i+1).coor(:,j+1) = [x;y];
           end    

           nedge = [];

           edges = thisBoundary.getElementsByTagName('Edge');
           for j=0:edges.getLength-1;
              edge = edges.item(j);
              from = str2double(edge.getAttribute('From'));
              to = str2double(edge.getAttribute('To'));
              cid = str2double(edge.getAttribute('ConditionId'));
              
              nedge(:,j+1) = [from;to;cid];

           end 
           
           domainsList(k+1).boundaries(i+1).edge = nedge;
           
           Gradient = thisBoundary.getElementsByTagName('Grad');
           for j=0:Gradient.getLength-1;
               Grad = Gradient.item(j);
               curgrad = str2num(Grad.getAttribute('dydx'));
               grad(j+1) = curgrad;
           end
           domainsList(k+1).boundaries(i+1).grad = grad;
           
           Type = thisBoundary.getElementsByTagName('Type');
           curType = Type.item(0);
           type = curType.getAttribute('type');
           domainsList(k+1).boundaries(i+1).type = type;
           
        end      
        
        
       genBoundary  = thisDomain.getElementsByTagName('GeneratedBoundary');
       if genBoundary.getLength ~= 0
            genBoundary = genBoundary.item(0);
       

           points = genBoundary.getElementsByTagName('Point');
           for j=0:points.getLength-1;
              point = points.item(j);
              x = str2double(point.getAttribute('x'));
              y = str2double(point.getAttribute('y'));
              domainsList(k+1).boundary.coor(:,j+1) = [x;y];
           end    

           nedge = [];

           edges = genBoundary.getElementsByTagName('Edge');
           for j=0:edges.getLength-1;
              edge = edges.item(j);
              from = str2double(edge.getAttribute('From'));
              to = str2double(edge.getAttribute('To'));
              eid = str2double(edge.getAttribute('IdMainEdge'));
              nedge(:,j+1) = [from;to;eid];

           end 

           domainsList(k+1).boundary.edge = nedge;
       end
        
       mesh = thisDomain.getElementsByTagName('Mesh');
        
        domainsList(k+1).changed = false;
        
        if mesh.getLength ~= 0
            mesh = mesh.item(0);
            points = mesh.getElementsByTagName('Point');
            
            cc = [];
            
            for i=0:points.getLength-1
              point = points.item(i);
              
              x = str2double(point.getAttribute('x'));
              y = str2double(point.getAttribute('y'));
              cc = [cc,[x;y]] ;
                
            end

            
            elements = mesh.getElementsByTagName('Element');
            
            e = [];
            for i=0:elements.getLength-1
              elt = elements.item(i);
              
              a = str2double(elt.getAttribute('a'));
              b = str2double(elt.getAttribute('b'));
              c = str2double(elt.getAttribute('c'));
              
              e = [e,[a;b;c]] ;
                
            end  

            
            domainsList(k+1).mesh = struct('coor',cc, 'conn', e );
            domainsList(k+1).changed = false;
        else
            domainsList(k+1).changed = true;
        end

        domainsList(k+1).showMesh = true;
        domainsList(k+1).activeBoundary = size( domainsList(k+1).boundaries,2);    
        
    end

    
    infos = struct('domains',{domainsList},'constraints',{constraintsList} );
    
    
    axes = doc.getElementsByTagName('Axes').item(0);
    grid = doc.getElementsByTagName('Grid').item(0);
    solution = doc.getElementsByTagName('SolutionDisplay').item(0);
    
    algos = doc.getElementsByTagName('Algorithms').item(0);
    
    incremental = doc.getElementsByTagName('Incremental').item(0);
    delaunay = doc.getElementsByTagName('Delaunay').item(0);
    front = doc.getElementsByTagName('Front').item(0);
    
    config.axes = [ str2double(axes.getAttribute('Xmin')), str2double(axes.getAttribute('Xmax')), str2double(axes.getAttribute('Ymin')), str2double(axes.getAttribute('Ymax')) ];
    config.grid = [ str2double(grid.getAttribute('enabled')), str2double(grid.getAttribute('X')), str2double(grid.getAttribute('Y')) ];
    config.solution = [str2double(solution.getAttribute('showMeshes')),str2double(solution.getAttribute('showGrid')),str2double(solution.getAttribute('smooth'))];
    
    
    
    config.algorithms.default = str2double(algos.getAttribute('default'));
    config.algorithms.incremental = struct ('bspace', {char(incremental.getAttribute('bspace'))}, 'smooth', {str2double(incremental.getAttribute('smooth'))},'coeff', {str2double(incremental.getAttribute('coeff'))},'power', {str2double(incremental.getAttribute('power'))},'boundary', {str2double(incremental.getAttribute('boundary'))});
    config.algorithms.delaunay = struct( 'weight', {char(delaunay.getAttribute('weight'))},'npropa', {str2double(delaunay.getAttribute('npropa'))}, 'boundary', {str2double(delaunay.getAttribute('boundary'))});
    config.algorithms.front = struct( 'weight', {char(front.getAttribute('weight'))}, 'boundary', {str2double(front.getAttribute('boundary'))});
    
    % added in to disenable advancing front radio button if square or
    % circle inner boundary exists
    rbutton_state = str2double(front.getAttribute('rbutton_state'));
    % extra boundary info NF 08/10/08


end
