function save_Domains(domainsList, conditionsList, configArray, filename, rbutton_state)

    docNode = com.mathworks.xml.XMLUtils.createDocument('Simulation');
    
    docRootNode = docNode.getDocumentElement;
    docRootNode.setAttribute('Format','1.0');
    
    conditions = docNode.createElement('Conditions');
    for k=1:size(conditionsList,2)
        
        curCondition = docNode.createElement('Condition');
        curCondition.setAttribute('Id',num2str(k) );
        curCondition.setAttribute('Name', conditionsList(k).name );
        curCondition.setAttribute('Expression', conditionsList(k).expression );
        curCondition.setAttribute('Type', num2str(conditionsList(k).type) );  
        curCondition.setAttribute('Tag', num2str(conditionsList(k).tag))
        conditions.appendChild(curCondition);
    end    
    
    docRootNode.appendChild(conditions);
    
    
    domains = docNode.createElement('Domains');
    
    for k=1:size(domainsList,2)
        
        d = domainsList(k);
        
                
        curDomain = docNode.createElement('Domain');
        curDomain.setAttribute('Name', d.name );
        
        boundaries = docNode.createElement('Boundaries');
        
        for i=1:size(d.boundaries,2)
            curBoundary = docNode.createElement('Boundary'); 
            curBoundary.setAttribute('Name',d.boundaries(i).name);
            curBoundary.setAttribute('Reverse',num2str(d.boundaries(i).reverse));
            
            points = docNode.createElement('Points');
            coor = d.boundaries(i).coor;

            for j=1:size(coor,2)
               p = docNode.createElement('Point');
               p.setAttribute('x', num2str(coor(1,j))  );
               p.setAttribute('y', num2str(coor(2,j)) );
               points.appendChild(p);
            end    
            curBoundary.appendChild(points);

            edges = docNode.createElement('Edges');
            edge = d.boundaries(i).edge;

            for j=1:size(edge,2)
               p = docNode.createElement('Edge');
               p.setAttribute('From', num2str(edge(1,j))  );
               p.setAttribute('To', num2str(edge(2,j)) );
               p.setAttribute('ConditionId', num2str(edge(3,j)) );

               
               edges.appendChild(p);
            end    
            curBoundary.appendChild(edges);
            
            % saves the grad of the defining boundary edges
            Grad = docNode.createElement('Gradients');
            for s = 1:size(d.boundaries(i).grad,2)
                grad = docNode.createElement('Grad');
                grad.setAttribute('dydx', num2str(d.boundaries(i).grad(s)));
                Grad.appendChild(grad);
            end
            curBoundary.appendChild(Grad);
            
            type = docNode.createElement('Type');
            type.setAttribute('type',d.boundaries(i).type);
            curBoundary.appendChild(type);
            
            boundaries.appendChild(curBoundary);
        end
        
        curDomain.appendChild(boundaries);

        genBoundary = docNode.createElement('GeneratedBoundary');
            
        points = docNode.createElement('Points');
        coor = d.boundary.coor;

        for j=1:size(coor,2)
           p = docNode.createElement('Point');
           p.setAttribute('x', num2str(coor(1,j))  );
           p.setAttribute('y', num2str(coor(2,j)) );
           points.appendChild(p);
        end    
        genBoundary.appendChild(points);

        edges = docNode.createElement('Edges');
        edge = d.boundary.edge;

        for j=1:size(edge,2)
           p = docNode.createElement('Edge');
           p.setAttribute('From', num2str(edge(1,j))  );
           p.setAttribute('To', num2str(edge(2,j)) );
           p.setAttribute('IdMainEdge', num2str(edge(3,j)) );
           p.setAttribute('Id', num2str(j) ); 
           
           edges.appendChild(p);
        end    
        genBoundary.appendChild(edges);   
        
        curDomain.appendChild(genBoundary);
        
        
        mesh = docNode.createElement('Mesh');

        coor = docNode.createElement('Points');

        for i=1:size(d.mesh.coor,2)
               p = docNode.createElement('Point');
               p.setAttribute('x', num2str(d.mesh.coor(1,i)) );
               p.setAttribute('y', num2str(d.mesh.coor(2,i)) );
               coor.appendChild(p);            
        end

        mesh.appendChild(coor);

        conn = docNode.createElement('Elements');

        for i=1:size(d.mesh.conn,2)
               p = docNode.createElement('Element');
               p.setAttribute('a', num2str(d.mesh.conn(1,i)) );
               p.setAttribute('b', num2str(d.mesh.conn(2,i)) );
               p.setAttribute('c', num2str(d.mesh.conn(3,i)) );
               conn.appendChild(p);            
        end

        mesh.appendChild(conn);       

        
        
        
        
        

        curDomain.appendChild(mesh);
        

        
        
        
         
        domains.appendChild(curDomain);
        
        
    end
    docRootNode.appendChild(domains);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%% added in save config when saving simulation %%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    axes = docNode.createElement('Axes');
    axes.setAttribute('Xmin', num2str(configArray.axes(1)) );
    axes.setAttribute('Xmax', num2str(configArray.axes(2)) );
    axes.setAttribute('Ymin', num2str(configArray.axes(3)) );
    axes.setAttribute('Ymax', num2str(configArray.axes(4)) );
    docRootNode.appendChild(axes);

    grid = docNode.createElement('Grid');
    grid.setAttribute('enabled', num2str(configArray.grid(1)) );
    grid.setAttribute('X', num2str(configArray.grid(2)) );
    grid.setAttribute('Y', num2str(configArray.grid(3)) );
    docRootNode.appendChild(grid);    
    
    solution = docNode.createElement('SolutionDisplay');
    solution.setAttribute('showMeshes', num2str(configArray.solution(1)));
    solution.setAttribute('showGrid', num2str(configArray.solution(2)));
%     solution.setAttribute('smooth', num2str(configArray.solution(3)));
    docRootNode.appendChild(solution);
    
    
    
    algos = docNode.createElement('Algorithms');
    algos.setAttribute('default', num2str(configArray.algorithms.default));
    
    
    incremental = docNode.createElement('Incremental');
    incremental.setAttribute('bspace', configArray.algorithms.incremental.bspace );
    incremental.setAttribute('smooth', num2str(configArray.algorithms.incremental.smooth) );
    incremental.setAttribute('coeff', num2str(configArray.algorithms.incremental.coeff) );
    incremental.setAttribute('power', num2str(configArray.algorithms.incremental.power) );
    incremental.setAttribute('boundary', num2str(configArray.algorithms.incremental.boundary) ); % NF 08/10/08
    algos.appendChild(incremental);
    
    delaunay = docNode.createElement('Delaunay');
    delaunay.setAttribute('weight', configArray.algorithms.delaunay.weight );
    delaunay.setAttribute('npropa', num2str(configArray.algorithms.delaunay.npropa) );
    delaunay.setAttribute('boundary', num2str(configArray.algorithms.delaunay.boundary) ); % NF 08/10/08
    
    algos.appendChild(delaunay);    
    
    front = docNode.createElement('Front');
    front.setAttribute('weight', configArray.algorithms.front.weight );
    front.setAttribute('boundary', num2str(configArray.algorithms.front.boundary) ); % NF 08/10/08
    front.setAttribute('rbutton_state',num2str(rbutton_state))
    algos.appendChild(front);        
    docRootNode.appendChild(algos);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    xmlFileName = filename;
    xmlwrite(xmlFileName,docNode);

end
